@@ -5,26 +5,20 @@
  *
  *    Kern Sibbald, March 2000
  *
- *    Version $Id: mysql.c,v 1.34 2004/10/07 12:11:55 kerns Exp $
+ *    Version $Id: mysql.c,v 1.37.2.3 2006/04/11 08:05:18 kerns Exp $
  */
-
 /*
-   Copyright (C) 2000-2004 Kern Sibbald and John Walker
+   Copyright (C) 2000-2006 Kern Sibbald
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of
-   the License, or (at your option) any later version.
+   modify it under the terms of the GNU General Public License
+   version 2 as amended with additional clauses defined in the
+   file LICENSE in the main source directory.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public
-   License along with this program; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+   the file LICENSE for additional details.
 
  */
 
@@ -32,7 +26,7 @@
 /* The following is necessary so that we do not include
  * the dummy external definition of DB.
  */
-#define __SQL_C 		      /* indicate that this is sql.c */
+#define __SQL_C                       /* indicate that this is sql.c */
 
 #include "bacula.h"
 #include "cats.h"
@@ -56,26 +50,26 @@ static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
  * never have errors, or it is really fatal.
  */
 B_DB *
-db_init_database(JCR *jcr, const char *db_name, const char *db_user, const char *db_password, 
-		 const char *db_address, int db_port, const char *db_socket,
-		 int mult_db_connections)
+db_init_database(JCR *jcr, const char *db_name, const char *db_user, const char *db_password,
+                 const char *db_address, int db_port, const char *db_socket,
+                 int mult_db_connections)
 {
    B_DB *mdb;
 
-   if (!db_user) {		
+   if (!db_user) {
       Jmsg(jcr, M_FATAL, 0, _("A user name for MySQL must be supplied.\n"));
       return NULL;
    }
-   P(mutex);			      /* lock DB queue */
+   P(mutex);                          /* lock DB queue */
    /* Look to see if DB already open */
    if (!mult_db_connections) {
       for (mdb=NULL; (mdb=(B_DB *)qnext(&db_list, &mdb->bq)); ) {
-	 if (strcmp(mdb->db_name, db_name) == 0) {
+         if (strcmp(mdb->db_name, db_name) == 0) {
             Dmsg2(100, "DB REopen %d %s\n", mdb->ref_count, db_name);
-	    mdb->ref_count++;
-	    V(mutex);
-	    return mdb; 		 /* already open */
-	 }
+            mdb->ref_count++;
+            V(mutex);
+            return mdb;                  /* already open */
+         }
       }
    }
    Dmsg0(100, "db_open first time\n");
@@ -103,7 +97,7 @@ db_init_database(JCR *jcr, const char *db_name, const char *db_user, const char
    mdb->fname = get_pool_memory(PM_FNAME);
    mdb->path = get_pool_memory(PM_FNAME);
    mdb->esc_name = get_pool_memory(PM_FNAME);
-   qinsert(&db_list, &mdb->bq); 	   /* put db in list */
+   qinsert(&db_list, &mdb->bq);            /* put db in list */
    V(mutex);
    return mdb;
 }
@@ -127,8 +121,8 @@ db_open_database(JCR *jcr, B_DB *mdb)
    mdb->connected = FALSE;
 
    if ((errstat=rwl_init(&mdb->lock)) != 0) {
-      Mmsg1(&mdb->errmsg, _("Unable to initialize DB lock. ERR=%s\n"), 
-	    strerror(errstat));
+      Mmsg1(&mdb->errmsg, _("Unable to initialize DB lock. ERR=%s\n"),
+            strerror(errstat));
       V(mutex);
       return 0;
    }
@@ -142,31 +136,32 @@ db_open_database(JCR *jcr, B_DB *mdb)
    /* If connection fails, try at 5 sec intervals for 30 seconds. */
    for (int retry=0; retry < 6; retry++) {
       mdb->db = mysql_real_connect(
-	   &(mdb->mysql),		 /* db */
-	   mdb->db_address,		 /* default = localhost */
-	   mdb->db_user,		 /*  login name */
-	   mdb->db_password,		 /*  password */
-	   mdb->db_name,		 /* database name */
-	   mdb->db_port,		 /* default port */
-	   mdb->db_socket,		 /* default = socket */
-	   CLIENT_FOUND_ROWS);		 /* flags */
+           &(mdb->mysql),                /* db */
+           mdb->db_address,              /* default = localhost */
+           mdb->db_user,                 /*  login name */
+           mdb->db_password,             /*  password */
+           mdb->db_name,                 /* database name */
+           mdb->db_port,                 /* default port */
+           mdb->db_socket,               /* default = socket */
+           CLIENT_FOUND_ROWS);           /* flags */
 
       /* If no connect, try once more in case it is a timing problem */
       if (mdb->db != NULL) {
-	 break;
+         break;
       }
       bmicrosleep(5,0);
    }
-    
+
+   mdb->mysql.reconnect = 1;             /* so connection does not timeout */
    Dmsg0(50, "mysql_real_connect done\n");
-   Dmsg3(50, "db_user=%s db_name=%s db_password=%s\n", mdb->db_user, mdb->db_name, 
+   Dmsg3(50, "db_user=%s db_name=%s db_password=%s\n", mdb->db_user, mdb->db_name,
             mdb->db_password==NULL?"(NULL)":mdb->db_password);
-  
+
    if (mdb->db == NULL) {
-      Mmsg2(&mdb->errmsg, _("Unable to connect to MySQL server. \n\
-Database=%s User=%s\n\
-It is probably not running or your password is incorrect.\n"), 
-	 mdb->db_name, mdb->db_user);
+      Mmsg2(&mdb->errmsg, _("Unable to connect to MySQL server. \n"
+"Database=%s User=%s\n"
+"It is probably not running or your password is incorrect.\n"),
+         mdb->db_name, mdb->db_user);
       V(mutex);
       return 0;
    }
@@ -191,6 +186,7 @@ db_close_database(JCR *jcr, B_DB *mdb)
    if (!mdb) {
       return;
    }
+   db_end_transaction(jcr, mdb);
    P(mutex);
    mdb->ref_count--;
 #ifdef HAVE_TREAD_SAFE_MYSQL
@@ -199,12 +195,12 @@ db_close_database(JCR *jcr, B_DB *mdb)
    if (mdb->ref_count == 0) {
       qdchain(&mdb->bq);
       if (mdb->connected && mdb->db) {
-	 sql_close(mdb);
+         sql_close(mdb);
 #ifdef HAVE_EMBEDDED_MYSQL
-	 mysql_server_end();
+         mysql_server_end();
 #endif
       }
-      rwl_destroy(&mdb->lock);	     
+      rwl_destroy(&mdb->lock);
       free_pool_memory(mdb->errmsg);
       free_pool_memory(mdb->cmd);
       free_pool_memory(mdb->cached_path);
@@ -212,19 +208,19 @@ db_close_database(JCR *jcr, B_DB *mdb)
       free_pool_memory(mdb->path);
       free_pool_memory(mdb->esc_name);
       if (mdb->db_name) {
-	 free(mdb->db_name);
+         free(mdb->db_name);
       }
       if (mdb->db_user) {
-	 free(mdb->db_user);
+         free(mdb->db_user);
       }
       if (mdb->db_password) {
-	 free(mdb->db_password);
+         free(mdb->db_password);
       }
       if (mdb->db_address) {
-	 free(mdb->db_address);
+         free(mdb->db_address);
       }
       if (mdb->db_socket) {
-	 free(mdb->db_socket);
+         free(mdb->db_socket);
       }
       free(mdb);
    }
@@ -234,7 +230,7 @@ db_close_database(JCR *jcr, B_DB *mdb)
 /*
  * Return the next unique index (auto-increment) for
  * the given table.  Return NULL on error.
- *  
+ *
  * For MySQL, NULL causes the auto-increment value
  *  to be updated.
  */
@@ -242,15 +238,15 @@ int db_next_index(JCR *jcr, B_DB *mdb, char *table, char *index)
 {
    strcpy(index, "NULL");
    return 1;
-}   
+}
 
 
 /*
  * Escape strings so that MySQL is happy
  *
  *   NOTE! len is the length of the old string. Your new
- *	   string must be long enough (max 2*old+1) to hold
- *	   the escaped output.
+ *         string must be long enough (max 2*old+1) to hold
+ *         the escaped output.
  */
 void
 db_escape_string(char *snew, char *old, int len)
@@ -271,40 +267,40 @@ unsigned long mysql_real_escape_string(MYSQL *mysql, char *to, const char *from,
       case 0:
          *n++= '\\';
          *n++= '0';
-	 o++;
-	 break;
+         o++;
+         break;
       case '\n':
          *n++= '\\';
          *n++= 'n';
-	 o++;
-	 break;
+         o++;
+         break;
       case '\r':
          *n++= '\\';
          *n++= 'r';
-	 o++;
-	 break;
+         o++;
+         break;
       case '\\':
          *n++= '\\';
          *n++= '\\';
-	 o++;
-	 break;
+         o++;
+         break;
       case '\'':
          *n++= '\\';
          *n++= '\'';
-	 o++;
-	 break;
+         o++;
+         break;
       case '"':
          *n++= '\\';
          *n++= '"';
-	 o++;
-	 break;
+         o++;
+         break;
       case '\032':
          *n++= '\\';
          *n++= 'Z';
-	 o++;
-	 break;
+         o++;
+         break;
       default:
-	 *n++= *o++;
+         *n++= *o++;
       }
    }
    *n = 0;
@@ -318,7 +314,8 @@ unsigned long mysql_real_escape_string(MYSQL *mysql, char *to, const char *from,
 int db_sql_query(B_DB *mdb, const char *query, DB_RESULT_HANDLER *result_handler, void *ctx)
 {
    SQL_ROW row;
-  
+   bool send = true;
+
    db_lock(mdb);
    if (sql_query(mdb, query) != 0) {
       Mmsg(mdb->errmsg, _("Query failed: %s: ERR=%s\n"), query, sql_strerror(mdb));
@@ -326,15 +323,24 @@ int db_sql_query(B_DB *mdb, const char *query, DB_RESULT_HANDLER *result_handler
       return 0;
    }
    if (result_handler != NULL) {
-      if ((mdb->result = sql_store_result(mdb)) != NULL) {
-	 int num_fields = sql_num_fields(mdb);
+      if ((mdb->result = sql_use_result(mdb)) != NULL) {
+         int num_fields = 0;                     
 
-	 while ((row = sql_fetch_row(mdb)) != NULL) {
-	    if (result_handler(ctx, num_fields, row))
-	       break;
-	 }
+         /* We *must* fetch all rows */
+         while ((row = sql_fetch_row(mdb)) != NULL) {
+            if (send) {
+               /* the result handler returns 1 when it has
+                *  seen all the data it wants.  However, we
+                *  loop to the end of the data.
+                */
+               num_fields++;
+               if (result_handler(ctx, num_fields, row)) {
+                  send = false;
+               }
+            }
+         }
 
-	 sql_free_result(mdb);
+         sql_free_result(mdb);
       }
    }
    db_unlock(mdb);
