@@ -2,44 +2,39 @@
  * Bacula Catalog Database Find record interface routines
  *
  *  Note, generally, these routines are more complicated
- *	  that a simple search by name or id. Such simple
- *	  request are in get.c
- * 
+ *        that a simple search by name or id. Such simple
+ *        request are in get.c
+ *
  *    Kern Sibbald, December 2000
  *
- *    Version $Id: sql_find.c,v 1.48 2004/10/07 16:23:25 kerns Exp $
+ *    Version $Id: sql_find.c,v 1.54.2.3 2006/03/14 21:41:39 kerns Exp $
  */
-
 /*
-   Copyright (C) 2000-2004 Kern Sibbald and John Walker
+   Copyright (C) 2000-2005 Kern Sibbald
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of
-   the License, or (at your option) any later version.
+   modify it under the terms of the GNU General Public License
+   version 2 as amended with additional clauses defined in the
+   file LICENSE in the main source directory.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public
-   License along with this program; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+   the file LICENSE for additional details.
 
  */
 
 
+
 /* The following is necessary so that we do not include
  * the dummy external definition of DB.
  */
-#define __SQL_C 		      /* indicate that this is sql.c */
+#define __SQL_C                       /* indicate that this is sql.c */
 
 #include "bacula.h"
 #include "cats.h"
 
-#if    HAVE_MYSQL || HAVE_SQLITE || HAVE_POSTGRESQL
+#if    HAVE_SQLITE3 || HAVE_MYSQL || HAVE_SQLITE || HAVE_POSTGRESQL
 
 /* -----------------------------------------------------------------------
  *
@@ -57,14 +52,15 @@ extern int QueryDB(const char *file, int line, JCR *jcr, B_DB *db, char *select_
  * find last full save for Incremental and Differential saves.
  *
  *  StartTime is returned in stime
- *	
+ *
  * Returns: 0 on failure
- *	    1 on success, jr is unchanged, but stime is set
+ *          1 on success, jr is unchanged, but stime is set
  */
-int
+bool
 db_find_job_start_time(JCR *jcr, B_DB *mdb, JOB_DBR *jr, POOLMEM **stime)
 {
    SQL_ROW row;
+   char ed1[50], ed2[50];
 
    db_lock(mdb);
 
@@ -72,67 +68,64 @@ db_find_job_start_time(JCR *jcr, B_DB *mdb, JOB_DBR *jr, POOLMEM **stime)
    /* If no Id given, we must find corresponding job */
    if (jr->JobId == 0) {
       /* Differential is since last Full backup */
-	 Mmsg(mdb->cmd, 
+         Mmsg(mdb->cmd,
 "SELECT StartTime FROM Job WHERE JobStatus='T' AND Type='%c' AND "
-"Level='%c' AND Name='%s' AND ClientId=%u AND FileSetId=%u "
+"Level='%c' AND Name='%s' AND ClientId=%s AND FileSetId=%s "
 "ORDER BY StartTime DESC LIMIT 1",
-	   jr->JobType, L_FULL, jr->Name, jr->ClientId, jr->FileSetId);
+           jr->JobType, L_FULL, jr->Name, 
+           edit_int64(jr->ClientId, ed1), edit_int64(jr->FileSetId, ed2));
 
       if (jr->JobLevel == L_DIFFERENTIAL) {
-	 /* SQL cmd for Differential backup already edited above */
+         /* SQL cmd for Differential backup already edited above */
 
       /* Incremental is since last Full, Incremental, or Differential */
       } else if (jr->JobLevel == L_INCREMENTAL) {
-	 /* 
-	  * For an Incremental job, we must first ensure
-	  *  that a Full backup was done (cmd edited above)
-	  *  then we do a second look to find the most recent
-	  *  backup
-	  */
-	 if (!QUERY_DB(jcr, mdb, mdb->cmd)) {
-            Mmsg2(&mdb->errmsg, _("Query error for start time request: ERR=%s\nCMD=%s\n"), 
-	       sql_strerror(mdb), mdb->cmd);
-	    db_unlock(mdb);
-	    return 0;
-	 }
-	 if ((row = sql_fetch_row(mdb)) == NULL) {
-	    sql_free_result(mdb);
+         /*
+          * For an Incremental job, we must first ensure
+          *  that a Full backup was done (cmd edited above)
+          *  then we do a second look to find the most recent
+          *  backup
+          */
+         if (!QUERY_DB(jcr, mdb, mdb->cmd)) {
+            Mmsg2(&mdb->errmsg, _("Query error for start time request: ERR=%s\nCMD=%s\n"),
+               sql_strerror(mdb), mdb->cmd);
+            goto bail_out;
+         }
+         if ((row = sql_fetch_row(mdb)) == NULL) {
+            sql_free_result(mdb);
             Mmsg(mdb->errmsg, _("No prior Full backup Job record found.\n"));
-	    db_unlock(mdb);
-	    return 0;
-	 }
-	 sql_free_result(mdb);
-	 /* Now edit SQL command for Incremental Job */
-	 Mmsg(mdb->cmd, 
+            goto bail_out;
+         }
+         sql_free_result(mdb);
+         /* Now edit SQL command for Incremental Job */
+         Mmsg(mdb->cmd,
 "SELECT StartTime FROM Job WHERE JobStatus='T' AND Type='%c' AND "
-"Level IN ('%c','%c','%c') AND Name='%s' AND ClientId=%u "
-"AND FileSetId=%u ORDER BY StartTime DESC LIMIT 1",
-	    jr->JobType, L_INCREMENTAL, L_DIFFERENTIAL, L_FULL, jr->Name,
-	    jr->ClientId, jr->FileSetId);
+"Level IN ('%c','%c','%c') AND Name='%s' AND ClientId=%s "
+"AND FileSetId=%s ORDER BY StartTime DESC LIMIT 1",
+            jr->JobType, L_INCREMENTAL, L_DIFFERENTIAL, L_FULL, jr->Name,
+            edit_int64(jr->ClientId, ed1), edit_int64(jr->FileSetId, ed2));
       } else {
-         Mmsg1(&mdb->errmsg, _("Unknown level=%d\n"), jr->JobLevel);
-	 db_unlock(mdb);
-	 return 0;
+         Mmsg1(mdb->errmsg, _("Unknown level=%d\n"), jr->JobLevel);
+         goto bail_out;
       }
    } else {
       Dmsg1(100, "Submitting: %s\n", mdb->cmd);
-      Mmsg(mdb->cmd, "SELECT StartTime FROM Job WHERE Job.JobId=%u", jr->JobId);
+      Mmsg(mdb->cmd, "SELECT StartTime FROM Job WHERE Job.JobId=%s", 
+           edit_int64(jr->JobId, ed1));
    }
 
    if (!QUERY_DB(jcr, mdb, mdb->cmd)) {
       pm_strcpy(stime, "");                   /* set EOS */
       Mmsg2(&mdb->errmsg, _("Query error for start time request: ERR=%s\nCMD=%s\n"),
-	 sql_strerror(mdb),  mdb->cmd);
-      db_unlock(mdb);
-      return 0;
+         sql_strerror(mdb),  mdb->cmd);
+      goto bail_out;
    }
 
    if ((row = sql_fetch_row(mdb)) == NULL) {
       Mmsg2(&mdb->errmsg, _("No Job record found: ERR=%s\nCMD=%s\n"),
-	 sql_strerror(mdb),  mdb->cmd);
+         sql_strerror(mdb),  mdb->cmd);
       sql_free_result(mdb);
-      db_unlock(mdb);
-      return 0;
+      goto bail_out;
    }
    Dmsg1(100, "Got start time: %s\n", row[0]);
    pm_strcpy(stime, row[0]);
@@ -140,31 +133,37 @@ db_find_job_start_time(JCR *jcr, B_DB *mdb, JOB_DBR *jr, POOLMEM **stime)
    sql_free_result(mdb);
 
    db_unlock(mdb);
-   return 1;
+   return true;
+
+bail_out:
+   db_unlock(mdb);
+   return false;
 }
 
 /*
- * Find last failed job since given start-time 
+ * Find last failed job since given start-time
  *   it must be either Full or Diff.
  *
  * Returns: false on failure
- *	    true  on success, jr is unchanged and stime unchanged
- *		  level returned in JobLevel
+ *          true  on success, jr is unchanged and stime unchanged
+ *                level returned in JobLevel
  */
 bool
 db_find_failed_job_since(JCR *jcr, B_DB *mdb, JOB_DBR *jr, POOLMEM *stime, int &JobLevel)
 {
    SQL_ROW row;
+   char ed1[50], ed2[50];
 
    db_lock(mdb);
    /* Differential is since last Full backup */
-   Mmsg(mdb->cmd, 
+   Mmsg(mdb->cmd,
 "SELECT Level FROM Job WHERE JobStatus!='T' AND Type='%c' AND "
-"Level IN ('%c','%c') AND Name='%s' AND ClientId=%u "
-"AND FileSetId=%u AND StartTime>'%s' "
+"Level IN ('%c','%c') AND Name='%s' AND ClientId=%s "
+"AND FileSetId=%s AND StartTime>'%s' "
 "ORDER BY StartTime DESC LIMIT 1",
-	 jr->JobType, L_FULL, L_DIFFERENTIAL, jr->Name,
-	 jr->ClientId, jr->FileSetId, stime);
+         jr->JobType, L_FULL, L_DIFFERENTIAL, jr->Name,
+         edit_int64(jr->ClientId, ed1), edit_int64(jr->FileSetId, ed2),
+         stime);
 
    if (!QUERY_DB(jcr, mdb, mdb->cmd)) {
       db_unlock(mdb);
@@ -184,116 +183,129 @@ db_find_failed_job_since(JCR *jcr, B_DB *mdb, JOB_DBR *jr, POOLMEM *stime, int &
 }
 
 
-/* 
+/*
  * Find JobId of last job that ran.  E.g. for
  *   VERIFY_CATALOG we want the JobId of the last INIT.
  *   For VERIFY_VOLUME_TO_CATALOG, we want the JobId of the last Job.
  *
- * Returns: 1 on success
- *	    0 on failure
+ * Returns: true  on success
+ *          false on failure
  */
-int
+bool
 db_find_last_jobid(JCR *jcr, B_DB *mdb, const char *Name, JOB_DBR *jr)
 {
    SQL_ROW row;
+   char ed1[50];
 
    /* Find last full */
    db_lock(mdb);
+   Dmsg2(100, "JobLevel=%d JobType=%d\n", jr->JobLevel, jr->JobType);
    if (jr->JobLevel == L_VERIFY_CATALOG) {
-      Mmsg(mdb->cmd, 
+      Mmsg(mdb->cmd,
 "SELECT JobId FROM Job WHERE Type='V' AND Level='%c' AND "
 " JobStatus='T' AND Name='%s' AND "
-"ClientId=%u ORDER BY StartTime DESC LIMIT 1",
-	   L_VERIFY_INIT, jr->Name, jr->ClientId);
+"ClientId=%s ORDER BY StartTime DESC LIMIT 1",
+           L_VERIFY_INIT, jr->Name, 
+           edit_int64(jr->ClientId, ed1));
    } else if (jr->JobLevel == L_VERIFY_VOLUME_TO_CATALOG ||
-	      jr->JobLevel == L_VERIFY_DISK_TO_CATALOG) {
+              jr->JobLevel == L_VERIFY_DISK_TO_CATALOG ||
+              jr->JobType == JT_BACKUP) {
       if (Name) {
-	 Mmsg(mdb->cmd,
+         Mmsg(mdb->cmd,
 "SELECT JobId FROM Job WHERE Type='B' AND JobStatus='T' AND "
 "Name='%s' ORDER BY StartTime DESC LIMIT 1", Name);
       } else {
-	 Mmsg(mdb->cmd, 
+         Mmsg(mdb->cmd,
 "SELECT JobId FROM Job WHERE Type='B' AND JobStatus='T' AND "
-"ClientId=%u ORDER BY StartTime DESC LIMIT 1", jr->ClientId);
+"ClientId=%s ORDER BY StartTime DESC LIMIT 1", 
+           edit_int64(jr->ClientId, ed1));
       }
    } else {
-      Mmsg1(&mdb->errmsg, _("Unknown Job level=%c\n"), jr->JobLevel);
+      Mmsg1(&mdb->errmsg, _("Unknown Job level=%d\n"), jr->JobLevel);
       db_unlock(mdb);
-      return 0;
+      return false;
    }
    Dmsg1(100, "Query: %s\n", mdb->cmd);
    if (!QUERY_DB(jcr, mdb, mdb->cmd)) {
       db_unlock(mdb);
-      return 0;
+      return false;
    }
    if ((row = sql_fetch_row(mdb)) == NULL) {
       Mmsg1(&mdb->errmsg, _("No Job found for: %s.\n"), mdb->cmd);
       sql_free_result(mdb);
       db_unlock(mdb);
-      return 0;
+      return false;
    }
 
-   jr->JobId = atoi(row[0]);
+   jr->JobId = str_to_int64(row[0]);
    sql_free_result(mdb);
 
    Dmsg1(100, "db_get_last_jobid: got JobId=%d\n", jr->JobId);
    if (jr->JobId <= 0) {
       Mmsg1(&mdb->errmsg, _("No Job found for: %s\n"), mdb->cmd);
       db_unlock(mdb);
-      return 0;
+      return false;
    }
 
    db_unlock(mdb);
-   return 1;
+   return true;
 }
 
-/* 
+/*
  * Find Available Media (Volume) for Pool
  *
  * Find a Volume for a given PoolId, MediaType, and Status.
  *
  * Returns: 0 on failure
- *	    numrows on success
+ *          numrows on success
  */
 int
-db_find_next_volume(JCR *jcr, B_DB *mdb, int item, bool InChanger, MEDIA_DBR *mr) 
+db_find_next_volume(JCR *jcr, B_DB *mdb, int item, bool InChanger, MEDIA_DBR *mr)
 {
-   SQL_ROW row;
+   SQL_ROW row = NULL;
    int numrows;
-   const char *changer, *order;
+   const char *order;
+
+   char ed1[50];
 
    db_lock(mdb);
-   if (item == -1) {	   /* find oldest volume */
+   if (item == -1) {       /* find oldest volume */
       /* Find oldest volume */
       Mmsg(mdb->cmd, "SELECT MediaId,VolumeName,VolJobs,VolFiles,VolBlocks,"
           "VolBytes,VolMounts,VolErrors,VolWrites,MaxVolBytes,VolCapacityBytes,"
           "VolRetention,VolUseDuration,MaxVolJobs,MaxVolFiles,Recycle,Slot,"
-          "FirstWritten,LastWritten,VolStatus,InChanger "
-          "FROM Media WHERE PoolId=%u AND MediaType='%s' AND VolStatus IN ('Full',"
+          "FirstWritten,LastWritten,VolStatus,InChanger,VolParts,"
+          "LabelType "
+          "FROM Media WHERE PoolId=%s AND MediaType='%s' AND VolStatus IN ('Full',"
           "'Recycle','Purged','Used','Append') "
-          "ORDER BY LastWritten LIMIT 1", mr->PoolId, mr->MediaType); 
+          "ORDER BY LastWritten LIMIT 1", 
+          edit_int64(mr->PoolId, ed1), mr->MediaType);
      item = 1;
    } else {
+      char changer[100];
       /* Find next available volume */
       if (InChanger) {
-         changer = "AND InChanger=1";
+         bsnprintf(changer, sizeof(changer), "AND InChanger=1 AND StorageId=%s",
+            edit_int64(mr->StorageId, ed1));
       } else {
-         changer = "";
+         changer[0] = 0;
       }
       if (strcmp(mr->VolStatus, "Recycled") == 0 ||
           strcmp(mr->VolStatus, "Purged") == 0) {
          order = "ORDER BY LastWritten ASC,MediaId";  /* take oldest */
       } else {
          order = "ORDER BY LastWritten IS NULL,LastWritten DESC,MediaId";   /* take most recently written */
-      }  
+      }
       Mmsg(mdb->cmd, "SELECT MediaId,VolumeName,VolJobs,VolFiles,VolBlocks,"
           "VolBytes,VolMounts,VolErrors,VolWrites,MaxVolBytes,VolCapacityBytes,"
           "VolRetention,VolUseDuration,MaxVolJobs,MaxVolFiles,Recycle,Slot,"
-          "FirstWritten,LastWritten,VolStatus,InChanger "
-          "FROM Media WHERE PoolId=%u AND MediaType='%s' AND VolStatus='%s' "
-          "%s " 
+          "FirstWritten,LastWritten,VolStatus,InChanger,VolParts,"
+          "LabelType "
+          "FROM Media WHERE PoolId=%s AND MediaType='%s' AND VolStatus='%s' "
+          "%s "
           "%s LIMIT %d",
-	  mr->PoolId, mr->MediaType, mr->VolStatus, changer, order, item);
+          edit_int64(mr->PoolId, ed1), mr->MediaType,
+          mr->VolStatus, changer, order, item);
    }
    if (!QUERY_DB(jcr, mdb, mdb->cmd)) {
       db_unlock(mdb);
@@ -301,23 +313,26 @@ db_find_next_volume(JCR *jcr, B_DB *mdb, int item, bool InChanger, MEDIA_DBR *mr
    }
 
    numrows = sql_num_rows(mdb);
-   if (item > numrows) {
-      Mmsg2(&mdb->errmsg, _("Request for Volume item %d greater than max %d\n"),
-	 item, numrows);
+   if (item > numrows || item < 1) {
+      Mmsg2(&mdb->errmsg, _("Request for Volume item %d greater than max %d or less than 1.\n"), 
+         item, numrows);
       db_unlock(mdb);
       return 0;
    }
-   
-   /* Seek to desired item 
-    * Note, we use base 1; SQL uses base 0
-    */
-   sql_data_seek(mdb, item-1);
 
-   if ((row = sql_fetch_row(mdb)) == NULL) {
-      Mmsg1(&mdb->errmsg, _("No Volume record found for item %d.\n"), item);
-      sql_free_result(mdb);
-      db_unlock(mdb);
-      return 0;
+   /* Note, we previously seeked to the row using:
+    *  sql_data_seek(mdb, item-1);
+    * but this failed on PostgreSQL, so now we loop
+    * over all the records.  This should not be too horrible since
+    * the maximum Volumes we look at in any case is 20.
+    */
+   while (item-- > 0) {
+      if ((row = sql_fetch_row(mdb)) == NULL) {
+         Mmsg1(&mdb->errmsg, _("No Volume record found for item %d.\n"), item);
+         sql_free_result(mdb);
+         db_unlock(mdb);
+         return 0;
+      }
    }
 
    /* Return fields in Media Record */
@@ -344,6 +359,8 @@ db_find_next_volume(JCR *jcr, B_DB *mdb, int item, bool InChanger, MEDIA_DBR *mr
    mr->LastWritten = (time_t)str_to_utime(mr->cLastWritten);
    bstrncpy(mr->VolStatus, row[19], sizeof(mr->VolStatus));
    mr->InChanger = str_to_int64(row[20]);
+   mr->VolParts = str_to_int64(row[21]);
+   mr->LabelType = str_to_int64(row[22]);
    sql_free_result(mdb);
 
    db_unlock(mdb);
@@ -351,4 +368,4 @@ db_find_next_volume(JCR *jcr, B_DB *mdb, int item, bool InChanger, MEDIA_DBR *mr
 }
 
 
-#endif /* HAVE_MYSQL || HAVE_SQLITE || HAVE_POSTGRESQL */
+#endif /* HAVE_SQLITE3 || HAVE_MYSQL || HAVE_SQLITE || HAVE_POSTGRESQL*/
