@@ -1,40 +1,34 @@
 /*
  * Bacula Catalog Database List records interface routines
- * 
+ *
  *    Kern Sibbald, March 2000
  *
- *    Version $Id: sql_list.c,v 1.28 2004/09/22 19:51:05 kerns Exp $
+ *    Version $Id: sql_list.c,v 1.41.2.1 2006/03/04 11:10:17 kerns Exp $
  */
-
 /*
-   Copyright (C) 2000, 2001, 2002 Kern Sibbald and John Walker
+   Copyright (C) 2000-2006 Kern Sibbald
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of
-   the License, or (at your option) any later version.
+   modify it under the terms of the GNU General Public License
+   version 2 as amended with additional clauses defined in the
+   file LICENSE in the main source directory.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public
-   License along with this program; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+   the file LICENSE for additional details.
 
  */
 
 /* The following is necessary so that we do not include
  * the dummy external definition of DB.
  */
-#define __SQL_C 		      /* indicate that this is sql.c */
+#define __SQL_C                       /* indicate that this is sql.c */
 
 #include "bacula.h"
 #include "cats.h"
 
-#if    HAVE_MYSQL || HAVE_SQLITE || HAVE_POSTGRESQL
+#if    HAVE_SQLITE3 || HAVE_MYSQL || HAVE_SQLITE || HAVE_POSTGRESQL
 
 /* -----------------------------------------------------------------------
  *
@@ -48,17 +42,17 @@ extern void list_result(JCR *jcr, B_DB *mdb, DB_LIST_HANDLER *sendit, void *ctx,
 extern int QueryDB(const char *file, int line, JCR *jcr, B_DB *db, char *select_cmd);
 
 
-/* 
+/*
  * Submit general SQL query
  */
-int db_list_sql_query(JCR *jcr, B_DB *mdb, char *query, DB_LIST_HANDLER *sendit, 
-		      void *ctx, int verbose, e_list_type type)
+int db_list_sql_query(JCR *jcr, B_DB *mdb, char *query, DB_LIST_HANDLER *sendit,
+                      void *ctx, int verbose, e_list_type type)
 {
    db_lock(mdb);
    if (sql_query(mdb, query) != 0) {
       Mmsg(mdb->errmsg, _("Query failed: %s\n"), sql_strerror(mdb));
       if (verbose) {
-	 sendit(ctx, mdb->errmsg);
+         sendit(ctx, mdb->errmsg);
       }
       db_unlock(mdb);
       return 0;
@@ -75,18 +69,32 @@ int db_list_sql_query(JCR *jcr, B_DB *mdb, char *query, DB_LIST_HANDLER *sendit,
 }
 
 void
-db_list_pool_records(JCR *jcr, B_DB *mdb, DB_LIST_HANDLER *sendit, void *ctx, e_list_type type) 
+db_list_pool_records(JCR *jcr, B_DB *mdb, POOL_DBR *pdbr, 
+                     DB_LIST_HANDLER *sendit, void *ctx, e_list_type type)
 {
    db_lock(mdb);
    if (type == VERT_LIST) {
-      Mmsg(mdb->cmd, "SELECT PoolId,Name,NumVols,MaxVols,UseOnce,UseCatalog,"
-         "AcceptAnyVolume,VolRetention,VolUseDuration,MaxVolJobs,MaxVolBytes,"
-         "AutoPrune,Recycle,PoolType,LabelFormat,Enabled,ScratchPoolId,"
-         "RecyclePoolId "
-          "FROM Pool ORDER BY PoolId");
+      if (pdbr->Name[0] != 0) {
+         Mmsg(mdb->cmd, "SELECT PoolId,Name,NumVols,MaxVols,UseOnce,UseCatalog,"
+            "AcceptAnyVolume,VolRetention,VolUseDuration,MaxVolJobs,MaxVolBytes,"
+            "AutoPrune,Recycle,PoolType,LabelFormat,Enabled,ScratchPoolId,"
+            "RecyclePoolId,LabelType "
+            " FROM Pool WHERE Name='%s'", pdbr->Name);
+      } else {
+         Mmsg(mdb->cmd, "SELECT PoolId,Name,NumVols,MaxVols,UseOnce,UseCatalog,"
+            "AcceptAnyVolume,VolRetention,VolUseDuration,MaxVolJobs,MaxVolBytes,"
+            "AutoPrune,Recycle,PoolType,LabelFormat,Enabled,ScratchPoolId,"
+            "RecyclePoolId,LabelType "
+            " FROM Pool ORDER BY PoolId");
+      }
    } else {
-      Mmsg(mdb->cmd, "SELECT PoolId,Name,NumVols,MaxVols,PoolType,LabelFormat "
-        "FROM Pool ORDER BY PoolId");
+      if (pdbr->Name[0] != 0) {
+         Mmsg(mdb->cmd, "SELECT PoolId,Name,NumVols,MaxVols,PoolType,LabelFormat "
+           "FROM Pool WHERE Name='%s'", pdbr->Name);
+      } else {
+         Mmsg(mdb->cmd, "SELECT PoolId,Name,NumVols,MaxVols,PoolType,LabelFormat "
+           "FROM Pool ORDER BY PoolId");
+      }
    }
 
    if (!QUERY_DB(jcr, mdb, mdb->cmd)) {
@@ -95,7 +103,7 @@ db_list_pool_records(JCR *jcr, B_DB *mdb, DB_LIST_HANDLER *sendit, void *ctx, e_
    }
 
    list_result(jcr, mdb, sendit, ctx, type);
-   
+
    sql_free_result(mdb);
    db_unlock(mdb);
 }
@@ -106,7 +114,7 @@ db_list_client_records(JCR *jcr, B_DB *mdb, DB_LIST_HANDLER *sendit, void *ctx,
    db_lock(mdb);
    if (type == VERT_LIST) {
       Mmsg(mdb->cmd, "SELECT ClientId,Name,Uname,AutoPrune,FileRetention,"
-         "FileRetention,JobRetention "
+         "JobRetention "
          "FROM Client ORDER BY ClientId");
    } else {
       Mmsg(mdb->cmd, "SELECT ClientId,Name,FileRetention,JobRetention "
@@ -119,7 +127,7 @@ db_list_client_records(JCR *jcr, B_DB *mdb, DB_LIST_HANDLER *sendit, void *ctx,
    }
 
    list_result(jcr, mdb, sendit, ctx, type);
-   
+
    sql_free_result(mdb);
    db_unlock(mdb);
 }
@@ -130,9 +138,10 @@ db_list_client_records(JCR *jcr, B_DB *mdb, DB_LIST_HANDLER *sendit, void *ctx,
  *   otherwise, list the Volumes in the Pool specified by PoolId
  */
 void
-db_list_media_records(JCR *jcr, B_DB *mdb, MEDIA_DBR *mdbr, 
-		      DB_LIST_HANDLER *sendit, void *ctx, e_list_type type)
+db_list_media_records(JCR *jcr, B_DB *mdb, MEDIA_DBR *mdbr,
+                      DB_LIST_HANDLER *sendit, void *ctx, e_list_type type)
 {
+   char ed1[50];
    db_lock(mdb);
    if (type == VERT_LIST) {
       if (mdbr->VolumeName[0] != 0) {
@@ -141,16 +150,17 @@ db_list_media_records(JCR *jcr, B_DB *mdb, MEDIA_DBR *mdbr,
             "VolFiles,VolBlocks,VolMounts,VolBytes,VolErrors,VolWrites,"
             "VolCapacityBytes,VolStatus,Recycle,VolRetention,"
             "VolUseDuration,MaxVolJobs,MaxVolFiles,MaxVolBytes,InChanger,"
-            "EndFile,EndBlock "
-            "FROM Media WHERE Media.VolumeName='%s'", mdbr->VolumeName);
+            "EndFile,EndBlock,VolParts,LabelType,StorageId"
+            " FROM Media WHERE Media.VolumeName='%s'", mdbr->VolumeName);
       } else {
          Mmsg(mdb->cmd, "SELECT MediaId,VolumeName,Slot,PoolId,"
             "MediaType,FirstWritten,LastWritten,LabelDate,VolJobs,"
             "VolFiles,VolBlocks,VolMounts,VolBytes,VolErrors,VolWrites,"
             "VolCapacityBytes,VolStatus,Recycle,VolRetention,"
             "VolUseDuration,MaxVolJobs,MaxVolFiles,MaxVolBytes,InChanger,"
-            "EndFile,EndBlock "
-            "FROM Media WHERE Media.PoolId=%u ORDER BY MediaId", mdbr->PoolId);
+            "EndFile,EndBlock,VolParts,LabelType,StorageId"
+            " FROM Media WHERE Media.PoolId=%s ORDER BY MediaId", 
+            edit_int64(mdbr->PoolId, ed1));
       }
    } else {
       if (mdbr->VolumeName[0] != 0) {
@@ -160,7 +170,8 @@ db_list_media_records(JCR *jcr, B_DB *mdb, MEDIA_DBR *mdbr,
       } else {
          Mmsg(mdb->cmd, "SELECT MediaId,VolumeName,VolStatus,"
             "VolBytes,VolFiles,VolRetention,Recycle,Slot,InChanger,MediaType,LastWritten "
-            "FROM Media WHERE Media.PoolId=%u ORDER BY MediaId", mdbr->PoolId);
+            "FROM Media WHERE Media.PoolId=%s ORDER BY MediaId", 
+            edit_int64(mdbr->PoolId, ed1));
       }
    }
 
@@ -170,34 +181,35 @@ db_list_media_records(JCR *jcr, B_DB *mdb, MEDIA_DBR *mdbr,
    }
 
    list_result(jcr, mdb, sendit, ctx, type);
-   
+
    sql_free_result(mdb);
    db_unlock(mdb);
 }
 
-void db_list_jobmedia_records(JCR *jcr, B_DB *mdb, uint32_t JobId, 
-			      DB_LIST_HANDLER *sendit, void *ctx, e_list_type type)
+void db_list_jobmedia_records(JCR *jcr, B_DB *mdb, uint32_t JobId,
+                              DB_LIST_HANDLER *sendit, void *ctx, e_list_type type)
 {
+   char ed1[50];
    db_lock(mdb);
    if (type == VERT_LIST) {
-      if (JobId > 0) {			 /* do by JobId */
-         Mmsg(mdb->cmd, "SELECT JobMediaId,JobId,MediaId,Media.VolumeName,"
+      if (JobId > 0) {                   /* do by JobId */
+         Mmsg(mdb->cmd, "SELECT JobMediaId,JobId,Media.MediaId,Media.VolumeName,"
             "FirstIndex,LastIndex,StartFile,JobMedia.EndFile,StartBlock,"
-            "JobMedia.EndBlock "
+            "JobMedia.EndBlock,Copy,Stripe "
             "FROM JobMedia,Media WHERE Media.MediaId=JobMedia.MediaId "
-            "AND JobMedia.JobId=%u", JobId);
+            "AND JobMedia.JobId=%s", edit_int64(JobId, ed1));
       } else {
-         Mmsg(mdb->cmd, "SELECT JobMediaId,JobId,MediaId,Media.VolumeName,"
+         Mmsg(mdb->cmd, "SELECT JobMediaId,JobId,Media.MediaId,Media.VolumeName,"
             "FirstIndex,LastIndex,StartFile,JobMedia.EndFile,StartBlock,"
-            "JobMedia.EndBlock "
+            "JobMedia.EndBlock,Copy,Stripe "
             "FROM JobMedia,Media WHERE Media.MediaId=JobMedia.MediaId");
       }
 
    } else {
-      if (JobId > 0) {			 /* do by JobId */
+      if (JobId > 0) {                   /* do by JobId */
          Mmsg(mdb->cmd, "SELECT JobId,Media.VolumeName,FirstIndex,LastIndex "
             "FROM JobMedia,Media WHERE Media.MediaId=JobMedia.MediaId "
-            "AND JobMedia.JobId=%u", JobId);
+            "AND JobMedia.JobId=%s", edit_int64(JobId, ed1));
       } else {
          Mmsg(mdb->cmd, "SELECT JobId,Media.VolumeName,FirstIndex,LastIndex "
             "FROM JobMedia,Media WHERE Media.MediaId=JobMedia.MediaId");
@@ -209,7 +221,7 @@ void db_list_jobmedia_records(JCR *jcr, B_DB *mdb, uint32_t JobId,
    }
 
    list_result(jcr, mdb, sendit, ctx, type);
-   
+
    sql_free_result(mdb);
    db_unlock(mdb);
 }
@@ -223,13 +235,20 @@ void db_list_jobmedia_records(JCR *jcr, B_DB *mdb, uint32_t JobId,
  *  only the job with the specified id.
  */
 void
-db_list_job_records(JCR *jcr, B_DB *mdb, JOB_DBR *jr, DB_LIST_HANDLER *sendit, 
-		    void *ctx, e_list_type type)
+db_list_job_records(JCR *jcr, B_DB *mdb, JOB_DBR *jr, DB_LIST_HANDLER *sendit,
+                    void *ctx, e_list_type type)
 {
+   char ed1[50];
+   char limit[100];
    db_lock(mdb);
+   if (jr->limit > 0) {
+      snprintf(limit, sizeof(limit), " LIMIT %d", jr->limit);
+   } else {
+      limit[0] = 0;
+   }
    if (type == VERT_LIST) {
       if (jr->JobId == 0 && jr->Job[0] == 0) {
-	 Mmsg(mdb->cmd, 
+         Mmsg(mdb->cmd,
             "SELECT JobId,Job,Job.Name,PurgedFiles,Type,Level,"
             "Job.ClientId,Client.Name,JobStatus,SchedTime,"
             "StartTime,EndTime,JobTDate,"
@@ -237,26 +256,36 @@ db_list_job_records(JCR *jcr, B_DB *mdb, JOB_DBR *jr, DB_LIST_HANDLER *sendit,
             "JobMissingFiles,Job.PoolId,Pool.Name,Job.FileSetId,FileSet.FileSet "
             "FROM Job,Client,Pool,FileSet WHERE "
             "Client.ClientId=Job.ClientId AND Pool.PoolId=Job.PoolId "
-            "AND FileSet.FileSetId=Job.FileSetId  ORDER BY StartTime");
-      } else {				 /* single record */
-	 Mmsg(mdb->cmd, 
+            "AND FileSet.FileSetId=Job.FileSetId  ORDER BY StartTime%s", limit);
+      } else {                           /* single record */
+         Mmsg(mdb->cmd,
             "SELECT JobId,Job,Job.Name,PurgedFiles,Type,Level,"
             "Job.ClientId,Client.Name,JobStatus,SchedTime,"
             "StartTime,EndTime,JobTDate,"
             "VolSessionId,VolSessionTime,JobFiles,JobErrors,"
             "JobMissingFiles,Job.PoolId,Pool.Name,Job.FileSetId,FileSet.FileSet "
-            "FROM Job,Client,Pool,FileSet WHERE Job.JobId=%u AND "
+            "FROM Job,Client,Pool,FileSet WHERE Job.JobId=%s AND "
             "Client.ClientId=Job.ClientId AND Pool.PoolId=Job.PoolId "
-            "AND FileSet.FileSetId=Job.FileSetId", jr->JobId);
+            "AND FileSet.FileSetId=Job.FileSetId", 
+            edit_int64(jr->JobId, ed1));
       }
    } else {
-      if (jr->JobId == 0 && jr->Job[0] == 0) {
-	 Mmsg(mdb->cmd, 
+      if (jr->Name[0] != 0) {
+         Mmsg(mdb->cmd,
+            "SELECT JobId,Name,StartTime,Type,Level,JobFiles,JobBytes,JobStatus "
+            "FROM Job WHERE Name='%s' ORDER BY StartTime,JobId ASC", jr->Name);
+      } else if (jr->Job[0] != 0) {
+         Mmsg(mdb->cmd,
+            "SELECT JobId,Name,StartTime,Type,Level,JobFiles,JobBytes,JobStatus "
+            "FROM Job WHERE Job='%s' ORDER BY StartTime,JobId ASC", jr->Job);
+      } else if (jr->JobId != 0) {
+         Mmsg(mdb->cmd, 
+            "SELECT JobId,Name,StartTime,Type,Level,JobFiles,JobBytes,JobStatus "
+            "FROM Job WHERE JobId=%s", edit_int64(jr->JobId, ed1));
+      } else {                           /* all records */
+         Mmsg(mdb->cmd,
            "SELECT JobId,Name,StartTime,Type,Level,JobFiles,JobBytes,JobStatus "
-           "FROM Job ORDER BY StartTime");
-      } else {				 /* single record */
-         Mmsg(mdb->cmd, "SELECT JobId,Name,StartTime,Type,Level,"
-            "JobFiles,JobBytes,JobStatus FROM Job WHERE JobId=%u", jr->JobId);
+           "FROM Job ORDER BY StartTime,JobId ASC%s", limit);
       }
    }
    if (!QUERY_DB(jcr, mdb, mdb->cmd)) {
@@ -264,7 +293,7 @@ db_list_job_records(JCR *jcr, B_DB *mdb, JOB_DBR *jr, DB_LIST_HANDLER *sendit,
       return;
    }
    list_result(jcr, mdb, sendit, ctx, type);
-   
+
    sql_free_result(mdb);
    db_unlock(mdb);
 }
@@ -288,12 +317,12 @@ db_list_job_totals(JCR *jcr, B_DB *mdb, JOB_DBR *jr, DB_LIST_HANDLER *sendit, vo
    }
 
    list_result(jcr, mdb, sendit, ctx, HORZ_LIST);
-   
+
    sql_free_result(mdb);
 
    /* Do Grand Total */
-   Mmsg(mdb->cmd, "SELECT count(*) AS Jobs,sum(JobFiles) \
-AS Files,sum(JobBytes) As Bytes FROM Job");
+   Mmsg(mdb->cmd, "SELECT count(*) AS Jobs,sum(JobFiles) "
+        "AS Files,sum(JobBytes) As Bytes FROM Job");
 
    if (!QUERY_DB(jcr, mdb, mdb->cmd)) {
       db_unlock(mdb);
@@ -301,13 +330,13 @@ AS Files,sum(JobBytes) As Bytes FROM Job");
    }
 
    list_result(jcr, mdb, sendit, ctx, HORZ_LIST);
-   
+
    sql_free_result(mdb);
    db_unlock(mdb);
 }
 
-/* 
- * Stupid MySQL is NON-STANDARD !  
+/*
+ * Stupid MySQL is NON-STANDARD !
  */
 #ifdef HAVE_MYSQL
 #define FN "CONCAT(Path.Path,Filename.Name)"
@@ -316,14 +345,15 @@ AS Files,sum(JobBytes) As Bytes FROM Job");
 #endif
 
 void
-db_list_files_for_job(JCR *jcr, B_DB *mdb, uint32_t jobid, DB_LIST_HANDLER *sendit, void *ctx)
+db_list_files_for_job(JCR *jcr, B_DB *mdb, JobId_t jobid, DB_LIST_HANDLER *sendit, void *ctx)
 {
+   char ed1[50];
    db_lock(mdb);
 
    Mmsg(mdb->cmd, "SELECT " FN " AS Filename FROM File,"
-"Filename,Path WHERE File.JobId=%u AND Filename.FilenameId=File.FilenameId "
+"Filename,Path WHERE File.JobId=%s AND Filename.FilenameId=File.FilenameId "
 "AND Path.PathId=File.PathId",
-      jobid);
+      edit_int64(jobid, ed1));
 
    if (!QUERY_DB(jcr, mdb, mdb->cmd)) {
       db_unlock(mdb);
@@ -331,10 +361,10 @@ db_list_files_for_job(JCR *jcr, B_DB *mdb, uint32_t jobid, DB_LIST_HANDLER *send
    }
 
    list_result(jcr, mdb, sendit, ctx, HORZ_LIST);
-   
+
    sql_free_result(mdb);
    db_unlock(mdb);
 }
 
 
-#endif /* HAVE_MYSQL || HAVE_SQLITE || HAVE_POSTGRESQL */
+#endif /* HAVE_SQLITE3 || HAVE_MYSQL || HAVE_SQLITE || HAVE_POSTGRESQL*/
