@@ -3,26 +3,21 @@
  *
  *    Kern Sibbald, August MMI
  *
- *   Version $Id: status.c,v 1.43 2004/10/13 16:10:09 kerns Exp $
+ *   Version $Id: status.c,v 1.51.2.4 2006/03/14 21:41:40 kerns Exp $
  *
  */
 /*
-   Copyright (C) 2000-2004 Kern Sibbald and John Walker
+   Copyright (C) 2001-2006 Kern Sibbald
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of
-   the License, or (at your option) any later version.
+   modify it under the terms of the GNU General Public License
+   version 2 as amended with additional clauses defined in the
+   file LICENSE in the main source directory.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public
-   License along with this program; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+   the file LICENSE for additional details.
 
  */
 
@@ -48,11 +43,18 @@ static char DotStatusJob[] = "JobId=%d JobStatus=%c JobErrors=%d\n";
 #if defined(HAVE_CYGWIN) || defined(HAVE_WIN32)
 static int privs = 0;
 #endif
+#ifdef WIN32_VSS
+#include "vss.h"
+#define VSS " VSS"
+extern VSSClient *g_pVSSClient;
+#else
+#define VSS ""
+#endif
 
 /*
  * General status generator
  */
-static void do_status(void sendit(const char *msg, int len, void *sarg), void *arg) 
+static void do_status(void sendit(const char *msg, int len, void *sarg), void *arg)
 {
    int sec, bps;
    char *msg, b1[32], b2[32], b3[32], b4[32];
@@ -62,99 +64,132 @@ static void do_status(void sendit(const char *msg, int len, void *sarg), void *a
 
    msg = (char *)get_pool_memory(PM_MESSAGE);
    found = 0;
-   len = Mmsg(msg, "%s Version: " VERSION " (" BDATE ") %s %s %s\n", my_name,
-	      HOST_OS, DISTNAME, DISTVER);
+   len = Mmsg(msg, _("%s Version: %s (%s) %s %s %s %s\n"), 
+              my_name, VERSION, BDATE, VSS, HOST_OS, DISTNAME, DISTVER);
    sendit(msg, len, arg);
    bstrftime_nc(dt, sizeof(dt), daemon_start_time);
-   len = Mmsg(msg, _("Daemon started %s, %d Job%s run since started.\n"), 
+   len = Mmsg(msg, _("Daemon started %s, %d Job%s run since started.\n"),
         dt, num_jobs_run, num_jobs_run == 1 ? "" : "s");
    sendit(msg, len, arg);
 #if defined(HAVE_CYGWIN) || defined(HAVE_WIN32)
    if (debug_level > 0) {
       if (!privs) {
-	 privs = enable_backup_privileges(NULL, 1);
+         privs = enable_backup_privileges(NULL, 1);
       }
-      len = Mmsg(msg, 
-         _(" Priv 0x%x APIs=%sOPT,%sATP,%sLPV,%sGFAE,%sBR,%sBW,%sSPSP\n"), privs,
-         p_OpenProcessToken?"":"!",
-         p_AdjustTokenPrivileges?"":"!",
-         p_LookupPrivilegeValue?"":"!",
-         p_GetFileAttributesEx?"":"!",
-         p_BackupRead?"":"!",
-         p_BackupWrite?"":"!",
-         p_SetProcessShutdownParameters?"":"!");
+      len = Mmsg(msg, "Priv 0x%x\n", privs);
       sendit(msg, len, arg);
+      len = Mmsg(msg, "APIs=%sOPT,%sATP,%sLPV,%sCFA,%sCFW,\n",
+                 p_OpenProcessToken?"":"!",
+                 p_AdjustTokenPrivileges?"":"!",
+                 p_LookupPrivilegeValue?"":"!",
+                 p_CreateFileA?"":"!",
+                 p_CreateFileW?"":"!");
+      sendit(msg, len, arg);
+      len = Mmsg(msg, " %sWUL,%sWMKD,%sWOP,%sGFAA,%sGFAW,%sGFAEA,%sGFAEW,%sSFAA,%sSFAW,%sBR,%sBW,%sSPSP,\n",
+                 p_wunlink?"":"!",
+                 p_wmkdir?"":"!",
+                 p_wopen?"":"!",
+                 p_GetFileAttributesA?"":"!",
+                 p_GetFileAttributesW?"":"!",
+                 p_GetFileAttributesExA?"":"!",
+                 p_GetFileAttributesExW?"":"!",
+                 p_SetFileAttributesA?"":"!",
+                 p_SetFileAttributesW?"":"!",
+                 p_BackupRead?"":"!",
+                 p_BackupWrite?"":"!",
+                 p_SetProcessShutdownParameters?"":"!");
+      sendit(msg, len, arg);
+      len = Mmsg(msg, " %sWC2MB,%sMB2WC,%sFFFA,%sFFFW,%sFNFA,%sFNFW,%sSCDA,%sSCDW,\n",
+                 p_WideCharToMultiByte?"":"!",
+                 p_MultiByteToWideChar?"":"!",
+                 p_FindFirstFileA?"":"!",
+                 p_FindFirstFileW?"":"!",
+                 p_FindNextFileA?"":"!",
+                 p_FindNextFileW?"":"!",
+                 p_SetCurrentDirectoryA?"":"!",
+                 p_SetCurrentDirectoryW?"":"!");
+      sendit(msg, len, arg);
+      len = Mmsg(msg, " %sGCDA,%sGCDW,%sGVPNW,%sGVNFVMPW\n",  
+                 p_GetCurrentDirectoryA?"":"!",
+                 p_GetCurrentDirectoryW?"":"!",
+                 p_GetVolumePathNameW?"":"!",
+                 p_GetVolumeNameForVolumeMountPointW?"":"!");
+     sendit(msg, len, arg);
    }
 #endif
    if (debug_level > 0) {
       len = Mmsg(msg, _(" Heap: bytes=%s max_bytes=%s bufs=%s max_bufs=%s\n"),
-	    edit_uint64_with_commas(sm_bytes, b1),
-	    edit_uint64_with_commas(sm_max_bytes, b2),
-	    edit_uint64_with_commas(sm_buffers, b3),
-	    edit_uint64_with_commas(sm_max_buffers, b4));
+            edit_uint64_with_commas(sm_bytes, b1),
+            edit_uint64_with_commas(sm_max_bytes, b2),
+            edit_uint64_with_commas(sm_buffers, b3),
+            edit_uint64_with_commas(sm_max_buffers, b4));
       sendit(msg, len, arg);
-      len = Mmsg(msg, _(" Sizeof: off_t=%d size_t=%d debug=%d trace=%d\n"), 
-	    sizeof(off_t), sizeof(size_t), debug_level, get_trace());
+      len = Mmsg(msg, _(" Sizeof: off_t=%d size_t=%d debug=%d trace=%d\n"),
+            sizeof(off_t), sizeof(size_t), debug_level, get_trace());
       sendit(msg, len, arg);
    }
 
    list_terminated_jobs(sendit, arg);
 
    /*
-    * List running jobs  
+    * List running jobs
     */
    Dmsg0(1000, "Begin status jcr loop.\n");
    len = Mmsg(msg, _("Running Jobs:\n"));
    sendit(msg, len, arg);
-   lock_jcr_chain();
+   char *vss = "";
+#ifdef WIN32_VSS
+   if (g_pVSSClient && g_pVSSClient->IsInitialized()) {
+      vss = "VSS ";
+   }
+#endif
    foreach_jcr(njcr) {
       bstrftime_nc(dt, sizeof(dt), njcr->start_time);
       if (njcr->JobId == 0) {
          len = Mmsg(msg, _("Director connected at: %s\n"), dt);
       } else {
-         len = Mmsg(msg, _("JobId %d Job %s is running.\n"), 
-		    njcr->JobId, njcr->Job);
-	 sendit(msg, len, arg);
-         len = Mmsg(msg, _("    %s Job started: %s\n"), 
-		    job_type_to_str(njcr->JobType), dt);
+         len = Mmsg(msg, _("JobId %d Job %s is running.\n"),
+                    njcr->JobId, njcr->Job);
+         sendit(msg, len, arg);
+         len = Mmsg(msg, _("    %s%s Job started: %s\n"),
+                    vss, job_type_to_str(njcr->JobType), dt);
       }
       sendit(msg, len, arg);
       if (njcr->JobId == 0) {
-	 free_locked_jcr(njcr);
-	 continue;
+         continue;
       }
       sec = time(NULL) - njcr->start_time;
       if (sec <= 0) {
-	 sec = 1;
+         sec = 1;
       }
       bps = (int)(njcr->JobBytes / sec);
-      len = Mmsg(msg,  _("    Files=%s Bytes=%s Bytes/sec=%s\n"), 
-	   edit_uint64_with_commas(njcr->JobFiles, b1),
-	   edit_uint64_with_commas(njcr->JobBytes, b2),
-	   edit_uint64_with_commas(bps, b3));
+      len = Mmsg(msg,  _("    Files=%s Bytes=%s Bytes/sec=%s\n"),
+           edit_uint64_with_commas(njcr->JobFiles, b1),
+           edit_uint64_with_commas(njcr->JobBytes, b2),
+           edit_uint64_with_commas(bps, b3));
       sendit(msg, len, arg);
-      len = Mmsg(msg, _("    Files Examined=%s\n"), 
-	   edit_uint64_with_commas(njcr->num_files_examined, b1));
+      len = Mmsg(msg, _("    Files Examined=%s\n"),
+           edit_uint64_with_commas(njcr->num_files_examined, b1));
       sendit(msg, len, arg);
       if (njcr->JobFiles > 0) {
-	 P(njcr->mutex);
+         njcr->lock();
          len = Mmsg(msg, _("    Processing file: %s\n"), njcr->last_fname);
-	 V(njcr->mutex);
-	 sendit(msg, len, arg);
+         njcr->unlock();
+         sendit(msg, len, arg);
       }
 
       found = 1;
       if (njcr->store_bsock) {
          len = Mmsg(msg, "    SDReadSeqNo=%" lld " fd=%d\n",
-	     njcr->store_bsock->read_seqno, njcr->store_bsock->fd);
-	 sendit(msg, len, arg);
+             njcr->store_bsock->read_seqno, njcr->store_bsock->fd);
+         sendit(msg, len, arg);
       } else {
          len = Mmsg(msg, _("    SDSocket closed.\n"));
-	 sendit(msg, len, arg);
+         sendit(msg, len, arg);
       }
-      free_locked_jcr(njcr);
    }
-   unlock_jcr_chain();
+   endeach_jcr(njcr);
+
    Dmsg0(1000, "Begin status jcr loop.\n");
    if (!found) {
       len = Mmsg(msg, _("No Jobs running.\n"));
@@ -165,7 +200,7 @@ static void do_status(void sendit(const char *msg, int len, void *sarg), void *a
    free_pool_memory(msg);
 }
 
-static void  list_terminated_jobs(void sendit(const char *msg, int len, void *sarg), void *arg) 
+static void  list_terminated_jobs(void sendit(const char *msg, int len, void *sarg), void *arg)
 {
    char dt[MAX_TIME_LENGTH], b1[30], b2[30];
    char level[10];
@@ -173,17 +208,17 @@ static void  list_terminated_jobs(void sendit(const char *msg, int len, void *sa
    const char *msg;
 
    if (last_jobs->size() == 0) {
-      msg = _("No Terminated Jobs.\n"); 
+      msg = _("No Terminated Jobs.\n");
       sendit(msg, strlen(msg), arg);
       return;
    }
    lock_last_jobs_list();
    sendit("\n", 1, arg);               /* send separately */
-   msg =  _("Terminated Jobs:\n"); 
+   msg =  _("Terminated Jobs:\n");
    sendit(msg, strlen(msg), arg);
    msg =  _(" JobId  Level     Files         Bytes  Status   Finished        Name \n");
    sendit(msg, strlen(msg), arg);
-   msg = _("======================================================================\n"); 
+   msg = _("======================================================================\n");
    sendit(msg, strlen(msg), arg);
    foreach_dlist(je, last_jobs) {
       char JobName[MAX_NAME_LENGTH];
@@ -195,51 +230,51 @@ static void  list_terminated_jobs(void sendit(const char *msg, int len, void *sa
       case JT_ADMIN:
       case JT_RESTORE:
          bstrncpy(level, "    ", sizeof(level));
-	 break;
+         break;
       default:
-	 bstrncpy(level, level_to_str(je->JobLevel), sizeof(level));
-	 level[4] = 0;
-	 break;
+         bstrncpy(level, level_to_str(je->JobLevel), sizeof(level));
+         level[4] = 0;
+         break;
       }
       switch (je->JobStatus) {
       case JS_Created:
-         termstat = "Created";
-	 break;
+         termstat = _("Created");
+         break;
       case JS_FatalError:
       case JS_ErrorTerminated:
-         termstat = "Error";
-	 break;
+         termstat = _("Error");
+         break;
       case JS_Differences:
-         termstat = "Diffs";
-	 break;
+         termstat = _("Diffs");
+         break;
       case JS_Canceled:
-         termstat = "Cancel";
-	 break;
+         termstat = _("Cancel");
+         break;
       case JS_Terminated:
-         termstat = "OK";
-	 break;
+         termstat = _("OK");
+         break;
       default:
-         termstat = "Other";
-	 break;
+         termstat = _("Other");
+         break;
       }
       bstrncpy(JobName, je->Job, sizeof(JobName));
       /* There are three periods after the Job name */
       char *p;
       for (int i=0; i<3; i++) {
          if ((p=strrchr(JobName, '.')) != NULL) {
-	    *p = 0;
-	 }
+            *p = 0;
+         }
       }
-      bsnprintf(buf, sizeof(buf), _("%6d  %-6s %8s %14s %-7s  %-8s %s\n"), 
-	 je->JobId,
-	 level, 
-	 edit_uint64_with_commas(je->JobFiles, b1),
-	 edit_uint64_with_commas(je->JobBytes, b2), 
-	 termstat,
-	 dt, JobName);
+      bsnprintf(buf, sizeof(buf), _("%6d  %-6s %8s %14s %-7s  %-8s %s\n"),
+         je->JobId,
+         level,
+         edit_uint64_with_commas(je->JobFiles, b1),
+         edit_uint64_with_commas(je->JobBytes, b2),
+         termstat,
+         dt, JobName);
       sendit(buf, strlen(buf), arg);
    }
-   sendit("====\n", 5, arg);
+   sendit(_("====\n"), 5, arg);
    unlock_last_jobs_list();
 }
 
@@ -256,7 +291,7 @@ static void bsock_sendit(const char *msg, int len, void *arg)
    user->msglen = len+1;
    bnet_send(user);
 }
-				   
+
 /*
  * Status command from Director
  */
@@ -282,44 +317,40 @@ int qstatus_cmd(JCR *jcr)
    s_last_job* job;
 
    time = get_memory(dir->msglen+1);
-   
+
    if (sscanf(dir->msg, qstatus, time) != 1) {
       pm_strcpy(&jcr->errmsg, dir->msg);
       Jmsg1(jcr, M_FATAL, 0, _("Bad .status command: %s\n"), jcr->errmsg);
-      bnet_fsend(dir, "2900 Bad .status command, missing argument.\n");
+      bnet_fsend(dir, _("2900 Bad .status command, missing argument.\n"));
       bnet_sig(dir, BNET_EOD);
       free_memory(time);
       return 0;
    }
    unbash_spaces(time);
-   
+
    if (strcmp(time, "current") == 0) {
       bnet_fsend(dir, OKqstatus, time);
-      lock_jcr_chain();
       foreach_jcr(njcr) {
-	 if (njcr->JobId != 0) {
-	    bnet_fsend(dir, DotStatusJob, njcr->JobId, njcr->JobStatus, njcr->JobErrors);
-	 }
-	 free_locked_jcr(njcr);
+         if (njcr->JobId != 0) {
+            bnet_fsend(dir, DotStatusJob, njcr->JobId, njcr->JobStatus, njcr->JobErrors);
+         }
       }
-      unlock_jcr_chain();
-   }
-   else if (strcmp(time, "last") == 0) {
+      endeach_jcr(njcr);
+   } else if (strcmp(time, "last") == 0) {
       bnet_fsend(dir, OKqstatus, time);
       if ((last_jobs) && (last_jobs->size() > 0)) {
-	 job = (s_last_job*)last_jobs->last();
-	 bnet_fsend(dir, DotStatusJob, job->JobId, job->JobStatus, job->Errors);
+         job = (s_last_job*)last_jobs->last();
+         bnet_fsend(dir, DotStatusJob, job->JobId, job->JobStatus, job->Errors);
       }
-   }
-   else {
+   } else {
       pm_strcpy(&jcr->errmsg, dir->msg);
       Jmsg1(jcr, M_FATAL, 0, _("Bad .status command: %s\n"), jcr->errmsg);
-      bnet_fsend(dir, "2900 Bad .status command, wrong argument.\n");
+      bnet_fsend(dir, _("2900 Bad .status command, wrong argument.\n"));
       bnet_sig(dir, BNET_EOD);
       free_memory(time);
       return 0;
    }
-   
+
    bnet_sig(dir, BNET_EOD);
    free_memory(time);
    return 1;
@@ -328,7 +359,7 @@ int qstatus_cmd(JCR *jcr)
 /*
  * Convert Job Level into a string
  */
-static const char *level_to_str(int level) 
+static const char *level_to_str(int level)
 {
    const char *str;
 
@@ -391,14 +422,14 @@ static void win32_sendit(const char *msg, int len, void *marg)
    struct s_win32_arg *arg = (struct s_win32_arg *)marg;
 
    if (len > 0 && msg[len-1] == '\n') {
-       // when compiling with visual studio some strings are read-only 
-       // and cause access violations.	So we creat a tmp copy.
+       // when compiling with visual studio some strings are read-only
+       // and cause access violations.  So we creat a tmp copy.
        char *_msg = (char *)alloca(len);
        bstrncpy(_msg, msg, len);
        msg = _msg;
    }
    SendDlgItemMessage(arg->hwnd, arg->idlist, LB_ADDSTRING, 0, (LONG)msg);
-   
+
 }
 
 void FillStatusBox(HWND hwnd, int idlist)
@@ -419,23 +450,21 @@ char *bac_status(char *buf, int buf_len)
    JCR *njcr;
    const char *termstat = _("Bacula Idle");
    struct s_last_job *job;
-   int stat = 0;		      /* Idle */
+   int stat = 0;                      /* Idle */
 
    if (!last_jobs) {
       goto done;
    }
    Dmsg0(1000, "Begin bac_status jcr loop.\n");
-   lock_jcr_chain();
    foreach_jcr(njcr) {
       if (njcr->JobId != 0) {
-	 stat = JS_Running;
+         stat = JS_Running;
          termstat = _("Bacula Running");
-	 free_locked_jcr(njcr);
-	 break;
+         break;
       }
-      free_locked_jcr(njcr);
    }
-   unlock_jcr_chain();
+   endeach_jcr(njcr);
+
    if (stat != 0) {
       goto done;
    }
@@ -445,16 +474,16 @@ char *bac_status(char *buf, int buf_len)
       switch (job->JobStatus) {
       case JS_Canceled:
          termstat = _("Last Job Canceled");
-	 break;
+         break;
       case JS_ErrorTerminated:
-      case JS_FatalError:  
+      case JS_FatalError:
          termstat = _("Last Job Failed");
-	 break;
+         break;
       default:
-	 if (job->Errors) {
+         if (job->Errors) {
             termstat = _("Last Job had Warnings");
-	 }
-	 break;
+         }
+         break;
       }
    }
    Dmsg0(1000, "End bac_status jcr loop.\n");
