@@ -1,13 +1,13 @@
 /*
- *  Bacula File Daemon	verify.c  Verify files.
+ *  Bacula File Daemon  verify.c  Verify files.
  *
  *    Kern Sibbald, October MM
  *
- *   Version $Id: verify.c,v 1.37 2004/11/15 22:43:33 kerns Exp $
+ *   Version $Id: verify.c,v 1.44.2.1 2006/03/14 21:41:40 kerns Exp $
  *
  */
 /*
-   Copyright (C) 2000-2004 Kern Sibbald and John Walker
+   Copyright (C) 2000-2004 Kern Sibbald
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -29,25 +29,26 @@
 #include "bacula.h"
 #include "filed.h"
 
-static int verify_file(FF_PKT *ff_pkt, void *my_pkt);
+static int verify_file(FF_PKT *ff_pkt, void *my_pkt, bool);
+static int read_chksum(BFILE *bfd, CHKSUM *chksum, JCR *jcr);
 
-/* 
+/*
  * Find all the requested files and send attributes
  * to the Director.
- * 
+ *
  */
 void do_verify(JCR *jcr)
 {
    set_jcr_job_status(jcr, JS_Running);
    jcr->buf_size = DEFAULT_NETWORK_BUFFER_SIZE;
    if ((jcr->big_buf = (char *) malloc(jcr->buf_size)) == NULL) {
-      Jmsg1(jcr, M_ABORT, 0, _("Cannot malloc %d network read buffer\n"), 
-	 DEFAULT_NETWORK_BUFFER_SIZE);
+      Jmsg1(jcr, M_ABORT, 0, _("Cannot malloc %d network read buffer\n"),
+         DEFAULT_NETWORK_BUFFER_SIZE);
    }
    set_find_options((FF_PKT *)jcr->ff, jcr->incremental, jcr->mtime);
    Dmsg0(10, "Start find files\n");
    /* Subroutine verify_file() is called for each file */
-   find_files(jcr, (FF_PKT *)jcr->ff, verify_file, (void *)jcr);  
+   find_files(jcr, (FF_PKT *)jcr->ff, verify_file, (void *)jcr);
    Dmsg0(10, "End find files\n");
 
    if (jcr->big_buf) {
@@ -55,34 +56,32 @@ void do_verify(JCR *jcr)
       jcr->big_buf = NULL;
    }
    set_jcr_job_status(jcr, JS_Terminated);
-}	   
+}
 
-/* 
+/*
  * Called here by find() for each file.
  *
  *  Find the file, compute the MD5 or SHA1 and send it back to the Director
  */
-static int verify_file(FF_PKT *ff_pkt, void *pkt) 
+static int verify_file(FF_PKT *ff_pkt, void *pkt, bool top_level)
 {
    char attribs[MAXSTRING];
-   int64_t n;
+   char attribsEx[MAXSTRING];
    int stat;
    BFILE bfd;
-   struct MD5Context md5c;
-   struct SHA1Context sha1c;
-   unsigned char signature[25];       /* large enough for either */
+   struct CHKSUM chksum;
    BSOCK *dir;
    JCR *jcr = (JCR *)pkt;
 
    if (job_canceled(jcr)) {
       return 0;
    }
-   
+
    dir = jcr->dir_bsock;
-   jcr->num_files_examined++;	      /* bump total file count */
+   jcr->num_files_examined++;         /* bump total file count */
 
    switch (ff_pkt->type) {
-   case FT_LNKSAVED:		      /* Hard linked, file already saved */
+   case FT_LNKSAVED:                  /* Hard linked, file already saved */
       Dmsg2(30, "FT_LNKSAVED saving: %s => %s\n", ff_pkt->fname, ff_pkt->link);
       break;
    case FT_REGE:
@@ -95,7 +94,7 @@ static int verify_file(FF_PKT *ff_pkt, void *pkt)
       Dmsg2(30, "FT_LNK saving: %s -> %s\n", ff_pkt->fname, ff_pkt->link);
       break;
    case FT_DIRBEGIN:
-      return 1; 		      /* ignored */
+      return 1;                       /* ignored */
    case FT_DIREND:
       Dmsg1(30, "FT_DIR saving: %s\n", ff_pkt->fname);
       break;
@@ -155,37 +154,23 @@ static int verify_file(FF_PKT *ff_pkt, void *pkt)
       return 1;
    }
 
-   binit(&bfd);
-
-   if (ff_pkt->type != FT_LNKSAVED && (S_ISREG(ff_pkt->statp.st_mode) && 
-	 ff_pkt->statp.st_size > 0) || 
-	 ff_pkt->type == FT_RAW || ff_pkt->type == FT_FIFO) {
-      if ((bopen(&bfd, ff_pkt->fname, O_RDONLY | O_BINARY, 0)) < 0) {
-	 ff_pkt->ff_errno = errno;
-	 berrno be;
-	 be.set_errno(bfd.berrno);
-         Jmsg(jcr, M_NOTSAVED, 1, _("     Cannot open %s: ERR=%s.\n"),
-	      ff_pkt->fname, be.strerror());
-	 jcr->Errors++;
-	 return 1;
-      }
-   }
-
+   /* Encode attributes and possibly extend them */
    encode_stat(attribs, ff_pkt, 0);
-     
-   P(jcr->mutex);
-   jcr->JobFiles++;		     /* increment number of files sent */
-   pm_strcpy(jcr->last_fname, ff_pkt->fname);
-   V(jcr->mutex);
+   encode_attribsEx(jcr, attribsEx, ff_pkt);
 
-   /* 
+   jcr->lock();
+   jcr->JobFiles++;                  /* increment number of files sent */
+   pm_strcpy(jcr->last_fname, ff_pkt->fname);
+   jcr->unlock();
+
+   /*
     * Send file attributes to Director
-    *	File_index
-    *	Stream
-    *	Verify Options
-    *	Filename (full path)
-    *	Encoded attributes
-    *	Link name (if type==FT_LNK)
+    *   File_index
+    *   Stream
+    *   Verify Options
+    *   Filename (full path)
+    *   Encoded attributes
+    *   Link name (if type==FT_LNK)
     * For a directory, link is the same as fname, but with trailing
     * slash. For a linked file, link is the link.
     */
@@ -193,75 +178,113 @@ static int verify_file(FF_PKT *ff_pkt, void *pkt)
    Dmsg2(400, "send ATTR inx=%d fname=%s\n", jcr->JobFiles, ff_pkt->fname);
    if (ff_pkt->type == FT_LNK || ff_pkt->type == FT_LNKSAVED) {
       stat = bnet_fsend(dir, "%d %d %s %s%c%s%c%s%c", jcr->JobFiles,
-		    STREAM_UNIX_ATTRIBUTES, ff_pkt->VerifyOpts, ff_pkt->fname, 
-		    0, attribs, 0, ff_pkt->link, 0);
+            STREAM_UNIX_ATTRIBUTES, ff_pkt->VerifyOpts, ff_pkt->fname,
+            0, attribs, 0, ff_pkt->link, 0);
    } else if (ff_pkt->type == FT_DIREND) {
-      /* Here link is the canonical filename (i.e. with trailing slash) */
-      stat = bnet_fsend(dir,"%d %d %s %s%c%s%c%c", jcr->JobFiles,
-		    STREAM_UNIX_ATTRIBUTES, ff_pkt->VerifyOpts, ff_pkt->link, 
-		    0, attribs, 0, 0);
+         /* Here link is the canonical filename (i.e. with trailing slash) */
+         stat = bnet_fsend(dir,"%d %d %s %s%c%s%c%c", jcr->JobFiles,
+               STREAM_UNIX_ATTRIBUTES, ff_pkt->VerifyOpts, ff_pkt->link,
+               0, attribs, 0, 0);
    } else {
       stat = bnet_fsend(dir,"%d %d %s %s%c%s%c%c", jcr->JobFiles,
-		    STREAM_UNIX_ATTRIBUTES, ff_pkt->VerifyOpts, ff_pkt->fname, 
-		    0, attribs, 0, 0);
+            STREAM_UNIX_ATTRIBUTES, ff_pkt->VerifyOpts, ff_pkt->fname,
+            0, attribs, 0, 0);
    }
    Dmsg2(20, "bfiled>bdird: attribs len=%d: msg=%s\n", dir->msglen, dir->msg);
    if (!stat) {
       Jmsg(jcr, M_FATAL, 0, _("Network error in send to Director: ERR=%s\n"), bnet_strerror(dir));
-      if (is_bopen(&bfd)) {
-	 bclose(&bfd);
-      }
       return 0;
    }
 
-   /* If file opened, compute MD5 or SHA1 hash */
-   if (is_bopen(&bfd)  && ff_pkt->flags & FO_MD5) {
-      char MD5buf[40];		      /* 24 should do */
-      MD5Init(&md5c);
-      while ((n=bread(&bfd, jcr->big_buf, jcr->buf_size)) > 0) {
-	 MD5Update(&md5c, ((unsigned char *)jcr->big_buf), (int)n);
-	 jcr->JobBytes += n;
-	 jcr->ReadBytes += n;
-      }
-      if (n < 0) {
-	 berrno be;
-	 be.set_errno(bfd.berrno);
-         Jmsg(jcr, M_ERROR, 1, _("Error reading file %s: ERR=%s\n"), 
-	      ff_pkt->fname, be.strerror());
-	 jcr->Errors++;
-      }
-      MD5Final(signature, &md5c);
+   /*
+    * The remainder of the function is all about getting the checksum.
+    * First we initialise, then we read files, other streams and Finder Info.
+    */
+   if (ff_pkt->type != FT_LNKSAVED && (S_ISREG(ff_pkt->statp.st_mode) &&
+            ff_pkt->flags & (FO_MD5|FO_SHA1))) {
+      chksum_init(&chksum, ff_pkt->flags);
+      binit(&bfd);
 
-      bin_to_base64(MD5buf, (char *)signature, 16); /* encode 16 bytes */
-      Dmsg2(400, "send inx=%d MD5=%s\n", jcr->JobFiles, MD5buf);
-      bnet_fsend(dir, "%d %d %s *MD5-%d*", jcr->JobFiles, STREAM_MD5_SIGNATURE, MD5buf,
-	 jcr->JobFiles);
-      Dmsg2(20, "bfiled>bdird: MD5 len=%d: msg=%s\n", dir->msglen, dir->msg);
-   } else if (is_bopen(&bfd) && ff_pkt->flags & FO_SHA1) {
-      char SHA1buf[40]; 	      /* 24 should do */
-      SHA1Init(&sha1c);
-      while ((n=bread(&bfd, jcr->big_buf, jcr->buf_size)) > 0) {
-	 SHA1Update(&sha1c, ((unsigned char *)jcr->big_buf), (int)n);
-	 jcr->JobBytes += n;
-	 jcr->ReadBytes += n;
+      if (ff_pkt->statp.st_size > 0 || ff_pkt->type == FT_RAW
+            || ff_pkt->type == FT_FIFO) {
+         if ((bopen(&bfd, ff_pkt->fname, O_RDONLY | O_BINARY, 0)) < 0) {
+            ff_pkt->ff_errno = errno;
+            berrno be;
+            be.set_errno(bfd.berrno);
+            Jmsg(jcr, M_NOTSAVED, 1, _("     Cannot open %s: ERR=%s.\n"),
+                 ff_pkt->fname, be.strerror());
+            jcr->Errors++;
+            return 1;
+         }
+         read_chksum(&bfd, &chksum, jcr);
+         bclose(&bfd);
       }
-      if (n < 0) {
-	 berrno be;
-	 be.set_errno(bfd.berrno);
-         Jmsg(jcr, M_ERROR, 1, _("Error reading file %s: ERR=%s\n"), 
-	      ff_pkt->fname, be.strerror());
-	 jcr->Errors++;
-      }
-      SHA1Final(&sha1c, signature);
 
-      bin_to_base64(SHA1buf, (char *)signature, 20); /* encode 20 bytes */
-      Dmsg2(400, "send inx=%d SHA1=%s\n", jcr->JobFiles, SHA1buf);
-      bnet_fsend(dir, "%d %d %s *SHA1-%d*", jcr->JobFiles, STREAM_SHA1_SIGNATURE, 
-	 SHA1buf, jcr->JobFiles);
-      Dmsg2(20, "bfiled>bdird: SHA1 len=%d: msg=%s\n", dir->msglen, dir->msg);
-   }
-   if (is_bopen(&bfd)) {
-      bclose(&bfd);
+#ifdef HAVE_DARWIN_OS
+      /* Open resource fork if necessary */
+      if (ff_pkt->flags & FO_HFSPLUS && ff_pkt->hfsinfo.rsrclength > 0) {
+         if (bopen_rsrc(&bfd, ff_pkt->fname, O_RDONLY | O_BINARY, 0) < 0) {
+            ff_pkt->ff_errno = errno;
+            berrno be;
+            Jmsg(jcr, M_NOTSAVED, -1, _("     Cannot open resource fork for %s: ERR=%s.\n"),
+                  ff_pkt->fname, be.strerror());
+            jcr->Errors++;
+            if (is_bopen(&ff_pkt->bfd)) {
+               bclose(&ff_pkt->bfd);
+            }
+            return 1;
+         }
+         read_chksum(&bfd, &chksum, jcr);
+         bclose(&bfd);
+      }
+      if (ff_pkt->flags & FO_HFSPLUS) {
+         chksum_update(&chksum, ((unsigned char *)ff_pkt->hfsinfo.fndrinfo), 32);
+      }
+#endif
+
+      /* compute MD5 or SHA1 hash */
+      if (chksum.updated) {
+         char chksumbuf[40];                  /* 24 should do */
+         int stream = 0;
+
+         chksum_final(&chksum);
+         if (chksum.type == CHKSUM_MD5) {
+            stream = STREAM_MD5_SIGNATURE;
+         } else if (chksum.type == CHKSUM_SHA1) {
+            stream = STREAM_SHA1_SIGNATURE;
+         }
+         bin_to_base64(chksumbuf, (char *)chksum.signature, chksum.length);
+         Dmsg3(400, "send inx=%d %s=%s\n", jcr->JobFiles, chksum.name, chksumbuf);
+         bnet_fsend(dir, "%d %d %s *%s-%d*", jcr->JobFiles, stream, chksumbuf,
+               chksum.name, jcr->JobFiles);
+         Dmsg3(20, "bfiled>bdird: %s len=%d: msg=%s\n", chksum.name,
+               dir->msglen, dir->msg);
+      }
    }
+
    return 1;
 }
+
+/*
+ * Read checksum of bfd, updating chksum
+ * In case of errors we need the job control record and file name.
+ */
+int read_chksum(BFILE *bfd, CHKSUM *chksum, JCR *jcr)
+{
+   int64_t n;
+
+   while ((n=bread(bfd, jcr->big_buf, jcr->buf_size)) > 0) {
+      chksum_update(chksum, ((unsigned char *)jcr->big_buf), (int)n);
+      jcr->JobBytes += n;
+      jcr->ReadBytes += n;
+   }
+   if (n < 0) {
+      berrno be;
+      be.set_errno(bfd->berrno);
+      Jmsg(jcr, M_ERROR, 1, _("Error reading file %s: ERR=%s\n"),
+            jcr->last_fname, be.strerror());
+      jcr->Errors++;
+      return -1;
+   }
+   return 0;
+}
