@@ -5,26 +5,21 @@
  *
  *    Kern Sibbald, October MMII
  *
- *   Version $Id: attribs.c,v 1.42 2004/10/04 20:34:01 kerns Exp $
+ *   Version $Id: attribs.c,v 1.56 2005/09/24 13:11:31 kerns Exp $
  *
  */
 /*
-   Copyright (C) 2002-2004 Kern Sibbald and John Walker
+   Copyright (C) 2000-2005 Kern Sibbald
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of
-   the License, or (at your option) any later version.
+   modify it under the terms of the GNU General Public License
+   version 2 as amended with additional clauses defined in the
+   file LICENSE in the main source directory.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public
-   License along with this program; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+   the file LICENSE for additional details.
 
  */
 
@@ -33,6 +28,9 @@
 
 #if defined(HAVE_CYGWIN) || defined(HAVE_WIN32)
 
+#include "../lib/winapi.h"
+
+
 /* Forward referenced subroutines */
 static bool set_win32_attributes(JCR *jcr, ATTR *attr, BFILE *ofd);
 void unix_name_to_win32(POOLMEM **win32_name, char *name);
@@ -46,13 +44,13 @@ HANDLE bget_handle(BFILE *bfd);
 #endif
 
 /*=============================================================*/
-/*							       */
-/*	       ***  A l l  S y s t e m s ***		       */
-/*							       */
+/*                                                             */
+/*             ***  A l l  S y s t e m s ***                   */
+/*                                                             */
 /*=============================================================*/
 
 /*
- * Return the data stream that will be used 
+ * Return the data stream that will be used
  */
 int select_data_stream(FF_PKT *ff_pkt)
 {
@@ -70,11 +68,11 @@ int select_data_stream(FF_PKT *ff_pkt)
 #ifdef HAVE_LIBZ
    if (ff_pkt->flags & FO_GZIP) {
       if (stream == STREAM_WIN32_DATA) {
-	 stream = STREAM_WIN32_GZIP_DATA;
+         stream = STREAM_WIN32_GZIP_DATA;
       } else if (stream == STREAM_FILE_DATA) {
-	 stream = STREAM_GZIP_DATA;
+         stream = STREAM_GZIP_DATA;
       } else {
-	 stream = STREAM_SPARSE_GZIP_DATA;
+         stream = STREAM_SPARSE_GZIP_DATA;
       }
    }
 #endif
@@ -82,8 +80,8 @@ int select_data_stream(FF_PKT *ff_pkt)
 }
 
 
-/* 
- * Encode a stat structure into a base64 character string   
+/*
+ * Encode a stat structure into a base64 character string
  *   All systems must create such a structure.
  *   In addition, we tack on the LinkFI, which is non-zero in
  *   the case of a hard linked file that has no data.  This
@@ -99,7 +97,7 @@ void encode_stat(char *buf, FF_PKT *ff_pkt, int data_stream)
    struct stat *statp = &ff_pkt->statp;
    /*
     *  Encode a stat packet.  I should have done this more intelligently
-    *	with a length so that it could be easily expanded.
+    *   with a length so that it could be easily expanded.
     */
    p += to_base64((int64_t)statp->st_dev, p);
    *p++ = ' ';                        /* separate fields with a space */
@@ -152,7 +150,7 @@ void encode_stat(char *buf, FF_PKT *ff_pkt, int data_stream)
 
 /* Do casting according to unknown type to keep compiler happy */
 #if !HAVE_GCC & HAVE_SUN_OS
-#define plug(st, val) st = val	      /* brain damaged compiler */
+#define plug(st, val) st = val        /* brain damaged compiler */
 #else
 template <class T> void plug(T &st, uint64_t val)
     { st = static_cast<T>(val); }
@@ -160,7 +158,7 @@ template <class T> void plug(T &st, uint64_t val)
 
 
 /* Decode a stat packet from base64 characters */
-int decode_stat(char *buf, struct stat *statp, int32_t *LinkFI) 
+int decode_stat(char *buf, struct stat *statp, int32_t *LinkFI)
 {
    char *p = buf;
    int64_t val;
@@ -233,7 +231,7 @@ int decode_stat(char *buf, struct stat *statp, int32_t *LinkFI)
       statp->st_flags  = 0;
 #endif
    }
-    
+
    /* Look for data stream id */
    if (*p == ' ' || (*p != 0 && *(p+1) == ' ')) {
       p++;
@@ -250,32 +248,32 @@ int32_t decode_LinkFI(char *buf, struct stat *statp)
    char *p = buf;
    int64_t val;
 
-   skip_nonspaces(&p);		      /* st_dev */
-   p++; 			      /* skip space */
-   skip_nonspaces(&p);		      /* st_ino */
+   skip_nonspaces(&p);                /* st_dev */
+   p++;                               /* skip space */
+   skip_nonspaces(&p);                /* st_ino */
    p++;
    p += from_base64(&val, p);
-   plug(statp->st_mode, val);	      /* st_mode */
+   plug(statp->st_mode, val);         /* st_mode */
    p++;
-   skip_nonspaces(&p);		      /* st_nlink */
+   skip_nonspaces(&p);                /* st_nlink */
    p++;
-   skip_nonspaces(&p);		      /* st_uid */
+   skip_nonspaces(&p);                /* st_uid */
    p++;
-   skip_nonspaces(&p);		      /* st_gid */
+   skip_nonspaces(&p);                /* st_gid */
    p++;
-   skip_nonspaces(&p);		      /* st_rdev */
+   skip_nonspaces(&p);                /* st_rdev */
    p++;
-   skip_nonspaces(&p);		      /* st_size */
+   skip_nonspaces(&p);                /* st_size */
    p++;
-   skip_nonspaces(&p);		      /* st_blksize */
+   skip_nonspaces(&p);                /* st_blksize */
    p++;
-   skip_nonspaces(&p);		      /* st_blocks */
+   skip_nonspaces(&p);                /* st_blocks */
    p++;
-   skip_nonspaces(&p);		      /* st_atime */
+   skip_nonspaces(&p);                /* st_atime */
    p++;
-   skip_nonspaces(&p);		      /* st_mtime */
+   skip_nonspaces(&p);                /* st_mtime */
    p++;
-   skip_nonspaces(&p);		      /* st_ctime */
+   skip_nonspaces(&p);                /* st_ctime */
 
    /* Optional FileIndex of hard linked file data */
    if (*p == ' ' || (*p != 0 && *(p+1) == ' ')) {
@@ -289,15 +287,15 @@ int32_t decode_LinkFI(char *buf, struct stat *statp)
 /*
  * Set file modes, permissions and times
  *
- *  fname is the original filename  
+ *  fname is the original filename
  *  ofile is the output filename (may be in a different directory)
  *
  * Returns:  true  on success
- *	     false on failure
+ *           false on failure
  */
 bool set_attributes(JCR *jcr, ATTR *attr, BFILE *ofd)
 {
-   struct utimbuf ut;	 
+   struct utimbuf ut;
    mode_t old_mask;
    bool ok = true;
    off_t fsize;
@@ -306,7 +304,7 @@ bool set_attributes(JCR *jcr, ATTR *attr, BFILE *ofd)
    if (attr->stream == STREAM_UNIX_ATTRIBUTES_EX &&
        set_win32_attributes(jcr, attr, ofd)) {
        if (is_bopen(ofd)) {
-	   bclose(ofd); 
+           bclose(ofd);
        }
        pm_strcpy(attr->ofname, "*none*");
        return true;
@@ -314,7 +312,7 @@ bool set_attributes(JCR *jcr, ATTR *attr, BFILE *ofd)
    if (attr->data_stream == STREAM_WIN32_DATA ||
        attr->data_stream == STREAM_WIN32_GZIP_DATA) {
       if (is_bopen(ofd)) {
-	 bclose(ofd); 
+         bclose(ofd);
       }
       pm_strcpy(attr->ofname, "*none*");
       return true;
@@ -323,7 +321,7 @@ bool set_attributes(JCR *jcr, ATTR *attr, BFILE *ofd)
 
    /*
     * If Windows stuff failed, e.g. attempt to restore Unix file
-    *  to Windows, simply fall through and we will do it the	 
+    *  to Windows, simply fall through and we will do it the
     *  universal way.
     */
 #endif
@@ -331,12 +329,12 @@ bool set_attributes(JCR *jcr, ATTR *attr, BFILE *ofd)
    old_mask = umask(0);
    if (is_bopen(ofd)) {
       char ec1[50], ec2[50];
-      fsize = blseek(ofd, 0, SEEK_CUR);
-      bclose(ofd);		      /* first close file */
-      if (fsize > 0 && fsize != attr->statp.st_size) {
+      fsize = blseek(ofd, 0, SEEK_END);
+      bclose(ofd);                    /* first close file */
+      if (fsize > 0 && fsize != (off_t)attr->statp.st_size) {
          Jmsg3(jcr, M_ERROR, 0, _("File size of restored file %s not correct. Original %s, restored %s.\n"),
-	    attr->ofname, edit_uint64(attr->statp.st_size, ec1),
-	    edit_uint64(fsize, ec2));
+            attr->ofname, edit_uint64(attr->statp.st_size, ec1),
+            edit_uint64(fsize, ec2));
       }
    }
 
@@ -344,40 +342,40 @@ bool set_attributes(JCR *jcr, ATTR *attr, BFILE *ofd)
    ut.modtime = attr->statp.st_mtime;
 
    /* ***FIXME**** optimize -- don't do if already correct */
-   /* 
+   /*
     * For link, change owner of link using lchown, but don't
-    *	try to do a chmod as that will update the file behind it.
+    *   try to do a chmod as that will update the file behind it.
     */
    if (attr->type == FT_LNK) {
       /* Change owner of link, not of real file */
       if (lchown(attr->ofname, attr->statp.st_uid, attr->statp.st_gid) < 0) {
-	 berrno be;
+         berrno be;
          Jmsg2(jcr, M_ERROR, 0, _("Unable to set file owner %s: ERR=%s\n"),
-	    attr->ofname, be.strerror());
-	 ok = false;
+            attr->ofname, be.strerror());
+         ok = false;
       }
    } else {
       if (chown(attr->ofname, attr->statp.st_uid, attr->statp.st_gid) < 0) {
-	 berrno be;
+         berrno be;
          Jmsg2(jcr, M_ERROR, 0, _("Unable to set file owner %s: ERR=%s\n"),
-	    attr->ofname, be.strerror());
-	 ok = false;
+            attr->ofname, be.strerror());
+         ok = false;
       }
       if (chmod(attr->ofname, attr->statp.st_mode) < 0) {
-	 berrno be;
+         berrno be;
          Jmsg2(jcr, M_ERROR, 0, _("Unable to set file modes %s: ERR=%s\n"),
-	    attr->ofname, be.strerror());
-	 ok = false;
+            attr->ofname, be.strerror());
+         ok = false;
       }
 
       /*
        * Reset file times.
        */
       if (utime(attr->ofname, &ut) < 0) {
-	 berrno be;
+         berrno be;
          Jmsg2(jcr, M_ERROR, 0, _("Unable to set file times %s: ERR=%s\n"),
-	    attr->ofname, be.strerror());
-	 ok = false;
+            attr->ofname, be.strerror());
+         ok = false;
       }
 #ifdef HAVE_CHFLAGS
       /*
@@ -388,10 +386,10 @@ bool set_attributes(JCR *jcr, ATTR *attr, BFILE *ofd)
        *  fail.
        */
       if (chflags(attr->ofname, attr->statp.st_flags) < 0) {
-	 berrno be;
+         berrno be;
          Jmsg2(jcr, M_ERROR, 0, _("Unable to set file flags %s: ERR=%s\n"),
-	    attr->ofname, be.strerror());
-	 ok = false;
+            attr->ofname, be.strerror());
+         ok = false;
       }
 #endif
    }
@@ -402,25 +400,38 @@ bool set_attributes(JCR *jcr, ATTR *attr, BFILE *ofd)
 
 
 /*=============================================================*/
-/*							       */
-/*		   * * *  U n i x * * * *		       */
-/*							       */
+/*                                                             */
+/*                 * * *  U n i x * * * *                      */
+/*                                                             */
 /*=============================================================*/
 
 #if !defined(HAVE_CYGWIN) && !defined(HAVE_WIN32)
-    
+
 /*
  * It is possible to piggyback additional data e.g. ACLs on
  *   the encode_stat() data by returning the extended attributes
  *   here.  They must be "self-contained" (i.e. you keep track
  *   of your own length), and they must be in ASCII string
  *   format. Using this feature is not recommended.
- * The code below shows how to return nothing.	See the Win32
+ * The code below shows how to return nothing.  See the Win32
  *   code below for returning something in the attributes.
  */
 int encode_attribsEx(JCR *jcr, char *attribsEx, FF_PKT *ff_pkt)
 {
-   *attribsEx = 0;		      /* no extended attributes */
+#ifdef HAVE_DARWIN_OS
+   /*
+    * We save the Mac resource fork length so that on a
+    * restore, we can be sure we put back the whole resource.
+    */
+   char *p;
+   p = attribsEx;
+   if (ff_pkt->flags & FO_HFSPLUS) {
+      p += to_base64((uint64_t)(ff_pkt->hfsinfo.rsrclength), p);
+   }
+   *p = 0;
+#else
+   *attribsEx = 0;                    /* no extended attributes */
+#endif
    return STREAM_UNIX_ATTRIBUTES;
 }
 
@@ -429,9 +440,9 @@ int encode_attribsEx(JCR *jcr, char *attribsEx, FF_PKT *ff_pkt)
 
 
 /*=============================================================*/
-/*							       */
-/*		   * * *  W i n 3 2 * * * *		       */
-/*							       */
+/*                                                             */
+/*                 * * *  W i n 3 2 * * * *                    */
+/*                                                             */
 /*=============================================================*/
 
 #if defined(HAVE_CYGWIN) || defined(HAVE_WIN32)
@@ -442,17 +453,34 @@ int encode_attribsEx(JCR *jcr, char *attribsEx, FF_PKT *ff_pkt)
    WIN32_FILE_ATTRIBUTE_DATA atts;
    ULARGE_INTEGER li;
 
-   attribsEx[0] = 0;		      /* no extended attributes */
+   attribsEx[0] = 0;                  /* no extended attributes */
 
-   if (!p_GetFileAttributesEx) {				 
-      return STREAM_UNIX_ATTRIBUTES;
+   // try unicode version
+   if (p_GetFileAttributesExW)  {
+      unix_name_to_win32(&ff_pkt->sys_fname, ff_pkt->fname);
+
+      POOLMEM* pwszBuf = get_pool_memory (PM_FNAME);   
+      UTF8_2_wchar(&pwszBuf, ff_pkt->sys_fname);
+
+      BOOL b=p_GetFileAttributesExW((LPCWSTR) pwszBuf, GetFileExInfoStandard, (LPVOID)&atts);
+      free_pool_memory(pwszBuf);
+
+      if (!b) {
+         win_error(jcr, "GetFileAttributesExW:", ff_pkt->sys_fname);
+         return STREAM_UNIX_ATTRIBUTES;
+      }
    }
+   else {
+      if (!p_GetFileAttributesExA)
+         return STREAM_UNIX_ATTRIBUTES;
 
-   unix_name_to_win32(&ff_pkt->sys_fname, ff_pkt->fname);
-   if (!p_GetFileAttributesEx(ff_pkt->sys_fname, GetFileExInfoStandard,
-			    (LPVOID)&atts)) {
-      win_error(jcr, "GetFileAttributesEx:", ff_pkt->sys_fname);
-      return STREAM_UNIX_ATTRIBUTES;
+      unix_name_to_win32(&ff_pkt->sys_fname, ff_pkt->fname);
+
+      if (!p_GetFileAttributesExA(ff_pkt->sys_fname, GetFileExInfoStandard,
+                              (LPVOID)&atts)) {
+         win_error(jcr, "GetFileAttributesExA:", ff_pkt->sys_fname);
+         return STREAM_UNIX_ATTRIBUTES;
+      }
    }
 
    p += to_base64((uint64_t)atts.dwFileAttributes, p);
@@ -485,17 +513,17 @@ int encode_attribsEx(JCR *jcr, char *attribsEx, FF_PKT *ff_pkt)
          FILE_ATTRIBUTE_OFFLINE| \
          FILE_ATTRIBUTE_READONLY| \
          FILE_ATTRIBUTE_SYSTEM| \
-	 FILE_ATTRIBUTE_TEMPORARY)
+         FILE_ATTRIBUTE_TEMPORARY)
 
 
 /*
  * Set Extended File Attributes for Win32
  *
- *  fname is the original filename  
+ *  fname is the original filename
  *  ofile is the output filename (may be in a different directory)
  *
  * Returns:  true  on success
- *	     false on failure
+ *           false on failure
  */
 static bool set_win32_attributes(JCR *jcr, ATTR *attr, BFILE *ofd)
 {
@@ -505,14 +533,14 @@ static bool set_win32_attributes(JCR *jcr, ATTR *attr, BFILE *ofd)
    ULARGE_INTEGER li;
    POOLMEM *win32_ofile;
 
-   if (!p_GetFileAttributesEx) {				 
+   // if we have neither ansi nor wchar version, we leave
+   if (!(p_SetFileAttributesW || p_SetFileAttributesA))
       return false;
-   }
 
-   if (!p || !*p) {		      /* we should have attributes */
+   if (!p || !*p) {                   /* we should have attributes */
       Dmsg2(100, "Attributes missing. of=%s ofd=%d\n", attr->ofname, ofd->fid);
       if (is_bopen(ofd)) {
-	 bclose(ofd);
+         bclose(ofd);
       }
       return false;
    } else {
@@ -521,22 +549,22 @@ static bool set_win32_attributes(JCR *jcr, ATTR *attr, BFILE *ofd)
 
    p += from_base64(&val, p);
    plug(atts.dwFileAttributes, val);
-   p++; 			      /* skip space */
+   p++;                               /* skip space */
    p += from_base64(&val, p);
    li.QuadPart = val;
    atts.ftCreationTime.dwLowDateTime = li.LowPart;
    atts.ftCreationTime.dwHighDateTime = li.HighPart;
-   p++; 			      /* skip space */
+   p++;                               /* skip space */
    p += from_base64(&val, p);
    li.QuadPart = val;
    atts.ftLastAccessTime.dwLowDateTime = li.LowPart;
    atts.ftLastAccessTime.dwHighDateTime = li.HighPart;
-   p++; 			      /* skip space */
+   p++;                               /* skip space */
    p += from_base64(&val, p);
    li.QuadPart = val;
    atts.ftLastWriteTime.dwLowDateTime = li.LowPart;
    atts.ftLastWriteTime.dwHighDateTime = li.HighPart;
-   p++;   
+   p++;
    p += from_base64(&val, p);
    plug(atts.nFileSizeHigh, val);
    p++;
@@ -553,24 +581,37 @@ static bool set_win32_attributes(JCR *jcr, ATTR *attr, BFILE *ofd)
 
    if (!is_bopen(ofd)) {
       Dmsg1(100, "File not open: %s\n", attr->ofname);
-      bopen(ofd, attr->ofname, O_WRONLY|O_BINARY, 0);	/* attempt to open the file */
+      bopen(ofd, attr->ofname, O_WRONLY|O_BINARY, 0);   /* attempt to open the file */
    }
 
    if (is_bopen(ofd)) {
       Dmsg1(100, "SetFileTime %s\n", attr->ofname);
       if (!SetFileTime(bget_handle(ofd),
-			 &atts.ftCreationTime,
-			 &atts.ftLastAccessTime,
-			 &atts.ftLastWriteTime)) {
+                         &atts.ftCreationTime,
+                         &atts.ftLastAccessTime,
+                         &atts.ftLastWriteTime)) {
          win_error(jcr, "SetFileTime:", win32_ofile);
       }
       bclose(ofd);
    }
 
    Dmsg1(100, "SetFileAtts %s\n", attr->ofname);
-   if (!(atts.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
-      if (!SetFileAttributes(win32_ofile, atts.dwFileAttributes & SET_ATTRS)) {
-         win_error(jcr, "SetFileAttributes:", win32_ofile);
+   if (!(atts.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) 
+   {
+      if (p_SetFileAttributesW) {
+         POOLMEM* pwszBuf = get_pool_memory (PM_FNAME);   
+         UTF8_2_wchar(&pwszBuf, win32_ofile);
+
+         BOOL b=p_SetFileAttributesW((LPCWSTR)pwszBuf, atts.dwFileAttributes & SET_ATTRS);
+         free_pool_memory(pwszBuf);
+      
+         if (!b) 
+            win_error(jcr, "SetFileAttributesW:", win32_ofile); 
+      }
+      else {
+         if (!p_SetFileAttributesA(win32_ofile, atts.dwFileAttributes & SET_ATTRS)) {
+            win_error(jcr, "SetFileAttributesA:", win32_ofile);
+         }
       }
    }
    free_pool_memory(win32_ofile);
@@ -582,13 +623,13 @@ void win_error(JCR *jcr, char *prefix, POOLMEM *win32_ofile)
    DWORD lerror = GetLastError();
    LPTSTR msg;
    FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|
-		 FORMAT_MESSAGE_FROM_SYSTEM,
-		 NULL,
-		 lerror,
-		 0,
-		 (LPTSTR)&msg,
-		 0,
-		 NULL);
+                 FORMAT_MESSAGE_FROM_SYSTEM,
+                 NULL,
+                 lerror,
+                 0,
+                 (LPTSTR)&msg,
+                 0,
+                 NULL);
    Dmsg3(100, "Error in %s on file %s: ERR=%s\n", prefix, win32_ofile, msg);
    strip_trailing_junk(msg);
    Jmsg(jcr, M_ERROR, 0, _("Error in %s file %s: ERR=%s\n"), prefix, win32_ofile, msg);
@@ -599,13 +640,13 @@ void win_error(JCR *jcr, char *prefix, DWORD lerror)
 {
    LPTSTR msg;
    FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|
-		 FORMAT_MESSAGE_FROM_SYSTEM,
-		 NULL,
-		 lerror,
-		 0,
-		 (LPTSTR)&msg,
-		 0,
-		 NULL);
+                 FORMAT_MESSAGE_FROM_SYSTEM,
+                 NULL,
+                 lerror,
+                 0,
+                 (LPTSTR)&msg,
+                 0,
+                 NULL);
    strip_trailing_junk(msg);
    if (jcr) {
       Jmsg2(jcr, M_ERROR, 0, _("Error in %s: ERR=%s\n"), prefix, msg);
@@ -616,14 +657,15 @@ void win_error(JCR *jcr, char *prefix, DWORD lerror)
 }
 
 
-/* Cygwin API definition */
-extern "C" void cygwin_conv_to_win32_path(const char *path, char *win32_path);
-
+/* Conversion of a Unix filename to a Win32 filename */
+extern void conv_unix_to_win32_path(const char *path, char *win32_path, DWORD dwSize);
 void unix_name_to_win32(POOLMEM **win32_name, char *name)
 {
    /* One extra byte should suffice, but we double it */
-   *win32_name = check_pool_memory_size(*win32_name, 2*strlen(name)+1);
-   cygwin_conv_to_win32_path(name, *win32_name);
+   /* add MAX_PATH bytes for VSS shadow copy name */
+   DWORD dwSize = 2*strlen(name)+MAX_PATH;
+   *win32_name = check_pool_memory_size(*win32_name, dwSize);
+   conv_unix_to_win32_path(name, *win32_name, dwSize);
 }
 
-#endif	/* HAVE_CYGWIN */
+#endif  /* HAVE_CYGWIN */
