@@ -1,40 +1,41 @@
 /*
-   Copyright (C) 2000-20054 Kern Sibbald
+   Copyright (C) 2000-2005 Kern Sibbald
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of
-   the License, or (at your option) any later version.
+   modify it under the terms of the GNU General Public License
+   version 2 as amended with additional clauses defined in the
+   file LICENSE in the main source directory.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+   the file LICENSE for additional details.
 
-   You should have received a copy of the GNU General Public
-   License along with this program; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+ */
+
+/*
 
    This file is based on GNU TAR source code. Except for a few key
-   ideas, it has been rewritten for Bacula.
+   ideas, it has been entirely rewritten for Bacula.
 
       Kern Sibbald, MM
 
    Thanks to the TAR programmers.
 
-     Version $Id: find_one.c,v 1.34.10.1 2005/02/14 10:02:24 kerns Exp $
+     Version $Id: find_one.c,v 1.49.2.1 2005/12/10 13:18:03 kerns Exp $
 
  */
 
 #include "bacula.h"
 #include "find.h"
 #ifdef HAVE_DARWIN_OS
+#include <sys/param.h>
+#include <sys/mount.h>
 #include <sys/attr.h>
 #endif
 
-extern int32_t name_max;	      /* filename max length */
-extern int32_t path_max;	      /* path name max length */
+extern int32_t name_max;              /* filename max length */
+extern int32_t path_max;              /* path name max length */
 
 /*
  * Structure for keeping track of hard linked files, we
@@ -45,11 +46,11 @@ extern int32_t path_max;	      /* path name max length */
  */
 struct f_link {
     struct f_link *next;
-    dev_t dev;			      /* device */
-    ino_t ino;			      /* inode with device is unique */
+    dev_t dev;                        /* device */
+    ino_t ino;                        /* inode with device is unique */
     short linkcount;
-    uint32_t FileIndex; 	      /* Bacula FileIndex of this file */
-    char name[1];		      /* The name */
+    uint32_t FileIndex;               /* Bacula FileIndex of this file */
+    char name[1];                     /* The name */
 };
 
 static void free_dir_ff_pkt(FF_PKT *dir_ff_pkt)
@@ -74,20 +75,53 @@ static int accept_fstype(FF_PKT *ff, void *dummy) {
       if (!fstype(ff->fname, fs, sizeof(fs))) {
          Dmsg1(50, "Cannot determine file system type for \"%s\"\n", ff->fname);
       } else {
-	 for (i = 0; i < ff->fstypes.size(); ++i) {
-	    if (strcmp(fs, (char *)ff->fstypes.get(i)) == 0) {
+         for (i = 0; i < ff->fstypes.size(); ++i) {
+            if (strcmp(fs, (char *)ff->fstypes.get(i)) == 0) {
                Dmsg2(100, "Accepting fstype %s for \"%s\"\n", fs, ff->fname);
-	       accept = true;
-	       break;
-	    }
+               accept = true;
+               break;
+            }
             Dmsg3(200, "fstype %s for \"%s\" does not match %s\n", fs,
-		  ff->fname, ff->fstypes.get(i));
-	 }
+                  ff->fname, ff->fstypes.get(i));
+         }
       }
    }
    return accept;
 }
 
+/*
+ * This function determines whether we can use getattrlist()
+ * It's odd, but we have to use the function to determine that...
+ * Also, the man pages talk about things as if they were implemented.
+ *
+ * On Mac OS X, this succesfully differentiates between HFS+ and UFS
+ * volumes, which makes me trust it is OK for others, too.
+ */
+static bool volume_has_attrlist(const char *fname)
+{
+#ifdef HAVE_DARWIN_OS
+   struct statfs st;
+   struct volinfo_struct {
+      unsigned long length;               /* Mandatory field */
+      vol_capabilities_attr_t info;       /* Volume capabilities */
+   } vol;
+   struct attrlist attrList;
+
+   memset(&attrList, 0, sizeof(attrList));
+   attrList.bitmapcount = ATTR_BIT_MAP_COUNT;
+   attrList.volattr = ATTR_VOL_INFO | ATTR_VOL_CAPABILITIES;
+   if (statfs(fname, &st) == 0) {
+      /* We need to check on the mount point */
+      if (getattrlist(st.f_mntonname, &attrList, &vol, sizeof(vol), FSOPT_NOFOLLOW) == 0
+            && (vol.info.capabilities[VOL_CAPABILITIES_INTERFACES] & VOL_CAP_INT_ATTRLIST)
+            && (vol.info.valid[VOL_CAPABILITIES_INTERFACES] & VOL_CAP_INT_ATTRLIST)) {
+         return true;
+      }
+   }
+#endif
+   return false;
+}
+
 /*
  * Find a single file.
  * handle_file is the callback for handling the file.
@@ -97,8 +131,9 @@ static int accept_fstype(FF_PKT *ff, void *dummy) {
  *  descending into a directory.
  */
 int
-find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
-	       void *pkt, char *fname, dev_t parent_device, int top_level)
+find_one_file(JCR *jcr, FF_PKT *ff_pkt, 
+               int handle_file(FF_PKT *ff, void *hpkt, bool top_level),
+               void *pkt, char *fname, dev_t parent_device, bool top_level)
 {
    struct utimbuf restore_times;
    int rtn_stat;
@@ -110,7 +145,7 @@ find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
        /* Cannot stat file */
        ff_pkt->type = FT_NOSTAT;
        ff_pkt->ff_errno = errno;
-       return handle_file(ff_pkt, pkt);
+       return handle_file(ff_pkt, pkt, top_level);
    }
 
    Dmsg1(300, "File ----: %s\n", fname);
@@ -124,13 +159,16 @@ find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
    /*
     * We check for allowed fstypes at top_level and fstype change (below).
     */
-   if (top_level && !accept_fstype(ff_pkt, NULL)) {
-      ff_pkt->type = FT_INVALIDFS;
-      if (ff_pkt->flags & FO_KEEPATIME) {
-	 utime(fname, &restore_times);
+   if (top_level) {
+      if (!accept_fstype(ff_pkt, NULL)) {
+         ff_pkt->type = FT_INVALIDFS;
+         if (ff_pkt->flags & FO_KEEPATIME) {
+            utime(fname, &restore_times);
+         }
+         Jmsg1(jcr, M_ERROR, 0, _("Top level directory \"%s\" has an unlisted fstype\n"), fname);
+         return 1;      /* Just ignore this error - or the whole backup is cancelled */
       }
-      Jmsg1(jcr, M_ERROR, 0, _("Top level directory \"%s\" has an unlisted fstype\n"), fname);
-      return 1;      /* Just ignore this error - or the whole backup is cancelled */
+      ff_pkt->volhas_attrlist = volume_has_attrlist(fname);
    }
 
    /*
@@ -142,16 +180,17 @@ find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
       Dmsg1(300, "Non-directory incremental: %s\n", ff_pkt->fname);
       /* Not a directory */
       if (ff_pkt->statp.st_mtime < ff_pkt->save_time
-	  && ((ff_pkt->flags & FO_MTIMEONLY) ||
-	      ff_pkt->statp.st_ctime < ff_pkt->save_time)) {
-	 /* Incremental option, file not changed */
-	 ff_pkt->type = FT_NOCHG;
-	 return handle_file(ff_pkt, pkt);
+          && ((ff_pkt->flags & FO_MTIMEONLY) ||
+              ff_pkt->statp.st_ctime < ff_pkt->save_time)) {
+         /* Incremental option, file not changed */
+         ff_pkt->type = FT_NOCHG;
+         return handle_file(ff_pkt, pkt, top_level);
       }
    }
 
 #ifdef HAVE_DARWIN_OS
-   if (S_ISREG(ff_pkt->statp.st_mode) && ff_pkt->flags & FO_HFSPLUS) {
+   if (ff_pkt->flags & FO_HFSPLUS && ff_pkt->volhas_attrlist
+         && S_ISREG(ff_pkt->statp.st_mode)) {
        /* TODO: initialise attrList once elsewhere? */
        struct attrlist attrList;
        memset(&attrList, 0, sizeof(attrList));
@@ -159,10 +198,10 @@ find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
        attrList.commonattr = ATTR_CMN_FNDRINFO;
        attrList.fileattr = ATTR_FILE_RSRCLENGTH;
        if (getattrlist(fname, &attrList, &ff_pkt->hfsinfo,
-		sizeof(ff_pkt->hfsinfo), 0) != 0) {
-	  ff_pkt->type = FT_NOSTAT;
-	  ff_pkt->ff_errno = errno;
-	  return handle_file(ff_pkt, pkt);
+                sizeof(ff_pkt->hfsinfo), FSOPT_NOFOLLOW) != 0) {
+          ff_pkt->type = FT_NOSTAT;
+          ff_pkt->ff_errno = errno;
+          return handle_file(ff_pkt, pkt, top_level);
        }
    }
 #endif
@@ -172,7 +211,7 @@ find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
    /* See if we are trying to dump the archive.  */
    if (ar_dev && ff_pkt->statp.st_dev == ar_dev && ff_pkt->statp.st_ino == ar_ino) {
        ff_pkt->type = FT_ISARCH;
-       return handle_file(ff_pkt, pkt);
+       return handle_file(ff_pkt, pkt, top_level);
    }
 #endif
    ff_pkt->LinkFI = 0;
@@ -186,28 +225,26 @@ find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
    if (!(ff_pkt->flags & FO_NO_HARDLINK)
        && ff_pkt->statp.st_nlink > 1
        && (S_ISREG(ff_pkt->statp.st_mode)
-	   || S_ISCHR(ff_pkt->statp.st_mode)
-	   || S_ISBLK(ff_pkt->statp.st_mode)
-	   || S_ISFIFO(ff_pkt->statp.st_mode)
-	   || S_ISSOCK(ff_pkt->statp.st_mode))) {
+           || S_ISCHR(ff_pkt->statp.st_mode)
+           || S_ISBLK(ff_pkt->statp.st_mode)
+           || S_ISFIFO(ff_pkt->statp.st_mode)
+           || S_ISSOCK(ff_pkt->statp.st_mode))) {
 
        struct f_link *lp;
 
       /* Search link list of hard linked files */
       for (lp = ff_pkt->linklist; lp; lp = lp->next)
-	 if (lp->ino == (ino_t)ff_pkt->statp.st_ino &&
-	     lp->dev == (dev_t)ff_pkt->statp.st_dev) {
+         if (lp->ino == (ino_t)ff_pkt->statp.st_ino &&
+             lp->dev == (dev_t)ff_pkt->statp.st_dev) {
              /* If we have already backed up the hard linked file don't do it again */
-	     if (strcmp(lp->name, fname) == 0) {
-                Jmsg1(jcr, M_WARNING, 0, _("Attempt to backup hard linked file %s twice ignored.\n"),
-		   fname);
-		return 1;	      /* ignore */
-	     }
-	     ff_pkt->link = lp->name;
-	     ff_pkt->type = FT_LNKSAVED;       /* Handle link, file already saved */
-	     ff_pkt->LinkFI = lp->FileIndex;
-	     return handle_file(ff_pkt, pkt);
-	 }
+             if (strcmp(lp->name, fname) == 0) {
+                return 1;             /* ignore */
+             }
+             ff_pkt->link = lp->name;
+             ff_pkt->type = FT_LNKSAVED;       /* Handle link, file already saved */
+             ff_pkt->LinkFI = lp->FileIndex;
+             return handle_file(ff_pkt, pkt, top_level);
+         }
 
       /* File not previously dumped. Chain it into our list. */
       len = strlen(fname) + 1;
@@ -217,7 +254,7 @@ find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
       bstrncpy(lp->name, fname, len);
       lp->next = ff_pkt->linklist;
       ff_pkt->linklist = lp;
-      ff_pkt->linked = lp;	      /* mark saved link */
+      ff_pkt->linked = lp;            /* mark saved link */
    } else {
       ff_pkt->linked = NULL;
    }
@@ -229,17 +266,20 @@ find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
       sizeleft = ff_pkt->statp.st_size;
 
       /* Don't bother opening empty, world readable files.  Also do not open
-	 files when archive is meant for /dev/null.  */
+         files when archive is meant for /dev/null.  */
       if (ff_pkt->null_output_device || (sizeleft == 0
-	      && MODE_RALL == (MODE_RALL & ff_pkt->statp.st_mode))) {
-	 ff_pkt->type = FT_REGE;
+              && MODE_RALL == (MODE_RALL & ff_pkt->statp.st_mode))) {
+         ff_pkt->type = FT_REGE;
       } else {
-	 ff_pkt->type = FT_REG;
+         ff_pkt->type = FT_REG;
       }
-      rtn_stat = handle_file(ff_pkt, pkt);
+      rtn_stat = handle_file(ff_pkt, pkt, top_level);
       if (ff_pkt->linked) {
-	 ff_pkt->linked->FileIndex = ff_pkt->FileIndex;
+         ff_pkt->linked->FileIndex = ff_pkt->FileIndex;
       }
+      if (ff_pkt->flags & FO_KEEPATIME) {
+         utime(fname, &restore_times);
+      }       
       return rtn_stat;
 
 
@@ -249,21 +289,21 @@ find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
 
       size = readlink(fname, buffer, path_max + name_max + 101);
       if (size < 0) {
-	 /* Could not follow link */
-	 ff_pkt->type = FT_NOFOLLOW;
-	 ff_pkt->ff_errno = errno;
-	 rtn_stat = handle_file(ff_pkt, pkt);
-	 if (ff_pkt->linked) {
-	    ff_pkt->linked->FileIndex = ff_pkt->FileIndex;
-	 }
-	 return rtn_stat;
+         /* Could not follow link */
+         ff_pkt->type = FT_NOFOLLOW;
+         ff_pkt->ff_errno = errno;
+         rtn_stat = handle_file(ff_pkt, pkt, top_level);
+         if (ff_pkt->linked) {
+            ff_pkt->linked->FileIndex = ff_pkt->FileIndex;
+         }
+         return rtn_stat;
       }
       buffer[size] = 0;
-      ff_pkt->link = buffer;	      /* point to link */
-      ff_pkt->type = FT_LNK;	      /* got a real link */
-      rtn_stat = handle_file(ff_pkt, pkt);
+      ff_pkt->link = buffer;          /* point to link */
+      ff_pkt->type = FT_LNK;          /* got a real link */
+      rtn_stat = handle_file(ff_pkt, pkt, top_level);
       if (ff_pkt->linked) {
-	 ff_pkt->linked->FileIndex = ff_pkt->FileIndex;
+         ff_pkt->linked->FileIndex = ff_pkt->FileIndex;
       }
       return rtn_stat;
 
@@ -276,6 +316,7 @@ find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
       int status;
       dev_t our_device = ff_pkt->statp.st_dev;
       bool recurse = true;
+      bool volhas_attrlist = ff_pkt->volhas_attrlist;    /* Remember this if we recurse */
 
       /*
        * If we are using Win32 (non-portable) backup API, don't check
@@ -283,16 +324,16 @@ find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
        *  in principle, we should be able to access everything.
        */
       if (!have_win32_api() || (ff_pkt->flags & FO_PORTABLE)) {
-	 if (access(fname, R_OK) == -1 && geteuid() != 0) {
-	    /* Could not access() directory */
-	    ff_pkt->type = FT_NOACCESS;
-	    ff_pkt->ff_errno = errno;
-	    rtn_stat = handle_file(ff_pkt, pkt);
-	    if (ff_pkt->linked) {
-	       ff_pkt->linked->FileIndex = ff_pkt->FileIndex;
-	    }
-	    return rtn_stat;
-	 }
+         if (access(fname, R_OK) == -1 && geteuid() != 0) {
+            /* Could not access() directory */
+            ff_pkt->type = FT_NOACCESS;
+            ff_pkt->ff_errno = errno;
+            rtn_stat = handle_file(ff_pkt, pkt, top_level);
+            if (ff_pkt->linked) {
+               ff_pkt->linked->FileIndex = ff_pkt->FileIndex;
+            }
+            return rtn_stat;
+         }
       }
 
       /* Build a canonical directory name with a trailing slash in link var */
@@ -302,18 +343,19 @@ find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
       bstrncpy(link, fname, link_len);
       /* Strip all trailing slashes */
       while (len >= 1 && link[len - 1] == '/')
-	len--;
+        len--;
       link[len++] = '/';             /* add back one */
       link[len] = 0;
 
       ff_pkt->link = link;
       if (ff_pkt->incremental &&
-	  (ff_pkt->statp.st_mtime < ff_pkt->save_time &&
-	   ff_pkt->statp.st_ctime < ff_pkt->save_time)) {
-	 /* Incremental option, directory entry not changed */
-	 ff_pkt->type = FT_DIRNOCHG;
+          (ff_pkt->statp.st_mtime < ff_pkt->save_time &&
+             ((ff_pkt->flags & FO_MTIMEONLY) ||
+               ff_pkt->statp.st_ctime < ff_pkt->save_time))) {
+         /* Incremental option, directory entry not changed */
+         ff_pkt->type = FT_DIRNOCHG;
       } else {
-	 ff_pkt->type = FT_DIRBEGIN;
+         ff_pkt->type = FT_DIRBEGIN;
       }
       /*
        * Note, we return the directory to the calling program (handle_file)
@@ -323,14 +365,14 @@ find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
        * do not immediately save it, but do so only after everything
        * in the directory is seen (i.e. the FT_DIREND).
        */
-      rtn_stat = handle_file(ff_pkt, pkt);
-      if (rtn_stat < 1) {	      /* ignore or error status */
-	 free(link);
-	 return rtn_stat;
+      rtn_stat = handle_file(ff_pkt, pkt, top_level);
+      if (rtn_stat < 1) {             /* ignore or error status */
+         free(link);
+         return rtn_stat;
       }
       /* Done with DIRBEGIN, next call will be DIREND */
       if (ff_pkt->type == FT_DIRBEGIN) {
-	 ff_pkt->type = FT_DIREND;
+         ff_pkt->type = FT_DIREND;
       }
 
       /*
@@ -360,30 +402,33 @@ find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
        * to cross, or we may be restricted by a list of permitted
        * file systems.
        */
-      if (ff_pkt->flags & FO_NO_RECURSION) {
-	 ff_pkt->type = FT_NORECURSE;
-	 recurse = false;
+      if (!top_level && ff_pkt->flags & FO_NO_RECURSION) {
+         ff_pkt->type = FT_NORECURSE;
+         recurse = false;
       } else if (!top_level && parent_device != ff_pkt->statp.st_dev) {
-	 if(!(ff_pkt->flags & FO_MULTIFS)) {
-	    ff_pkt->type = FT_NOFSCHG;
-	    recurse = false;
-	 } else if (!accept_fstype(ff_pkt, NULL)) {
-	    ff_pkt->type = FT_INVALIDFS;
-	    recurse = false;
-	 }
+         if(!(ff_pkt->flags & FO_MULTIFS)) {
+            ff_pkt->type = FT_NOFSCHG;
+            recurse = false;
+         } else if (!accept_fstype(ff_pkt, NULL)) {
+            ff_pkt->type = FT_INVALIDFS;
+            recurse = false;
+         } else {
+            ff_pkt->volhas_attrlist = volume_has_attrlist(fname);
+         }
       }
+      /* If not recursing, just backup dir and return */
       if (!recurse) {
-	 rtn_stat = handle_file(ff_pkt, pkt);
-	 if (ff_pkt->linked) {
-	    ff_pkt->linked->FileIndex = ff_pkt->FileIndex;
-	 }
-	 free(link);
-	 free_dir_ff_pkt(dir_ff_pkt);
+         rtn_stat = handle_file(ff_pkt, pkt, top_level);
+         if (ff_pkt->linked) {
+            ff_pkt->linked->FileIndex = ff_pkt->FileIndex;
+         }
+         free(link);
+         free_dir_ff_pkt(dir_ff_pkt);
          ff_pkt->link = ff_pkt->fname;     /* reset "link" */
-	 if (ff_pkt->flags & FO_KEEPATIME) {
-	    utime(fname, &restore_times);
-	 }
-	 return rtn_stat;
+         if (ff_pkt->flags & FO_KEEPATIME) {
+            utime(fname, &restore_times);
+         }
+         return rtn_stat;
       }
 
       ff_pkt->link = ff_pkt->fname;     /* reset "link" */
@@ -394,15 +439,15 @@ find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
        */
       errno = 0;
       if ((directory = opendir(fname)) == NULL) {
-	 ff_pkt->type = FT_NOOPEN;
-	 ff_pkt->ff_errno = errno;
-	 rtn_stat = handle_file(ff_pkt, pkt);
-	 if (ff_pkt->linked) {
-	    ff_pkt->linked->FileIndex = ff_pkt->FileIndex;
-	 }
-	 free(link);
-	 free_dir_ff_pkt(dir_ff_pkt);
-	 return rtn_stat;
+         ff_pkt->type = FT_NOOPEN;
+         ff_pkt->ff_errno = errno;
+         rtn_stat = handle_file(ff_pkt, pkt, top_level);
+         if (ff_pkt->linked) {
+            ff_pkt->linked->FileIndex = ff_pkt->FileIndex;
+         }
+         free(link);
+         free_dir_ff_pkt(dir_ff_pkt);
+         return rtn_stat;
       }
 
       /*
@@ -413,38 +458,38 @@ find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
       rtn_stat = 1;
       entry = (struct dirent *)malloc(sizeof(struct dirent) + name_max + 100);
       for ( ; !job_canceled(jcr); ) {
-	 char *p, *q;
-	 int i;
+         char *p, *q;
+         int i;
 
-	 status  = readdir_r(directory, entry, &result);
-	 if (status != 0 || result == NULL) {
+         status  = readdir_r(directory, entry, &result);
+         if (status != 0 || result == NULL) {
 //          Dmsg2(99, "readdir returned stat=%d result=0x%x\n",
-//	       status, (long)result);
-	    break;
-	 }
-	 ASSERT(name_max+1 > (int)sizeof(struct dirent) + (int)NAMELEN(entry));
-	 p = entry->d_name;
+//             status, (long)result);
+            break;
+         }
+         ASSERT(name_max+1 > (int)sizeof(struct dirent) + (int)NAMELEN(entry));
+         p = entry->d_name;
          /* Skip `.', `..', and excluded file names.  */
          if (p[0] == '\0' || (p[0] == '.' && (p[1] == '\0' ||
              (p[1] == '.' && p[2] == '\0')))) {
-	    continue;
-	 }
+            continue;
+         }
 
-	 if ((int)NAMELEN(entry) + len >= link_len) {
-	     link_len = len + NAMELEN(entry) + 1;
-	     link = (char *)brealloc(link, link_len + 1);
-	 }
-	 q = link + len;
-	 for (i=0; i < (int)NAMELEN(entry); i++) {
-	    *q++ = *p++;
-	 }
-	 *q = 0;
-	 if (!file_is_excluded(ff_pkt, link)) {
-	    rtn_stat = find_one_file(jcr, ff_pkt, handle_file, pkt, link, our_device, 0);
-	    if (ff_pkt->linked) {
-	       ff_pkt->linked->FileIndex = ff_pkt->FileIndex;
-	    }
-	 }
+         if ((int)NAMELEN(entry) + len >= link_len) {
+             link_len = len + NAMELEN(entry) + 1;
+             link = (char *)brealloc(link, link_len + 1);
+         }
+         q = link + len;
+         for (i=0; i < (int)NAMELEN(entry); i++) {
+            *q++ = *p++;
+         }
+         *q = 0;
+         if (!file_is_excluded(ff_pkt, link)) {
+            rtn_stat = find_one_file(jcr, ff_pkt, handle_file, pkt, link, our_device, false);
+            if (ff_pkt->linked) {
+               ff_pkt->linked->FileIndex = ff_pkt->FileIndex;
+            }
+         }
       }
       closedir(directory);
       free(link);
@@ -457,15 +502,16 @@ find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
        *  the directory modes and dates.  Temp directory values
        *  were used without this record.
        */
-      handle_file(dir_ff_pkt, pkt);	  /* handle directory entry */
+      handle_file(dir_ff_pkt, pkt, top_level);       /* handle directory entry */
       if (ff_pkt->linked) {
-	 ff_pkt->linked->FileIndex = dir_ff_pkt->FileIndex;
+         ff_pkt->linked->FileIndex = dir_ff_pkt->FileIndex;
       }
       free_dir_ff_pkt(dir_ff_pkt);
 
       if (ff_pkt->flags & FO_KEEPATIME) {
-	 utime(fname, &restore_times);
+         utime(fname, &restore_times);
       }
+      ff_pkt->volhas_attrlist = volhas_attrlist;      /* Restore value in case it changed. */
       return rtn_stat;
    } /* end check for directory */
 
@@ -477,8 +523,8 @@ find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
 #ifdef HAVE_FREEBSD_OS
    /*
     * On FreeBSD, all block devices are character devices, so
-    *	to be able to read a raw disk, we need the check for
-    *	a character device.
+    *   to be able to read a raw disk, we need the check for
+    *   a character device.
  * crw-r----- 1 root operator - 116, 0x00040002 Jun 9 19:32 /dev/ad0s3
  * crw-r----- 1 root operator - 116, 0x00040002 Jun 9 19:32 /dev/rad0s3
     */
@@ -486,15 +532,15 @@ find_one_file(JCR *jcr, FF_PKT *ff_pkt, int handle_file(FF_PKT *ff, void *hpkt),
 #else
    if (top_level && S_ISBLK(ff_pkt->statp.st_mode)) {
 #endif
-      ff_pkt->type = FT_RAW;	      /* raw partition */
+      ff_pkt->type = FT_RAW;          /* raw partition */
    } else if (top_level && S_ISFIFO(ff_pkt->statp.st_mode) &&
-	      ff_pkt->flags & FO_READFIFO) {
+              ff_pkt->flags & FO_READFIFO) {
       ff_pkt->type = FT_FIFO;
    } else {
       /* The only remaining types are special (character, ...) files */
       ff_pkt->type = FT_SPEC;
    }
-   rtn_stat = handle_file(ff_pkt, pkt);
+   rtn_stat = handle_file(ff_pkt, pkt, top_level);
    if (ff_pkt->linked) {
       ff_pkt->linked->FileIndex = ff_pkt->FileIndex;
    }
@@ -511,9 +557,10 @@ int term_find_one(FF_PKT *ff)
       lc = lp;
       lp = lp->next;
       if (lc) {
-	 free(lc);
-	 count++;
+         free(lc);
+         count++;
       }
    }
+   ff->linklist = NULL;
    return count;
 }
