@@ -3,26 +3,20 @@
  *
  *    Kern Sibbald, December MMII
  *
- *   Version $Id: edit.c,v 1.20.4.1 2005/02/14 10:02:25 kerns Exp $
+ *   Version $Id: edit.c,v 1.28.2.1 2006/03/04 11:10:18 kerns Exp $
  */
-
 /*
-   Copyright (C) 2000-2005 Kern Sibbald
+   Copyright (C) 2002-2005 Kern Sibbald
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of
-   the License, or (at your option) any later version.
+   modify it under the terms of the GNU General Public License
+   version 2 as amended with additional clauses defined in the
+   file LICENSE in the main source directory.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public
-   License along with this program; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+   the file LICENSE for additional details.
 
  */
 
@@ -45,7 +39,7 @@ uint64_t str_to_uint64(char *str)
       p++;
    }
    while (B_ISDIGIT(*p)) {
-      value = value * 10 + *p - '0';
+      value = B_TIMES10(value) + *p - '0';
       p++;
    }
    return value;
@@ -89,19 +83,52 @@ char *edit_uint64_with_commas(uint64_t val, char *buf)
     */
    char mbuf[50];
    mbuf[sizeof(mbuf)-1] = 0;
-   int i = sizeof(mbuf)-2;		   /* edit backward */
+   int i = sizeof(mbuf)-2;                 /* edit backward */
    if (val == 0) {
       mbuf[i--] = '0';
    } else {
       while (val != 0) {
          mbuf[i--] = "0123456789"[val%10];
-	 val /= 10;
+         val /= 10;
       }
    }
-   strcpy(buf, &mbuf[i+1]);
+   bstrncpy(buf, &mbuf[i+1], 27);
    return add_commas(buf, buf);
 }
 
+/*
+ * Edit an integer into "human-readable" format with four or fewer
+ * significant digits followed by a suffix that indicates the scale
+ * factor.  The buf array inherits a 27 byte minimim length
+ * requirement from edit_unit64_with_commas(), although the output
+ * string is limited to eight characters.
+ */
+char *edit_uint64_with_suffix(uint64_t val, char *buf)
+{
+  int commas = 0;
+  char *c, mbuf[50];
+  char *suffix[] =
+    { "", "K", "M", "G", "T", "P", "E", "Z", "Y", "FIX ME" };
+  int suffixes = sizeof(suffix) / sizeof(*suffix);
+
+  edit_uint64_with_commas(val, mbuf);
+
+  if ((c = strchr(mbuf, ',')) != NULL) {
+    commas++;
+    *c++ = '.';
+    while  ((c = strchr(c, ',')) != NULL) {
+      commas++;
+      *c++ = '\0';
+    }
+    mbuf[5] = '\0'; // drop this to get '123.456 TB' rather than '123.4 TB'
+  }
+
+  if (commas >= suffixes)
+    commas = suffixes - 1;
+  bsnprintf(buf, 27, "%s %s", mbuf, suffix[commas]);
+  return buf;
+}
+
 /*
  * Edit an integer number, the supplied buffer
  * must be at least 27 bytes long.  The incoming number
@@ -114,16 +141,16 @@ char *edit_uint64(uint64_t val, char *buf)
     */
    char mbuf[50];
    mbuf[sizeof(mbuf)-1] = 0;
-   int i = sizeof(mbuf)-2;		   /* edit backward */
+   int i = sizeof(mbuf)-2;                 /* edit backward */
    if (val == 0) {
       mbuf[i--] = '0';
    } else {
       while (val != 0) {
          mbuf[i--] = "0123456789"[val%10];
-	 val /= 10;
+         val /= 10;
       }
    }
-   strcpy(buf, &mbuf[i+1]);
+   bstrncpy(buf, &mbuf[i+1], 27);
    return buf;
 }
 
@@ -135,53 +162,49 @@ char *edit_int64(int64_t val, char *buf)
    char mbuf[50];
    bool negative = false;
    mbuf[sizeof(mbuf)-1] = 0;
-   int i = sizeof(mbuf)-2;		   /* edit backward */
+   int i = sizeof(mbuf)-2;                 /* edit backward */
    if (val == 0) {
       mbuf[i--] = '0';
    } else {
       if (val < 0) {
-	 negative = true;
-	 val = -val;
+         negative = true;
+         val = -val;
       }
       while (val != 0) {
          mbuf[i--] = "0123456789"[val%10];
-	 val /= 10;
+         val /= 10;
       }
    }
    if (negative) {
       mbuf[i--] = '-';
    }
-   strcpy(buf, &mbuf[i+1]);
+   bstrncpy(buf, &mbuf[i+1], 27);
    return buf;
 }
 
 
 /*
- * Given a string "str", separate the integer part into
+ * Given a string "str", separate the numeric part into
  *   str, and the modifier into mod.
  */
 static bool get_modifier(char *str, char *num, int num_len, char *mod, int mod_len)
 {
    int i, len, num_begin, num_end, mod_begin, mod_end;
 
-   /*
-    * Look for modifier by walking back looking for the first
-    *	space or digit.
-    */
    strip_trailing_junk(str);
    len = strlen(str);
 
    for (i=0; i<len; i++) {
       if (!B_ISSPACE(str[i])) {
-	 break;
+         break;
       }
    }
    num_begin = i;
 
    /* Walk through integer part */
    for ( ; i<len; i++) {
-      if (!B_ISDIGIT(str[i])) {
-	 break;
+      if (!B_ISDIGIT(str[i]) && str[i] != '.') {
+         break;
       }
    }
    num_end = i;
@@ -191,15 +214,16 @@ static bool get_modifier(char *str, char *num, int num_len, char *mod, int mod_l
    if (num_len == 0) {
       return false;
    }
+   /* Eat any spaces in front of modifier */
    for ( ; i<len; i++) {
       if (!B_ISSPACE(str[i])) {
-	 break;
+         break;
       }
    }
    mod_begin = i;
    for ( ; i<len; i++) {
       if (!B_ISALPHA(str[i])) {
-	 break;
+         break;
       }
    }
    mod_end = i;
@@ -214,16 +238,17 @@ static bool get_modifier(char *str, char *num, int num_len, char *mod, int mod_l
       return false;
    }
    bstrncpy(str, &str[mod_end], len);
+   Dmsg2(900, "num=%s mod=%s\n", num, mod);
 
    return true;
 }
 
 /*
  * Convert a string duration to utime_t (64 bit seconds)
- * Returns 0: if error
-	   1: if OK, and value stored in value
+ * Returns false: if error
+           true:  if OK, and value stored in value
  */
-int duration_to_utime(char *str, utime_t *value)
+bool duration_to_utime(char *str, utime_t *value)
 {
    int i, mod_len;
    double val, total = 0.0;
@@ -232,41 +257,41 @@ int duration_to_utime(char *str, utime_t *value)
    /*
     * The "n" = mins and months appears before minutes so that m maps
     *   to months. These "kludges" make it compatible with pre 1.31
-    *	Baculas.
+    *   Baculas.
     */
-   static const char *mod[] = {"n", "seconds", "months", "minutes",
+   static const char *mod[] = {"n", "seconds", "months", "minutes", "mins",
                   "hours", "days", "weeks",   "quarters",   "years", NULL};
-   static const int32_t mult[] = {60,	1, 60*60*24*30, 60,
-		  60*60, 60*60*24, 60*60*24*7, 60*60*24*91, 60*60*24*365};
+   static const int32_t mult[] = {60,   1, 60*60*24*30, 60, 60,
+                  3600, 3600*24, 3600*24*7, 3600*24*91, 3600*24*365};
 
    while (*str) {
       if (!get_modifier(str, num_str, sizeof(num_str), mod_str, sizeof(mod_str))) {
-	 return 0;
+         return false;
       }
       /* Now find the multiplier corresponding to the modifier */
       mod_len = strlen(mod_str);
       if (mod_len == 0) {
-	 i = 1; 			 /* assume seconds */
+         i = 1;                          /* default to seconds */
       } else {
-	 for (i=0; mod[i]; i++) {
-	    if (strncasecmp(mod_str, mod[i], mod_len) == 0) {
-	       break;
-	    }
-	 }
-	 if (mod[i] == NULL) {
-	    i = 1;			 /* no modifier, assume secs */
-	 }
+         for (i=0; mod[i]; i++) {
+            if (strncasecmp(mod_str, mod[i], mod_len) == 0) {
+               break;
+            }
+         }
+         if (mod[i] == NULL) {
+            return false;
+         }
       }
       Dmsg2(900, "str=%s: mult=%d\n", num_str, mult[i]);
       errno = 0;
       val = strtod(num_str, NULL);
       if (errno != 0 || val < 0) {
-	 return 0;
+         return false;
       }
       total += val * mult[i];
    }
    *value = (utime_t)total;
-   return 1;
+   return true;
 }
 
 /*
@@ -284,9 +309,9 @@ char *edit_utime(utime_t val, char *buf, int buf_len)
    for (i=0; i<5; i++) {
       times = (uint32_t)(val / mult[i]);
       if (times > 0) {
-	 val = val - (utime_t)times * mult[i];
+         val = val - (utime_t)times * mult[i];
          bsnprintf(mybuf, sizeof(mybuf), "%d %s%s ", times, mod[i], times>1?"s":"");
-	 bstrncat(buf, mybuf, buf_len);
+         bstrncat(buf, mybuf, buf_len);
       }
    }
    if (val == 0 && strlen(buf) == 0) {
@@ -300,45 +325,49 @@ char *edit_utime(utime_t val, char *buf, int buf_len)
 
 /*
  * Convert a size in bytes to uint64_t
- * Returns 0: if error
-	   1: if OK, and value stored in value
+ * Returns false: if error
+           true:  if OK, and value stored in value
  */
-int size_to_uint64(char *str, int str_len, uint64_t *value)
+bool size_to_uint64(char *str, int str_len, uint64_t *value)
 {
    int i, mod_len;
    double val;
    char mod_str[20];
    char num_str[50];
    static const char *mod[]  = {"*", "k", "kb", "m", "mb",  "g", "gb",  NULL}; /* first item * not used */
-   const int64_t mult[] = {1,		  /* byte */
-			   1024,	  /* kilobyte */
-			   1000,	  /* kb kilobyte */
-			   1048576,	  /* megabyte */
-			   1000000,	  /* mb megabyte */
-			   1073741824,	  /* gigabyte */
-			   1000000000};   /* gb gigabyte */
+   const int64_t mult[] = {1,             /* byte */
+                           1024,          /* kilobyte */
+                           1000,          /* kb kilobyte */
+                           1048576,       /* megabyte */
+                           1000000,       /* mb megabyte */
+                           1073741824,    /* gigabyte */
+                           1000000000};   /* gb gigabyte */
 
    if (!get_modifier(str, num_str, sizeof(num_str), mod_str, sizeof(mod_str))) {
       return 0;
    }
    /* Now find the multiplier corresponding to the modifier */
    mod_len = strlen(mod_str);
-   for (i=0; mod[i]; i++) {
-      if (strncasecmp(mod_str, mod[i], mod_len) == 0) {
-	 break;
+   if (mod_len == 0) {
+      i = 0;                          /* default with no modifier = 1 */
+   } else {
+      for (i=0; mod[i]; i++) {
+         if (strncasecmp(mod_str, mod[i], mod_len) == 0) {
+            break;
+         }
+      }
+      if (mod[i] == NULL) {
+         return false;
       }
-   }
-   if (mod[i] == NULL) {
-      i = 0;			      /* no modifier found, assume 1 */
    }
    Dmsg2(900, "str=%s: mult=%d\n", str, mult[i]);
    errno = 0;
    val = strtod(num_str, NULL);
    if (errno != 0 || val < 0) {
-      return 0;
+      return false;
    }
-  *value = (utime_t)(val * mult[i]);
-   return 1;
+   *value = (utime_t)(val * mult[i]);
+   return true;
 }
 
 /*
@@ -362,7 +391,7 @@ bool is_a_number(const char *n)
    }
    if (digit_seen && (*n == 'e' || *n == 'E')
        && (B_ISDIGIT(n[1]) || ((n[1]=='-' || n[1] == '+') && B_ISDIGIT(n[2])))) {
-      n += 2;			      /* skip e- or e+ or e digit */
+      n += 2;                         /* skip e- or e+ or e digit */
       while (B_ISDIGIT(*n)) { n++; }
    }
    return digit_seen && *n==0;
@@ -398,7 +427,7 @@ bool is_name_valid(char *name, POOLMEM **msg)
    /* Restrict the characters permitted in the Volume name */
    for (p=name; *p; p++) {
       if (B_ISALPHA(*p) || B_ISDIGIT(*p) || strchr(accept, (int)(*p))) {
-	 continue;
+         continue;
       }
       if (msg) {
          Mmsg(msg, _("Illegal character \"%c\" in name.\n"), *p);
@@ -446,7 +475,7 @@ char *add_commas(char *val, char *buf)
    *q-- = *p--;
    for ( ; nc; nc--) {
       for (i=0; i < 3; i++) {
-	  *q-- = *p--;
+          *q-- = *p--;
       }
       *q-- = ',';
    }
@@ -467,7 +496,7 @@ int main(int argc, char *argv[])
       strcpy(buf, str[i]);
       if (!duration_to_utime(buf, &val)) {
          printf("Error return from duration_to_utime for in=%s\n", str[i]);
-	 continue;
+         continue;
       }
       edit_utime(val, outval);
       printf("in=%s val=%" lld " outval=%s\n", str[i], val, outval);
