@@ -1,30 +1,39 @@
 /*
- * Manipulation routines for Job Control Records
+ * Manipulation routines for Job Control Records and
+ *  handling of last_jobs_list.
  *
  *  Kern E. Sibbald, December 2000
  *
- *  Version $Id: jcr.c,v 1.61.2.1 2005/02/14 10:02:25 kerns Exp $
+ *  Version $Id: jcr.c,v 1.75.2.8 2006/03/28 16:56:33 kerns Exp $
  *
  *  These routines are thread safe.
  *
+ *  The job list routines were re-written in May 2005 to
+ *  eliminate the global lock while traversing the list, and
+ *  to use the dlist subroutines.  The locking is now done
+ *  on the list each time the list is modified or traversed.
+ *  That is it is "micro-locked" rather than globally locked.
+ *  The result is that there is one lock/unlock for each entry
+ *  in the list while traversing it rather than a single lock
+ *  at the beginning of a traversal and one at the end.  This
+ *  incurs slightly more overhead, but effectively eliminates 
+ *  the possibilty of race conditions.  In addition, with the
+ *  exception of the global locking of the list during the
+ *  re-reading of the config file, no recursion is needed.
+ *
  */
 /*
-   Copyright (C) 2000-2005 Kern Sibbald
+   Copyright (C) 2000-2006 Kern Sibbald
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of
-   the License, or (at your option) any later version.
+   modify it under the terms of the GNU General Public License
+   version 2 as amended with additional clauses defined in the
+   file LICENSE in the main source directory.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public
-   License along with this program; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+   the file LICENSE for additional details.
 
  */
 
@@ -36,79 +45,105 @@ extern time_t watchdog_time;
 
 /* Forward referenced functions */
 extern "C" void timeout_handler(int sig);
-
 static void jcr_timeout_check(watchdog_t *self);
+#ifdef TRACE_JCR_CHAIN
+static void b_lock_jcr_chain(const char *filen, int line);
+static void b_unlock_jcr_chain(const char *filen, int line);
+#define lock_jcr_chain() b_lock_jcr_chain(__FILE__, __LINE__);
+#define unlock_jcr_chain() b_unlock_jcr_chain(__FILE__, __LINE__);
+#else
+static void lock_jcr_chain();
+static void unlock_jcr_chain();
+#endif
+
 
 int num_jobs_run;
 dlist *last_jobs = NULL;
 const int max_last_jobs = 10;
+ 
+static dlist *jcrs = NULL;            /* JCR chain */
+static pthread_mutex_t jcr_lock = PTHREAD_MUTEX_INITIALIZER;
 
-JCR *jobs = NULL;		      /* pointer to JCR chain */
-static brwlock_t lock;		      /* lock for last jobs and JCR chain */
+static pthread_mutex_t job_start_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+static pthread_mutex_t last_jobs_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+
+void lock_jobs()
+{
+   P(job_start_mutex);
+}
+
+void unlock_jobs()
+{
+   V(job_start_mutex);
+}
 
 void init_last_jobs_list()
 {
-   int errstat;
+   JCR *jcr = NULL;
    struct s_last_job *job_entry = NULL;
    if (!last_jobs) {
       last_jobs = New(dlist(job_entry, &job_entry->link));
-      if ((errstat=rwl_init(&lock)) != 0) {
-         Emsg1(M_ABORT, 0, _("Unable to initialize jcr_chain lock. ERR=%s\n"),
-	       strerror(errstat));
-      }
    }
-
+   if (!jcrs) {
+      jcrs = New(dlist(jcr, &jcr->link));
+   }
 }
 
 void term_last_jobs_list()
 {
    if (last_jobs) {
       while (!last_jobs->empty()) {
-	 void *je = last_jobs->first();
-	 last_jobs->remove(je);
-	 free(je);
+         void *je = last_jobs->first();
+         last_jobs->remove(je);
+         free(je);
       }
       delete last_jobs;
       last_jobs = NULL;
-      rwl_destroy(&lock);
+   }
+   if (jcrs) {
+      delete jcrs;
+      jcrs = NULL;
    }
 }
 
-void read_last_jobs_list(int fd, uint64_t addr)
+bool read_last_jobs_list(int fd, uint64_t addr)
 {
    struct s_last_job *je, job;
    uint32_t num;
 
    Dmsg1(100, "read_last_jobs seek to %d\n", (int)addr);
    if (addr == 0 || lseek(fd, (off_t)addr, SEEK_SET) < 0) {
-      return;
+      return false;
    }
    if (read(fd, &num, sizeof(num)) != sizeof(num)) {
-      return;
+      return false;
    }
    Dmsg1(100, "Read num_items=%d\n", num);
    if (num > 4 * max_last_jobs) {  /* sanity check */
-      return;
+      return false;
    }
    for ( ; num; num--) {
       if (read(fd, &job, sizeof(job)) != sizeof(job)) {
          Dmsg1(000, "Read job entry. ERR=%s\n", strerror(errno));
-	 return;
+         return false;
       }
       if (job.JobId > 0) {
-	 je = (struct s_last_job *)malloc(sizeof(struct s_last_job));
-	 memcpy((char *)je, (char *)&job, sizeof(job));
-	 if (!last_jobs) {
-	    init_last_jobs_list();
-	 }
-	 last_jobs->append(je);
-	 if (last_jobs->size() > max_last_jobs) {
-	    je = (struct s_last_job *)last_jobs->first();
-	    last_jobs->remove(je);
-	    free(je);
-	 }
+         je = (struct s_last_job *)malloc(sizeof(struct s_last_job));
+         memcpy((char *)je, (char *)&job, sizeof(job));
+         if (!last_jobs) {
+            init_last_jobs_list();
+         }
+         last_jobs->append(je);
+         if (last_jobs->size() > max_last_jobs) {
+            je = (struct s_last_job *)last_jobs->first();
+            last_jobs->remove(je);
+            free(je);
+         }
       }
    }
+   return true;
 }
 
 uint64_t write_last_jobs_list(int fd, uint64_t addr)
@@ -125,13 +160,13 @@ uint64_t write_last_jobs_list(int fd, uint64_t addr)
       num = last_jobs->size();
       if (write(fd, &num, sizeof(num)) != sizeof(num)) {
          Dmsg1(000, "Error writing num_items: ERR=%s\n", strerror(errno));
-	 return 0;
+         return 0;
       }
       foreach_dlist(je, last_jobs) {
-	 if (write(fd, je, sizeof(struct s_last_job)) != sizeof(struct s_last_job)) {
+         if (write(fd, je, sizeof(struct s_last_job)) != sizeof(struct s_last_job)) {
             Dmsg1(000, "Error writing job: ERR=%s\n", strerror(errno));
-	    return 0;
-	 }
+            return 0;
+         }
       }
    }
    /* Return current address */
@@ -145,14 +180,12 @@ uint64_t write_last_jobs_list(int fd, uint64_t addr)
 
 void lock_last_jobs_list()
 {
-   /* Use jcr chain mutex */
-   lock_jcr_chain();
+   P(last_jobs_mutex);
 }
 
 void unlock_last_jobs_list()
 {
-   /* Use jcr chain mutex */
-   unlock_jcr_chain();
+   V(last_jobs_mutex);
 }
 
 /*
@@ -196,17 +229,17 @@ JCR *new_jcr(int size, JCR_free_HANDLER *daemon_free_jcr)
    jcr->job_end_push.init(1, false);
    jcr->sched_time = time(NULL);
    jcr->daemon_free_jcr = daemon_free_jcr;    /* plug daemon free routine */
-   jcr->use_count = 1;
-   pthread_mutex_init(&(jcr->mutex), NULL);
+   jcr->init_mutex();
+   jcr->inc_use_count();   
    jcr->JobStatus = JS_Created;       /* ready to run */
    jcr->VolumeName = get_pool_memory(PM_FNAME);
    jcr->VolumeName[0] = 0;
    jcr->errmsg = get_pool_memory(PM_MESSAGE);
    jcr->errmsg[0] = 0;
    /* Setup some dummy values */
-   jcr->Job[0] = 0;		      /* no job name by default */
+   bstrncpy(jcr->Job, "*System*", sizeof(jcr->Job));
    jcr->JobId = 0;
-   jcr->JobType = JT_SYSTEM;	      /* internal job until defined */
+   jcr->JobType = JT_SYSTEM;          /* internal job until defined */
    jcr->JobLevel = L_NONE;
    jcr->JobStatus = JS_Created;
 
@@ -215,14 +248,21 @@ JCR *new_jcr(int size, JCR_free_HANDLER *daemon_free_jcr)
    sigfillset(&sigtimer.sa_mask);
    sigaction(TIMEOUT_SIGNAL, &sigtimer, NULL);
 
+   /*
+    * Locking jobs is a global lock that is needed
+    * so that the Director can stop new jobs from being
+    * added to the jcr chain while it processes a new
+    * conf file and does the job_end_push().
+    */
+   lock_jobs();
    lock_jcr_chain();
-   jcr->prev = NULL;
-   jcr->next = jobs;
-   if (jobs) {
-      jobs->prev = jcr;
+   if (!jcrs) {
+      jcrs = New(dlist(jcr, &jcr->link));
    }
-   jobs = jcr;
+   jcrs->append(jcr);
    unlock_jcr_chain();
+   unlock_jobs();
+
    return jcr;
 }
 
@@ -230,22 +270,15 @@ JCR *new_jcr(int size, JCR_free_HANDLER *daemon_free_jcr)
 /*
  * Remove a JCR from the chain
  * NOTE! The chain must be locked prior to calling
- *	 this routine.
+ *       this routine.
  */
 static void remove_jcr(JCR *jcr)
 {
    Dmsg0(3400, "Enter remove_jcr\n");
    if (!jcr) {
-      Emsg0(M_ABORT, 0, "NULL jcr.\n");
-   }
-   if (!jcr->prev) {		      /* if no prev */
-      jobs = jcr->next; 	      /* set new head */
-   } else {
-      jcr->prev->next = jcr->next;    /* update prev */
-   }
-   if (jcr->next) {
-      jcr->next->prev = jcr->prev;
+      Emsg0(M_ABORT, 0, _("NULL jcr.\n"));
    }
+   jcrs->remove(jcr);
    Dmsg0(3400, "Leave remove_jcr\n");
 }
 
@@ -262,6 +295,8 @@ static void free_common_jcr(JCR *jcr)
    case JT_BACKUP:
    case JT_VERIFY:
    case JT_RESTORE:
+   case JT_MIGRATE:
+   case JT_COPY:
    case JT_ADMIN:
       num_jobs_run++;
       last_job.Errors = jcr->Errors;
@@ -278,26 +313,26 @@ static void free_common_jcr(JCR *jcr)
       last_job.end_time = time(NULL);
       /* Keep list of last jobs, but not Console where JobId==0 */
       if (last_job.JobId > 0) {
-	 je = (struct s_last_job *)malloc(sizeof(struct s_last_job));
-	 memcpy((char *)je, (char *)&last_job, sizeof(last_job));
-	 if (!last_jobs) {
-	    init_last_jobs_list();
-	 }
-	 last_jobs->append(je);
-	 if (last_jobs->size() > max_last_jobs) {
-	    je = (struct s_last_job *)last_jobs->first();
-	    last_jobs->remove(je);
-	    free(je);
-	 }
+         je = (struct s_last_job *)malloc(sizeof(struct s_last_job));
+         memcpy((char *)je, (char *)&last_job, sizeof(last_job));
+         if (!last_jobs) {
+            init_last_jobs_list();
+         }
+         last_jobs->append(je);
+         if (last_jobs->size() > max_last_jobs) {
+            je = (struct s_last_job *)last_jobs->first();
+            last_jobs->remove(je);
+            free(je);
+         }
       }
       break;
    default:
       break;
    }
-   pthread_mutex_destroy(&jcr->mutex);
+   jcr->destroy_mutex();
 
    delete jcr->msg_queue;
-   close_msg(jcr);		      /* close messages for this job */
+   close_msg(jcr);                    /* close messages for this job */
 
    /* do this after closing messages */
    if (jcr->client_name) {
@@ -305,6 +340,11 @@ static void free_common_jcr(JCR *jcr)
       jcr->client_name = NULL;
    }
 
+   if (jcr->attr) {
+      free_pool_memory(jcr->attr);
+      jcr->attr = NULL;
+   }
+
    if (jcr->sd_auth_key) {
       free(jcr->sd_auth_key);
       jcr->sd_auth_key = NULL;
@@ -349,100 +389,78 @@ void b_free_jcr(const char *file, int line, JCR *jcr)
 void free_jcr(JCR *jcr)
 {
 
-   Dmsg1(3400, "Enter free_jcr 0x%x\n", jcr);
+   Dmsg2(3400, "Enter free_jcr 0x%x job=%d\n", jcr, jcr->JobId);
 
 #endif
 
    dequeue_messages(jcr);
    lock_jcr_chain();
-   jcr->use_count--;		      /* decrement use count */
-   if (jcr->use_count < 0) {
+   jcr->dec_use_count();              /* decrement use count */
+   if (jcr->use_count() < 0) {
       Emsg2(M_ERROR, 0, _("JCR use_count=%d JobId=%d\n"),
-	 jcr->use_count, jcr->JobId);
+         jcr->use_count(), jcr->JobId);
    }
-   Dmsg3(3400, "Dec free_jcr 0x%x use_count=%d jobid=%d\n", jcr, jcr->use_count, jcr->JobId);
-   if (jcr->use_count > 0) {	      /* if in use */
+   Dmsg3(3400, "Dec free_jcr 0x%x use_count=%d jobid=%d\n", jcr, jcr->use_count(), jcr->JobId);
+   if (jcr->use_count() > 0) {          /* if in use */
       unlock_jcr_chain();
-      Dmsg2(3400, "free_jcr 0x%x use_count=%d\n", jcr, jcr->use_count);
+      Dmsg3(3400, "free_jcr 0x%x job=%d use_count=%d\n", jcr, jcr->JobId, jcr->use_count());
       return;
    }
-   remove_jcr(jcr);		      /* remove Jcr from chain */
+
+   remove_jcr(jcr);                   /* remove Jcr from chain */
    unlock_jcr_chain();
 
-   job_end_pop(jcr);		      /* pop and call hooked routines */
+   job_end_pop(jcr);                  /* pop and call hooked routines */
 
    Dmsg1(3400, "End job=%d\n", jcr->JobId);
    if (jcr->daemon_free_jcr) {
       jcr->daemon_free_jcr(jcr);      /* call daemon free routine */
    }
    free_common_jcr(jcr);
-   close_msg(NULL);		      /* flush any daemon messages */
+   close_msg(NULL);                   /* flush any daemon messages */
+   garbage_collect_memory_pool();
    Dmsg0(3400, "Exit free_jcr\n");
 }
 
 
-/*
- * Global routine to free a jcr
- *  JCR chain is already locked
- */
-void free_locked_jcr(JCR *jcr)
-{
-   jcr->use_count--;		      /* decrement use count */
-   Dmsg2(3400, "Dec free_locked_jcr 0x%x use_count=%d\n", jcr, jcr->use_count);
-   if (jcr->use_count > 0) {	      /* if in use */
-      return;
-   }
-   remove_jcr(jcr);
-   jcr->daemon_free_jcr(jcr);	      /* call daemon free routine */
-   free_common_jcr(jcr);
-}
-
-
-
 /*
  * Given a JobId, find the JCR
  *   Returns: jcr on success
- *	      NULL on failure
+ *            NULL on failure
  */
 JCR *get_jcr_by_id(uint32_t JobId)
 {
    JCR *jcr;
 
-   lock_jcr_chain();			/* lock chain */
-   for (jcr = jobs; jcr; jcr=jcr->next) {
+   foreach_jcr(jcr) {
       if (jcr->JobId == JobId) {
-	 P(jcr->mutex);
-	 jcr->use_count++;
-	 V(jcr->mutex);
-         Dmsg2(3400, "Inc get_jcr 0x%x use_count=%d\n", jcr, jcr->use_count);
-	 break;
+         jcr->inc_use_count();
+         Dmsg2(3400, "Inc get_jcr 0x%x use_count=%d\n", jcr, jcr->use_count());
+         break;
       }
    }
-   unlock_jcr_chain();
+   endeach_jcr(jcr);
    return jcr;
 }
 
 /*
  * Given a SessionId and SessionTime, find the JCR
  *   Returns: jcr on success
- *	      NULL on failure
+ *            NULL on failure
  */
 JCR *get_jcr_by_session(uint32_t SessionId, uint32_t SessionTime)
 {
    JCR *jcr;
 
-   lock_jcr_chain();
-   for (jcr = jobs; jcr; jcr=jcr->next) {
+   foreach_jcr(jcr) {
       if (jcr->VolSessionId == SessionId &&
-	  jcr->VolSessionTime == SessionTime) {
-	 P(jcr->mutex);
-	 jcr->use_count++;
-	 V(jcr->mutex);
-         Dmsg2(3400, "Inc get_jcr 0x%x use_count=%d\n", jcr, jcr->use_count);
-	 break;
+          jcr->VolSessionTime == SessionTime) {
+         jcr->inc_use_count();
+         Dmsg2(3400, "Inc get_jcr 0x%x use_count=%d\n", jcr, jcr->use_count());
+         break;
       }
    }
-   unlock_jcr_chain();
+   endeach_jcr(jcr);
    return jcr;
 }
 
@@ -452,7 +470,7 @@ JCR *get_jcr_by_session(uint32_t SessionId, uint32_t SessionTime)
  *  compares on the number of characters in Job
  *  thus allowing partial matches.
  *   Returns: jcr on success
- *	      NULL on failure
+ *            NULL on failure
  */
 JCR *get_jcr_by_partial_name(char *Job)
 {
@@ -462,18 +480,15 @@ JCR *get_jcr_by_partial_name(char *Job)
    if (!Job) {
       return NULL;
    }
-   lock_jcr_chain();
    len = strlen(Job);
-   for (jcr = jobs; jcr; jcr=jcr->next) {
+   foreach_jcr(jcr) {
       if (strncmp(Job, jcr->Job, len) == 0) {
-	 P(jcr->mutex);
-	 jcr->use_count++;
-	 V(jcr->mutex);
-         Dmsg2(3400, "Inc get_jcr 0x%x use_count=%d\n", jcr, jcr->use_count);
-	 break;
+         jcr->inc_use_count();
+         Dmsg2(3400, "Inc get_jcr 0x%x use_count=%d\n", jcr, jcr->use_count());
+         break;
       }
    }
-   unlock_jcr_chain();
+   endeach_jcr(jcr);
    return jcr;
 }
 
@@ -482,7 +497,7 @@ JCR *get_jcr_by_partial_name(char *Job)
  * Given a Job, find the JCR
  *  requires an exact match of names.
  *   Returns: jcr on success
- *	      NULL on failure
+ *            NULL on failure
  */
 JCR *get_jcr_by_full_name(char *Job)
 {
@@ -491,17 +506,14 @@ JCR *get_jcr_by_full_name(char *Job)
    if (!Job) {
       return NULL;
    }
-   lock_jcr_chain();
-   for (jcr = jobs; jcr; jcr=jcr->next) {
+   foreach_jcr(jcr) {
       if (strcmp(jcr->Job, Job) == 0) {
-	 P(jcr->mutex);
-	 jcr->use_count++;
-	 V(jcr->mutex);
-         Dmsg2(3400, "Inc get_jcr 0x%x use_count=%d\n", jcr, jcr->use_count);
-	 break;
+         jcr->inc_use_count();
+         Dmsg2(3400, "Inc get_jcr 0x%x use_count=%d\n", jcr, jcr->use_count());
+         break;
       }
    }
-   unlock_jcr_chain();
+   endeach_jcr(jcr);
    return jcr;
 }
 
@@ -531,68 +543,108 @@ static int lock_count = 0;
  * Lock the chain
  */
 #ifdef TRACE_JCR_CHAIN
-void b_lock_jcr_chain(const char *fname, int line)
+static void b_lock_jcr_chain(const char *fname, int line)
 #else
-void lock_jcr_chain()
+static void lock_jcr_chain()
 #endif
 {
-   int errstat;
 #ifdef TRACE_JCR_CHAIN
    Dmsg3(3400, "Lock jcr chain %d from %s:%d\n", ++lock_count,
       fname, line);
 #endif
-   if ((errstat=rwl_writelock(&lock)) != 0) {
-      Emsg1(M_ABORT, 0, "rwl_writelock failure. ERR=%s\n",
-	   strerror(errstat));
-   }
+   P(jcr_lock);
 }
 
 /*
  * Unlock the chain
  */
 #ifdef TRACE_JCR_CHAIN
-void b_unlock_jcr_chain(const char *fname, int line)
+static void b_unlock_jcr_chain(const char *fname, int line)
 #else
-void unlock_jcr_chain()
+static void unlock_jcr_chain()
 #endif
 {
-   int errstat;
 #ifdef TRACE_JCR_CHAIN
    Dmsg3(3400, "Unlock jcr chain %d from %s:%d\n", lock_count--,
       fname, line);
 #endif
-   if ((errstat=rwl_writeunlock(&lock)) != 0) {
-      Emsg1(M_ABORT, 0, "rwl_writeunlock failure. ERR=%s\n",
-	   strerror(errstat));
-   }
+   V(jcr_lock);
 }
 
 
-JCR *get_next_jcr(JCR *prev_jcr)
+/*
+ * Start walk of jcr chain
+ * The proper way to walk the jcr chain is:
+ *    JCR *jcr;
+ *    foreach_jcr(jcr) {
+ *      ...
+ *    }
+ *    endeach_jcr(jcr);
+ *
+ *  It is possible to leave out the endeach_jcr(jcr), but
+ *   in that case, the last jcr referenced must be explicitly
+ *   released with:
+ *
+ *    free_jcr(jcr);
+ *  
+ */
+JCR *jcr_walk_start() 
+{
+   JCR *jcr;
+   lock_jcr_chain();
+   jcr = (JCR *)jcrs->first();
+   if (jcr) {
+      jcr->inc_use_count();
+      Dmsg3(3400, "Inc jcr_walk_start 0x%x job=%d use_count=%d\n", jcr, 
+            jcr->JobId, jcr->use_count());
+   }
+   unlock_jcr_chain();
+   return jcr;
+}
+
+/*
+ * Get next jcr from chain, and release current one
+ */
+JCR *jcr_walk_next(JCR *prev_jcr)
 {
    JCR *jcr;
 
-   if (prev_jcr == NULL) {
-      jcr = jobs;
-   } else {
-      jcr = prev_jcr->next;
-   }
+   lock_jcr_chain();
+   jcr = (JCR *)jcrs->next(prev_jcr);
    if (jcr) {
-      P(jcr->mutex);
-      jcr->use_count++;
-      V(jcr->mutex);
-      Dmsg2(3400, "Inc get_next_jcr 0x%x use_count=%d\n", jcr, jcr->use_count);
+      jcr->inc_use_count();
+      Dmsg3(3400, "Inc jcr_walk_next 0x%x job=%d use_count=%d\n", jcr, 
+         jcr->JobId, jcr->use_count());
+   }
+   unlock_jcr_chain();
+   if (prev_jcr) {
+      free_jcr(prev_jcr);
    }
    return jcr;
 }
 
+/*
+ * Release last jcr referenced
+ */
+void jcr_walk_end(JCR *jcr)
+{
+   if (jcr) {
+      free_jcr(jcr);
+   }
+}
+
+
+/*
+ * Setup to call the timeout check routine every 30 seconds
+ *  This routine will check any timers that have been enabled.
+ */
 bool init_jcr_subsystem(void)
 {
    watchdog_t *wd = new_watchdog();
 
    wd->one_shot = false;
-   wd->interval = 30;	/* FIXME: should be configurable somewhere, even
-			 if only with a #define */
+   wd->interval = 30;   /* FIXME: should be configurable somewhere, even
+                         if only with a #define */
    wd->callback = jcr_timeout_check;
 
    register_watchdog(wd);
@@ -611,51 +663,49 @@ static void jcr_timeout_check(watchdog_t *self)
    /* Walk through all JCRs checking if any one is
     * blocked for more than specified max time.
     */
-   lock_jcr_chain();
    foreach_jcr(jcr) {
-      free_locked_jcr(jcr);	      /* OK to free now cuz chain is locked */
+      Dmsg2(3400, "jcr_timeout_check JobId=%u jcr=0x%x\n", jcr->JobId, jcr);
       if (jcr->JobId == 0) {
-	 continue;
+         continue;
       }
       fd = jcr->store_bsock;
       if (fd) {
-	 timer_start = fd->timer_start;
-	 if (timer_start && (watchdog_time - timer_start) > fd->timeout) {
-	    fd->timer_start = 0;      /* turn off timer */
-	    fd->timed_out = TRUE;
-	    Jmsg(jcr, M_ERROR, 0, _(
+         timer_start = fd->timer_start;
+         if (timer_start && (watchdog_time - timer_start) > fd->timeout) {
+            fd->timer_start = 0;      /* turn off timer */
+            fd->timed_out = true;
+            Jmsg(jcr, M_ERROR, 0, _(
 "Watchdog sending kill after %d secs to thread stalled reading Storage daemon.\n"),
-		 watchdog_time - timer_start);
-	    pthread_kill(jcr->my_thread_id, TIMEOUT_SIGNAL);
-	 }
+                 watchdog_time - timer_start);
+            pthread_kill(jcr->my_thread_id, TIMEOUT_SIGNAL);
+         }
       }
       fd = jcr->file_bsock;
       if (fd) {
-	 timer_start = fd->timer_start;
-	 if (timer_start && (watchdog_time - timer_start) > fd->timeout) {
-	    fd->timer_start = 0;      /* turn off timer */
-	    fd->timed_out = TRUE;
-	    Jmsg(jcr, M_ERROR, 0, _(
+         timer_start = fd->timer_start;
+         if (timer_start && (watchdog_time - timer_start) > fd->timeout) {
+            fd->timer_start = 0;      /* turn off timer */
+            fd->timed_out = true;
+            Jmsg(jcr, M_ERROR, 0, _(
 "Watchdog sending kill after %d secs to thread stalled reading File daemon.\n"),
-		 watchdog_time - timer_start);
-	    pthread_kill(jcr->my_thread_id, TIMEOUT_SIGNAL);
-	 }
+                 watchdog_time - timer_start);
+            pthread_kill(jcr->my_thread_id, TIMEOUT_SIGNAL);
+         }
       }
       fd = jcr->dir_bsock;
       if (fd) {
-	 timer_start = fd->timer_start;
-	 if (timer_start && (watchdog_time - timer_start) > fd->timeout) {
-	    fd->timer_start = 0;      /* turn off timer */
-	    fd->timed_out = TRUE;
-	    Jmsg(jcr, M_ERROR, 0, _(
+         timer_start = fd->timer_start;
+         if (timer_start && (watchdog_time - timer_start) > fd->timeout) {
+            fd->timer_start = 0;      /* turn off timer */
+            fd->timed_out = true;
+            Jmsg(jcr, M_ERROR, 0, _(
 "Watchdog sending kill after %d secs to thread stalled reading Director.\n"),
-		 watchdog_time - timer_start);
-	    pthread_kill(jcr->my_thread_id, TIMEOUT_SIGNAL);
-	 }
+                 watchdog_time - timer_start);
+            pthread_kill(jcr->my_thread_id, TIMEOUT_SIGNAL);
+         }
       }
-
    }
-   unlock_jcr_chain();
+   endeach_jcr(jcr);
 
    Dmsg0(3400, "Finished JCR timeout checks\n");
 }
@@ -665,5 +715,5 @@ static void jcr_timeout_check(watchdog_t *self)
  */
 extern "C" void timeout_handler(int sig)
 {
-   return;			      /* thus interrupting the function */
+   return;                            /* thus interrupting the function */
 }
