@@ -5,26 +5,21 @@
  *
  *   Kern Sibbald, MM
  *
- *   Version $Id: lex.h,v 1.9.4.1 2005/02/14 10:02:26 kerns Exp $
+ *   Version $Id: lex.h,v 1.11.2.1 2006/06/04 12:24:40 kerns Exp $
  *
  */
 /*
-   Copyright (C) 2000-2004 Kern Sibbald
+   Copyright (C) 2000-2006 Kern Sibbald
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of
-   the License, or (at your option) any later version.
+   modify it under the terms of the GNU General Public License
+   version 2 as amended with additional clauses defined in the
+   file LICENSE in the main source directory.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public
-   License along with this program; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+   the file LICENSE for additional details.
 
  */
 
@@ -101,6 +96,7 @@ typedef struct s_lex_context {
    int32_t int32_val;
    int64_t int64_val;
    void (*scan_error)(const char *file, int line, struct s_lex_context *lc, const char *msg, ...);
+   int err_type;                      /* message level for scan_error (M_..) */
    void *caller_ctx;                  /* caller private data */
 } LEX;
 
