@@ -1,42 +1,41 @@
-/* 
+/*
  *   Library includes for Bacula lib directory
  *
  *   This file contains an include for each library file
  *   that we use within Bacula. bacula.h includes this
  *   file and thus picks up everything we need in lib.
  *
- *   Version $Id: lib.h,v 1.17 2004/08/12 18:58:13 kerns Exp $
+ *   Version $Id: lib.h,v 1.19.2.1 2006/01/07 11:18:11 kerns Exp $
  */
 
 /*
-   Copyright (C) 2000-2003 Kern Sibbald and John Walker
+   Copyright (C) 2000-2006 Kern Sibbald
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of
-   the License, or (at your option) any later version.
+   modify it under the terms of the GNU General Public License
+   version 2 as amended with additional clauses defined in the
+   file LICENSE in the main source directory.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public
-   License along with this program; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+   the file LICENSE for additional details.
 
  */
 
 #include "smartall.h"
 #include "alist.h"
 #include "dlist.h"
+#include "base64.h"
 #include "bits.h"
 #include "btime.h"
+#include "crypto.h"
 #include "mem_pool.h"
 #include "message.h"
+/* #include "openssl.h" */
 #include "lex.h"
 #include "parse_conf.h"
+#include "tls.h"
 #include "bsock.h"
 #include "bshm.h"
 #include "workq.h"
