@@ -1,28 +1,23 @@
 /*
 
         Definitions for the smart memory allocator
-  
-     Version $Id: smartall.h,v 1.10 2004/08/13 20:27:05 kerns Exp $
+
+     Version $Id: smartall.h,v 1.12.2.1 2005/11/12 17:30:52 kerns Exp $
 
 */
 
 /*
-   Copyright (C) 2000-2004 Kern Sibbald and John Walker
+   Copyright (C) 2000-2005 Kern Sibbald
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of
-   the License, or (at your option) any later version.
+   modify it under the terms of the GNU General Public License
+   version 2 as amended with additional clauses defined in the
+   file LICENSE in the main source directory.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public
-   License along with this program; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+   the file LICENSE for additional details.
 
  */
 
@@ -84,7 +79,7 @@ extern int sm_check_rtn(const char *fname, int lineno, bool bufdump);
 #define sm_check_rtn(f, l, fl) 1
 
 extern void *b_malloc();
-#define malloc(x) b_malloc(__FILE__, __LINE__, (x))                  
+#define malloc(x) b_malloc(__FILE__, __LINE__, (x))
 
 
 #endif
@@ -100,12 +95,12 @@ public:
 
 void *operator new(size_t s, const char *fname, int line)
 {
-  void *p = sm_malloc(fname, line, s > sizeof(int) ? s : sizeof(int));
+  void *p = sm_malloc(fname, line, s > sizeof(int) ? (unsigned int)s : sizeof(int));
   return p;
 }
 void *operator new[](size_t s, const char *fname, int line)
 {
-   void *p = sm_malloc(fname, line, s > sizeof(int) ? s : sizeof(int));
+   void *p = sm_malloc(fname, line, s > sizeof(int) ? (unsigned int)s : sizeof(int));
    return p;
 }
 void  operator delete(void *ptr)
@@ -121,7 +116,7 @@ void  operator delete(void *ptr, const char *fname, int line)
 {
    free(ptr);
 }
-void  operator delete[](void *ptr, size_t i, const char *fname, int line) 
+void  operator delete[](void *ptr, size_t i, const char *fname, int line)
 {
    free(ptr);
 }
@@ -131,7 +126,7 @@ private:
 void *operator new(size_t s) throw() { return 0; }
 void *operator new[](size_t s) throw() { return 0; }
 };
- 
+
 
 #else
 
