@@ -3,26 +3,21 @@
  *
  *    Kern Sibbald, June 2002
  *
- *   Version $Id: bsr.h,v 1.15.8.1 2005/02/14 10:02:27 kerns Exp $
+ *   Version $Id: bsr.h,v 1.17.2.1 2006/03/14 21:41:41 kerns Exp $
  *
  */
 /*
-   Copyright (C) 2000-2005 Kern Sibbald
+   Copyright (C) 2002-2006 Kern Sibbald
 
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of
-   the License, or (at your option) any later version.
+   modify it under the terms of the GNU General Public License
+   version 2 as amended with additional clauses defined in the
+   file LICENSE in the main source directory.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public
-   License along with this program; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-   MA 02111-1307, USA.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+   the file LICENSE for additional details.
 
  */
 
@@ -38,6 +33,7 @@ struct VOL_LIST {
    VOL_LIST *next;
    char VolumeName[MAX_NAME_LENGTH];
    char MediaType[MAX_NAME_LENGTH];
+   char device[MAX_NAME_LENGTH];   /* ***FIXME*** use alist here */
    int Slot;
    uint32_t start_file;
 };
@@ -56,6 +52,8 @@ struct BSR_VOLUME {
    BSR_VOLUME *next;
    char VolumeName[MAX_NAME_LENGTH];
    char MediaType[MAX_NAME_LENGTH];
+   char device[MAX_NAME_LENGTH];   /* ***FIXME*** use alist here */
+   int32_t       Slot;                /* Slot */
 };
 
 struct BSR_CLIENT {
@@ -133,7 +131,6 @@ struct BSR {
    bool          use_fast_rejection;  /* set if fast rejection can be used */
    bool          use_positioning;     /* set if we can position the archive */
    BSR_VOLUME   *volume;
-   int32_t       Slot;                /* Slot */
    uint32_t      count;               /* count of files to restore this bsr */
    uint32_t      found;               /* count of restored files this bsr */
    BSR_VOLFILE  *volfile;
