@@ -4,11 +4,11 @@
  *
  *     Nicolas Boichat, August MMIV
  *
- *     Version $Id: tray-monitor.c,v 1.18.4.1.2.1 2005/03/28 13:33:27 nboichat Exp $
+ *     Version $Id: tray-monitor.c,v 1.25.2.2 2005/11/22 10:50:55 kerns Exp $
  */
 
 /*
-   Copyright (C) 2004 Kern Sibbald and John Walker
+   Copyright (C) 2004-2005 Kern Sibbald
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -41,6 +41,9 @@ int authenticate_director(JCR *jcr, MONITOR *monitor, DIRRES *director);
 int authenticate_file_daemon(JCR *jcr, MONITOR *monitor, CLIENT* client);
 int authenticate_storage_daemon(JCR *jcr, MONITOR *monitor, STORE* store);
 
+/* Dummy functions */
+int generate_daemon_event(JCR *jcr, const char *event) { return 1; }
+
 /* Forward referenced functions */
 void writecmd(monitoritem* item, const char* command);
 int docmd(monitoritem* item, const char* command, GSList** list);
@@ -98,17 +101,17 @@ static void usage()
    fprintf(stderr, _(
 "Copyright (C) 2000-2004 Kern Sibbald and John Walker\n"
 "Written by Nicolas Boichat (2004)\n"
-"\nVersion: " VERSION " (" BDATE ") %s %s %s\n\n"
+"\nVersion: %s (%s) %s %s %s\n\n"
 "Usage: tray-monitor [-c config_file] [-d debug_level]\n"
 "       -c <file>     set configuration file to file\n"
 "       -dnn          set debug level to nn\n"
 "       -t            test - read configuration and exit\n"
 "       -?            print this message.\n"
-"\n"), HOST_OS, DISTNAME, DISTVER);
+"\n"), VERSION, BDATE, HOST_OS, DISTNAME, DISTVER);
 }
 
 static GtkWidget *new_image_button(const gchar *stock_id,
-				   const gchar *label_text) {
+                                   const gchar *label_text) {
     GtkWidget *button;
     GtkWidget *box;
     GtkWidget *label;
@@ -157,7 +160,7 @@ void     smt_free(gpointer mem) {
 
 /*********************************************************************
  *
- *	   Main Bacula Tray Monitor -- User Interface Program
+ *         Main Bacula Tray Monitor -- User Interface Program
  *
  */
 int main(int argc, char *argv[])
@@ -179,9 +182,12 @@ int main(int argc, char *argv[])
    CLIENT* filed;
    STORE* stored;
 
+   setlocale(LC_ALL, "");
+   bindtextdomain("bacula", LOCALEDIR);
+   textdomain("bacula");
+
    init_stack_dump();
    my_name_is(argc, argv, "tray-monitor");
-   textdomain("bacula");
    init_msg(NULL, NULL);
    working_directory = "/tmp";
 
@@ -196,28 +202,28 @@ int main(int argc, char *argv[])
    while ((ch = getopt(argc, argv, "bc:d:th?f:s:")) != -1) {
       switch (ch) {
       case 'c':                    /* configuration file */
-	 if (configfile != NULL) {
-	    free(configfile);
-	 }
-	 configfile = bstrdup(optarg);
-	 break;
+         if (configfile != NULL) {
+            free(configfile);
+         }
+         configfile = bstrdup(optarg);
+         break;
 
       case 'd':
-	 debug_level = atoi(optarg);
-	 if (debug_level <= 0) {
-	    debug_level = 1;
-	 }
-	 break;
+         debug_level = atoi(optarg);
+         if (debug_level <= 0) {
+            debug_level = 1;
+         }
+         break;
 
       case 't':
-	 test_config = true;
-	 break;
+         test_config = true;
+         break;
 
       case 'h':
       case '?':
       default:
-	 usage();
-	 exit(1);
+         usage();
+         exit(1);
       }
    }
    argc -= optind;
@@ -242,7 +248,7 @@ int main(int argc, char *argv[])
 
    if (nitems != 1) {
       Emsg2(M_ERROR_TERM, 0,
-	 _("Error: %d Monitor resource defined in %s. You must define one and only one Monitor resource.\n"), nitems, configfile);
+         _("Error: %d Monitor resource defined in %s. You must define one and only one Monitor resource.\n"), nitems, configfile);
    }
 
    nitems = 0;
@@ -310,13 +316,13 @@ int main(int argc, char *argv[])
 
    GtkWidget *entry;
 
-   entry = gtk_menu_item_new_with_label("Open status window...");
+   entry = gtk_menu_item_new_with_label(_("Open status window..."));
    g_signal_connect(G_OBJECT(entry), "activate", G_CALLBACK(TrayIconActivate), NULL);
    gtk_menu_shell_append(GTK_MENU_SHELL(mTrayMenu), entry);
 
    gtk_menu_shell_append(GTK_MENU_SHELL(mTrayMenu), gtk_separator_menu_item_new());
 
-   entry = gtk_menu_item_new_with_label("Exit");
+   entry = gtk_menu_item_new_with_label(_("Exit"));
    g_signal_connect(G_OBJECT(entry), "activate", G_CALLBACK(TrayIconExit), NULL);
    gtk_menu_shell_append(GTK_MENU_SHELL(mTrayMenu), entry);
 
@@ -328,7 +334,7 @@ int main(int argc, char *argv[])
 
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
-   gtk_window_set_title(GTK_WINDOW(window), "Bacula tray monitor");
+   gtk_window_set_title(GTK_WINDOW(window), _("Bacula tray monitor"));
 
    g_signal_connect(G_OBJECT(window), "delete_event", G_CALLBACK(delete_event), NULL);
    //g_signal_connect (G_OBJECT (window), "destroy", G_CALLBACK (destroy), NULL);
@@ -352,19 +358,19 @@ int main(int argc, char *argv[])
    for (int i = 0; i < nitems; i++) {
       switch (items[i].type) {
       case R_DIRECTOR:
-	 str = g_string_new(((DIRRES*)(items[i].resource))->hdr.name);
-	 g_string_append(str, _(" (DIR)"));
-	 break;
+         str = g_string_new(((DIRRES*)(items[i].resource))->hdr.name);
+         g_string_append(str, _(" (DIR)"));
+         break;
       case R_CLIENT:
-	 str = g_string_new(((CLIENT*)(items[i].resource))->hdr.name);
-	 g_string_append(str, _(" (FD)"));
-	 break;
+         str = g_string_new(((CLIENT*)(items[i].resource))->hdr.name);
+         g_string_append(str, _(" (FD)"));
+         break;
       case R_STORAGE:
-	 str = g_string_new(((STORE*)(items[i].resource))->hdr.name);
-	 g_string_append(str, _(" (SD)"));
-	 break;
+         str = g_string_new(((STORE*)(items[i].resource))->hdr.name);
+         g_string_append(str, _(" (SD)"));
+         break;
       default:
-	 continue;
+         continue;
       }
 
       radio = gtk_radio_button_new_with_label(group, str->str);
@@ -379,11 +385,11 @@ int main(int argc, char *argv[])
       gtk_container_add(GTK_CONTAINER(align), items[i].label);
 
       gtk_table_attach(GTK_TABLE(daemon_table), radio, 0, 1, (i*2)+1, (i*2)+2,
-	 GTK_FILL, (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
+         GTK_FILL, (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
       gtk_table_attach(GTK_TABLE(daemon_table), items[i].image, 1, 2, (i*2)+1, (i*2)+2,
-	 GTK_FILL, (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
+         GTK_FILL, (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
       gtk_table_attach(GTK_TABLE(daemon_table), align, 2, 3, (i*2)+1, (i*2)+2,
-	 (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
+         (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
 
       separator = gtk_hseparator_new();
       gtk_table_attach_defaults(GTK_TABLE(daemon_table), separator, 0, 3, (i*2)+2, (i*2)+3);
@@ -449,13 +455,26 @@ int main(int argc, char *argv[])
 
    for (i = 0; i < nitems; i++) {
       if (items[i].D_sock) {
-	 writecmd(&items[i], "quit");
-	 bnet_sig(items[i].D_sock, BNET_TERMINATE); /* send EOF */
-	 bnet_close(items[i].D_sock);
+         switch (items[i].type) {
+         case R_DIRECTOR:
+            trayMessage(_("Disconnecting from Director %s:%d\n"), ((DIRRES*)items[i].resource)->address, ((DIRRES*)items[i].resource)->DIRport);
+            break;
+         case R_CLIENT:
+            trayMessage(_("Disconnecting from Client %s:%d\n"), ((CLIENT*)items[i].resource)->address, ((CLIENT*)items[i].resource)->FDport);
+            break;
+         case R_STORAGE:
+            trayMessage(_("Disconnecting from Storage %s:%d\n"), ((STORE*)items[i].resource)->address, ((STORE*)items[i].resource)->SDport);
+            break;          
+         default:
+            break;
+         }
+         //writecmd(&items[i], "quit");
+         bnet_sig(items[i].D_sock, BNET_TERMINATE); /* send EOF */
+         bnet_close(items[i].D_sock);
       }
    }
 
-   (void)WSACleanup();		     /* Cleanup Windows sockets */
+   (void)WSACleanup();               /* Cleanup Windows sockets */
 
    //Free xpm_generic_var
    for (i = 0; i < (int)(sizeof(xpm_generic)/sizeof(const char*)); i++) {
@@ -476,19 +495,25 @@ int main(int argc, char *argv[])
 
 static void MonitorAbout(GtkWidget *widget, gpointer data) {
 #if HAVE_GTK_2_4
-   GtkWidget* about = gtk_message_dialog_new_with_markup(GTK_WINDOW(window),GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE, _(
-      "<span size='x-large' weight='bold'>Bacula Tray Monitor</span>\n\n"
-      "Copyright (C) 2004 Kern Sibbald and John Walker\n"
-      "Written by Nicolas Boichat\n"
-      "\n<small>Version: " VERSION " (" BDATE ") %s %s %s</small>"
-   ), HOST_OS, DISTNAME, DISTVER);
+   GtkWidget* about = gtk_message_dialog_new_with_markup(GTK_WINDOW(window),GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
+      "<span size='x-large' weight='bold'>%s</span>\n\n"
+      "%s"
+      "\n<small>%s: %s (%s) %s %s %s</small>",
+      _("Bacula Tray Monitor"),
+      _("Copyright (C) 2004-2005 Kern Sibbald\n"
+        "Written by Nicolas Boichat\n"),
+      _("Version:"),
+      VERSION, BDATE, HOST_OS, DISTNAME, DISTVER);
 #else
-   GtkWidget* about = gtk_message_dialog_new(GTK_WINDOW(window),GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE, _(
-      "Bacula Tray Monitor\n\n"
-      "Copyright (C) 2004 Kern Sibbald and John Walker\n"
-      "Written by Nicolas Boichat\n"
-      "\nVersion: " VERSION " (" BDATE ") %s %s %s"
-   ), HOST_OS, DISTNAME, DISTVER);
+   GtkWidget* about = gtk_message_dialog_new(GTK_WINDOW(window),GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,   
+      "%s\n\n"
+      "%s"
+      "\n%s %s (%s) %s %s %s",
+      _("Bacula Tray Monitor"),
+      _("Copyright (C) 2004-2005 Kern Sibbald\n"
+        "Written by Nicolas Boichat\n"),
+      _("Version:"),
+      VERSION, BDATE, HOST_OS, DISTNAME, DISTVER);
 #endif
    gtk_dialog_run(GTK_DIALOG(about));
    gtk_widget_destroy(about);
@@ -501,8 +526,8 @@ static void MonitorRefresh(GtkWidget *widget, gpointer data) {
 }
 
 static gboolean delete_event( GtkWidget *widget,
-			      GdkEvent  *event,
-			      gpointer   data ) {
+                              GdkEvent  *event,
+                              gpointer   data ) {
    gtk_widget_hide(window);
    return TRUE; /* do not destroy the window */
 }
@@ -524,7 +549,7 @@ static void IntervalChanged(GtkWidget *widget, gpointer data) {
    g_source_remove(timerTag);
    timerTag = g_timeout_add(
       (guint)(
-	 gtk_spin_button_get_value(GTK_SPIN_BUTTON(timeoutspinner))*1000/nitems
+         gtk_spin_button_get_value(GTK_SPIN_BUTTON(timeoutspinner))*1000/nitems
       ), fd_read, NULL );
 }
 
@@ -532,10 +557,10 @@ static void DaemonChanged(GtkWidget *widget, monitoritem* data) {
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
       fullitem = -1;
       for (int i = 0; i < nitems; i++) {
-	 if (data == &(items[i])) {
-	    fullitem = i;
-	    break;
-	 }
+         if (data == &(items[i])) {
+            fullitem = i;
+            break;
+         }
       }
       g_return_if_fail(fullitem != -1);
 
@@ -550,18 +575,16 @@ static int authenticate_daemon(monitoritem* item, JCR *jcr) {
    switch (item->type) {
    case R_DIRECTOR:
       return authenticate_director(jcr, monitor, (DIRRES*)item->resource);
-      break;
    case R_CLIENT:
       return authenticate_file_daemon(jcr, monitor, (CLIENT*)item->resource);
-      break;
    case R_STORAGE:
       return authenticate_storage_daemon(jcr, monitor, (STORE*)item->resource);
-      break;
    default:
-      printf("Error, currentitem is not a Client or a Storage..\n");
+      printf(_("Error, currentitem is not a Client or a Storage..\n"));
       gtk_main_quit();
       return FALSE;
    }
+   return false;
 }
 
 static gboolean blink(gpointer data) {
@@ -594,29 +617,29 @@ static gboolean fd_read(gpointer data) {
       newbuffer = gtk_text_buffer_new(NULL);
       
       if (items[lastupdated].type == R_DIRECTOR)
-	 docmd(&items[lastupdated], "status Director\n", &list);
+         docmd(&items[lastupdated], "status Director\n", &list);
       else
-	 docmd(&items[lastupdated], "status\n", &list);
+         docmd(&items[lastupdated], "status\n", &list);
 
       it = list->next;
       do {
-	 gtk_text_buffer_get_end_iter(newbuffer, &stop);
-	 gtk_text_buffer_insert (newbuffer, &stop, ((GString*)it->data)->str, -1);
-	 if (it->data) g_string_free((GString*)it->data, TRUE);
+         gtk_text_buffer_get_end_iter(newbuffer, &stop);
+         gtk_text_buffer_insert (newbuffer, &stop, ((GString*)it->data)->str, -1);
+         if (it->data) g_string_free((GString*)it->data, TRUE);
       } while ((it = it->next) != NULL);
 
       g_slist_free(list);
             
       /* Keep the selection if necessary */
       if (gtk_text_buffer_get_selection_bounds(buffer, &start, &stop)) {
-	 gtk_text_buffer_get_iter_at_offset(newbuffer, &nstart, gtk_text_iter_get_offset(&start));
-	 gtk_text_buffer_get_iter_at_offset(newbuffer, &nstop,  gtk_text_iter_get_offset(&stop ));
+         gtk_text_buffer_get_iter_at_offset(newbuffer, &nstart, gtk_text_iter_get_offset(&start));
+         gtk_text_buffer_get_iter_at_offset(newbuffer, &nstop,  gtk_text_iter_get_offset(&stop ));
 
    #if HAVE_GTK_2_4
-	 gtk_text_buffer_select_range(newbuffer, &nstart, &nstop);
+         gtk_text_buffer_select_range(newbuffer, &nstart, &nstop);
    #else
-	 gtk_text_buffer_move_mark(newbuffer, gtk_text_buffer_get_mark(newbuffer, "insert"), &nstart);
-	 gtk_text_buffer_move_mark(newbuffer, gtk_text_buffer_get_mark(newbuffer, "selection_bound"), &nstop);
+         gtk_text_buffer_move_mark(newbuffer, gtk_text_buffer_get_mark(newbuffer, "insert"), &nstart);
+         gtk_text_buffer_move_mark(newbuffer, gtk_text_buffer_get_mark(newbuffer, "selection_bound"), &nstop);
    #endif
       }
 
@@ -630,7 +653,7 @@ static gboolean fd_read(gpointer data) {
    getstatus(&items[lastupdated], 0, &strlast);
    updateStatusIcon(&items[lastupdated]);
 
-   changeStatusMessage(&items[lastupdated], "Current job: %s\nLast job: %s", strcurrent->str, strlast->str);
+   changeStatusMessage(&items[lastupdated], _("Current job: %s\nLast job: %s"), strcurrent->str, strlast->str);
 
    updateStatusIcon(NULL);
 
@@ -640,6 +663,15 @@ static gboolean fd_read(gpointer data) {
    return 1;
 }
 
+void append_error_string(GString* str, int joberrors) {
+   if (joberrors > 1) {
+      g_string_append_printf(str, _(" (%d errors)"), joberrors);
+   }
+   else {
+      g_string_append_printf(str, _(" (%d error)"), joberrors);
+   }
+}
+
 void getstatus(monitoritem* item, int current, GString** str) {
    GSList *list, *it;
    stateenum ret = error;
@@ -652,121 +684,141 @@ void getstatus(monitoritem* item, int current, GString** str) {
 
    if (current) {
       if (item->type == R_DIRECTOR)
-	 docmd(&items[lastupdated], ".status dir current\n", &list);
+         docmd(&items[lastupdated], ".status dir current\n", &list);
       else
-	 docmd(&items[lastupdated], ".status current\n", &list);
+         docmd(&items[lastupdated], ".status current\n", &list);
    }
    else {
       if (item->type == R_DIRECTOR)
-	 docmd(&items[lastupdated], ".status dir last\n", &list);
+         docmd(&items[lastupdated], ".status dir last\n", &list);
       else
-	 docmd(&items[lastupdated], ".status last\n", &list);
+         docmd(&items[lastupdated], ".status last\n", &list);
    }
 
    it = list->next;
    if ((it == NULL) || (sscanf(((GString*)it->data)->str, OKqstatus, &num) != 1)) {
       g_string_append_printf(*str, ".status error : %s", (it == NULL) ? "" : ((GString*)it->data)->str);
       while (((*str)->str[(*str)->len-1] == '\n') || ((*str)->str[(*str)->len-1] == '\r')) {
-	 g_string_set_size(*str, (*str)->len-1);
+         g_string_set_size(*str, (*str)->len-1);
       }
       ret = error;
    }
    else if ((it = it->next) == NULL) {
       if (current) {
-	 g_string_append(*str, _("No current job."));
+         g_string_append(*str, _("No current job."));
       }
       else {
-	 g_string_append(*str, _("No last job."));
+         g_string_append(*str, _("No last job."));
       }
       ret = idle;
    }
    else if ((k = sscanf(((GString*)it->data)->str, DotStatusJob, &jobid, &jobstatus, &joberrors)) == 3) {
       switch (jobstatus) {
       case JS_Created:
-	 ret = (joberrors > 0) ? warn : running;
-	 g_string_append_printf(*str, _("Job status: Created (%d error%s)"), joberrors, (joberrors > 1) ? "s" : "");
-	 break;
+         ret = (joberrors > 0) ? warn : running;
+         g_string_append_printf(*str, _("Job status: Created"));
+         append_error_string(*str, joberrors);
+         break;
       case JS_Running:
-	 ret = (joberrors > 0) ? warn : running;
-	 g_string_append_printf(*str, _("Job status: Running (%d error%s)"), joberrors, (joberrors > 1) ? "s" : "");
-	 break;
+         ret = (joberrors > 0) ? warn : running;
+         g_string_append_printf(*str, _("Job status: Running"));
+         append_error_string(*str, joberrors);
+         break;
       case JS_Blocked:
-	 g_string_append_printf(*str, _("Job status: Blocked (%d error%s)"), joberrors, (joberrors > 1) ? "s" : "");
-	 ret = warn;
-	 break;
+         g_string_append_printf(*str, _("Job status: Blocked"));
+         append_error_string(*str, joberrors);
+         ret = warn;
+         break;
       case JS_Terminated:
-	 g_string_append_printf(*str, _("Job status: Terminated (%d error%s)"), joberrors, (joberrors > 1) ? "s" : "");
-	 ret = (joberrors > 0) ? warn : idle;
-	 break;
+         g_string_append_printf(*str, _("Job status: Terminated"));
+         append_error_string(*str, joberrors);
+         ret = (joberrors > 0) ? warn : idle;
+         break;
       case JS_ErrorTerminated:
-	 g_string_append_printf(*str, _("Job status: Terminated in error (%d error%s)"), joberrors, (joberrors > 1) ? "s" : "");
-	 ret = error;
-	 break;
+         g_string_append_printf(*str, _("Job status: Terminated in error"));
+         append_error_string(*str, joberrors);
+         ret = error;
+         break;
       case JS_Error:
-	 ret = (joberrors > 0) ? warn : running;
-	 g_string_append_printf(*str, _("Job status: Error (%d error%s)"), joberrors, (joberrors > 1) ? "s" : "");
-	 break;
+         ret = (joberrors > 0) ? warn : running;
+         g_string_append_printf(*str, _("Job status: Error"));
+         append_error_string(*str, joberrors);
+         break;
       case JS_FatalError:
-	 g_string_append_printf(*str, _("Job status: Fatal error (%d error%s)"), joberrors, (joberrors > 1) ? "s" : "");
-	 ret = error;
-	 break;
+         g_string_append_printf(*str, _("Job status: Fatal error"));
+         append_error_string(*str, joberrors);
+         ret = error;
+         break;
       case JS_Differences:
-	 g_string_append_printf(*str, _("Job status: Verify differences (%d error%s)"), joberrors, (joberrors > 1) ? "s" : "");
-	 ret = warn;
-	 break;
+         g_string_append_printf(*str, _("Job status: Verify differences"));
+         append_error_string(*str, joberrors);
+         ret = warn;
+         break;
       case JS_Canceled:
-	 g_string_append_printf(*str, _("Job status: Canceled (%d error%s)"), joberrors, (joberrors > 1) ? "s" : "");
-	 ret = warn;
-	 break;
+         g_string_append_printf(*str, _("Job status: Canceled"));
+         append_error_string(*str, joberrors);
+         ret = warn;
+         break;
       case JS_WaitFD:
-	 g_string_append_printf(*str, _("Job status: Waiting on File daemon (%d error%s)"), joberrors, (joberrors > 1) ? "s" : "");
-	 ret = warn;
-	 break;
+         g_string_append_printf(*str, _("Job status: Waiting on File daemon"));
+         append_error_string(*str, joberrors);
+         ret = warn;
+         break;
       case JS_WaitSD:
-	 g_string_append_printf(*str, _("Job status: Waiting on the Storage daemon (%d error%s)"), joberrors, (joberrors > 1) ? "s" : "");
-	 ret = warn;
-	 break;
+         g_string_append_printf(*str, _("Job status: Waiting on the Storage daemon"));
+         append_error_string(*str, joberrors);
+         ret = warn;
+         break;
       case JS_WaitMedia:
-	 g_string_append_printf(*str, _("Job status: Waiting for new media (%d error%s)"), joberrors, (joberrors > 1) ? "s" : "");
-	 ret = warn;
-	 break;
+         g_string_append_printf(*str, _("Job status: Waiting for new media"));
+         append_error_string(*str, joberrors);
+         ret = warn;
+         break;
       case JS_WaitMount:
-	 g_string_append_printf(*str, _("Job status: Waiting for Mount (%d error%s)"), joberrors, (joberrors > 1) ? "s" : "");
-	 ret = warn;
-	 break;
+         g_string_append_printf(*str, _("Job status: Waiting for Mount"));
+         append_error_string(*str, joberrors);
+         ret = warn;
+         break;
       case JS_WaitStoreRes:
-	 g_string_append_printf(*str, _("Job status: Waiting for storage resource (%d error%s)"), joberrors, (joberrors > 1) ? "s" : "");
-	 ret = warn;
-	 break;
+         g_string_append_printf(*str, _("Job status: Waiting for storage resource"));
+         append_error_string(*str, joberrors);
+         ret = warn;
+         break;
       case JS_WaitJobRes:
-	 g_string_append_printf(*str, _("Job status: Waiting for job resource (%d error%s)"), joberrors, (joberrors > 1) ? "s" : "");
-	 ret = warn;
-	 break;
+         g_string_append_printf(*str, _("Job status: Waiting for job resource"));
+         append_error_string(*str, joberrors);
+         ret = warn;
+         break;
       case JS_WaitClientRes:
-	 g_string_append_printf(*str, _("Job status: Waiting for Client resource (%d error%s)"), joberrors, (joberrors > 1) ? "s" : "");
-	 ret = warn;
-	 break;
+         g_string_append_printf(*str, _("Job status: Waiting for Client resource"));
+         append_error_string(*str, joberrors);
+         ret = warn;
+         break;
       case JS_WaitMaxJobs:
-	 g_string_append_printf(*str, _("Job status: Waiting for maximum jobs (%d error%s)"), joberrors, (joberrors > 1) ? "s" : "");
-	 ret = warn;
-	 break;
+         g_string_append_printf(*str, _("Job status: Waiting for maximum jobs"));
+         append_error_string(*str, joberrors);
+         ret = warn;
+         break;
       case JS_WaitStartTime:
-	 g_string_append_printf(*str, _("Job status: Waiting for start time (%d error%s)"), joberrors, (joberrors > 1) ? "s" : "");
-	 ret = warn;
-	 break;
+         g_string_append_printf(*str, _("Job status: Waiting for start time"));
+         append_error_string(*str, joberrors);
+         ret = warn;
+         break;
       case JS_WaitPriority:
-	 g_string_append_printf(*str, _("Job status: Waiting for higher priority jobs to finish (%d error%s)"), joberrors, (joberrors > 1) ? "s" : "");
-	 ret = warn;
-	 break;
+         g_string_append_printf(*str, _("Job status: Waiting for higher priority jobs to finish"));
+         append_error_string(*str, joberrors);
+         ret = warn;
+         break;
       default:
-	 g_warning("Unknown job status %c.", jobstatus);
-	 g_string_append_printf(*str, _("Job status: Unknown(%c) (%d error%s)"), jobstatus, joberrors, (joberrors > 1) ? "s" : "");
-	 ret = warn;
-	 break;
+         g_warning(_("Unknown job status %c."), jobstatus);
+         g_string_append_printf(*str, _("Job status: Unknown(%c)"), jobstatus);
+         append_error_string(*str, joberrors);
+         ret = warn;
+         break;
       }
    }
    else {
-      fprintf(stderr, "Bad scan : '%s' %d\n", (it == NULL) ? "" : ((GString*)it->data)->str, k);
+      fprintf(stderr, _("Bad scan : '%s' %d\n"), (it == NULL) ? "" : ((GString*)it->data)->str, k);
       ret = error;
    }
 
@@ -802,80 +854,80 @@ int docmd(monitoritem* item, const char* command, GSList** list) {
 
       switch (item->type) {
       case R_DIRECTOR:
-	 dird = (DIRRES*)item->resource;
-	 trayMessage("Connecting to Director %s:%d\n", dird->address, dird->DIRport);
-	 changeStatusMessage(item, "Connecting to Director %s:%d", dird->address, dird->DIRport);
-	 item->D_sock = bnet_connect(NULL, 0, 0, "Director daemon", dird->address, NULL, dird->DIRport, 0);
-	 jcr.dir_bsock = item->D_sock;
-	 break;
+         dird = (DIRRES*)item->resource;
+         trayMessage(_("Connecting to Director %s:%d\n"), dird->address, dird->DIRport);
+         changeStatusMessage(item, _("Connecting to Director %s:%d"), dird->address, dird->DIRport);
+         item->D_sock = bnet_connect(NULL, 0, 0, _("Director daemon"), dird->address, NULL, dird->DIRport, 0);
+         jcr.dir_bsock = item->D_sock;
+         break;
       case R_CLIENT:
-	 filed = (CLIENT*)item->resource;
-	 trayMessage("Connecting to Client %s:%d\n", filed->address, filed->FDport);
-	 changeStatusMessage(item, "Connecting to Client %s:%d", filed->address, filed->FDport);
-	 item->D_sock = bnet_connect(NULL, 0, 0, "File daemon", filed->address, NULL, filed->FDport, 0);
-	 jcr.file_bsock = item->D_sock;
-	 break;
+         filed = (CLIENT*)item->resource;
+         trayMessage(_("Connecting to Client %s:%d\n"), filed->address, filed->FDport);
+         changeStatusMessage(item, _("Connecting to Client %s:%d"), filed->address, filed->FDport);
+         item->D_sock = bnet_connect(NULL, 0, 0, _("File daemon"), filed->address, NULL, filed->FDport, 0);
+         jcr.file_bsock = item->D_sock;
+         break;
       case R_STORAGE:
-	 stored = (STORE*)item->resource;
-	 trayMessage("Connecting to Storage %s:%d\n", stored->address, stored->SDport);
-	 changeStatusMessage(item, "Connecting to Storage %s:%d", stored->address, stored->SDport);
-	 item->D_sock = bnet_connect(NULL, 0, 0, "Storage daemon", stored->address, NULL, stored->SDport, 0);
-	 jcr.store_bsock = item->D_sock;
-	 break;
+         stored = (STORE*)item->resource;
+         trayMessage(_("Connecting to Storage %s:%d\n"), stored->address, stored->SDport);
+         changeStatusMessage(item, _("Connecting to Storage %s:%d"), stored->address, stored->SDport);
+         item->D_sock = bnet_connect(NULL, 0, 0, _("Storage daemon"), stored->address, NULL, stored->SDport, 0);
+         jcr.store_bsock = item->D_sock;
+         break;
       default:
-	 printf("Error, currentitem is not a Client, a Storage or a Director..\n");
-	 gtk_main_quit();
-	 return 0;
+         printf(_("Error, currentitem is not a Client, a Storage or a Director..\n"));
+         gtk_main_quit();
+         return 0;
       }
 
       if (item->D_sock == NULL) {
-	 g_slist_append(*list, g_string_new("Cannot connect to daemon.\n"));
-	 changeStatusMessage(item, "Cannot connect to daemon.");
-	 item->state = error;
-	 item->oldstate = error;
-	 return 0;
+         g_slist_append(*list, g_string_new(_("Cannot connect to daemon.\n")));
+         changeStatusMessage(item, _("Cannot connect to daemon."));
+         item->state = error;
+         item->oldstate = error;
+         return 0;
       }
 
       if (!authenticate_daemon(item, &jcr)) {
-	 str = g_string_sized_new(64);
-	 g_string_printf(str, "ERR=%s\n", item->D_sock->msg);
-	 g_slist_append(*list, str);
-	 item->state = error;
-	 item->oldstate = error;
-	 changeStatusMessage(item, "Authentication error : %s", item->D_sock->msg);
-	 item->D_sock = NULL;
-	 return 0;
+         str = g_string_sized_new(64);
+         g_string_printf(str, "ERR=%s\n", item->D_sock->msg);
+         g_slist_append(*list, str);
+         item->state = error;
+         item->oldstate = error;
+         changeStatusMessage(item, _("Authentication error : %s"), item->D_sock->msg);
+         item->D_sock = NULL;
+         return 0;
       }
 
       switch (item->type) {
       case R_DIRECTOR:
-	 trayMessage("Opened connection with Director daemon.\n");
-	 changeStatusMessage(item, "Opened connection with Director daemon.");
-	 break;
+         trayMessage(_("Opened connection with Director daemon.\n"));
+         changeStatusMessage(item, _("Opened connection with Director daemon."));
+         break;
       case R_CLIENT:
-	 trayMessage("Opened connection with File daemon.\n");
-	 changeStatusMessage(item, "Opened connection with File daemon.");
-	 break;
+         trayMessage(_("Opened connection with File daemon.\n"));
+         changeStatusMessage(item, _("Opened connection with File daemon."));
+         break;
       case R_STORAGE:
-	 trayMessage("Opened connection with Storage daemon.\n");
-	 changeStatusMessage(item, "Opened connection with Storage daemon.");
-	 break;
+         trayMessage(_("Opened connection with Storage daemon.\n"));
+         changeStatusMessage(item, _("Opened connection with Storage daemon."));
+         break;
       default:
-	 printf("Error, currentitem is not a Client, a Storage or a Director..\n");
-	 gtk_main_quit();
-	 return 0;
-	 break;
+         printf(_("Error, currentitem is not a Client, a Storage or a Director..\n"));
+         gtk_main_quit();
+         return 0;
+         break;
       }
 
       if (item->type == R_DIRECTOR) { /* Read connection messages... */
-	 GSList *list, *it;
-	 docmd(item, "", &list); /* Usually invalid, but no matter */
-	 it = list;
-	 do {
-	    if (it->data) g_string_free((GString*)it->data, TRUE);
-	 } while ((it = it->next) != NULL);
+         GSList *list, *it;
+         docmd(item, "", &list); /* Usually invalid, but no matter */
+         it = list;
+         do {
+            if (it->data) g_string_free((GString*)it->data, TRUE);
+         } while ((it = it->next) != NULL);
 
-	 g_slist_free(list);
+         g_slist_free(list);
       }
    }
 
@@ -884,46 +936,46 @@ int docmd(monitoritem* item, const char* command, GSList** list) {
 
    while(1) {
       if ((stat = bnet_recv(item->D_sock)) >= 0) {
-	 g_slist_append(*list, g_string_new(item->D_sock->msg));
+         g_slist_append(*list, g_string_new(item->D_sock->msg));
       }
       else if (stat == BNET_SIGNAL) {
-	 if (item->D_sock->msglen == BNET_EOD) {
-	    //fprintf(stderr, "<< EOD >>\n");
-	    return 1;
-	 }
-	 else if (item->D_sock->msglen == BNET_PROMPT) {
-	    //fprintf(stderr, "<< PROMPT >>\n");
-	    g_slist_append(*list, g_string_new("<< Error: BNET_PROMPT signal received. >>\n"));
-	    return 0;
-	 }
-	 else if (item->D_sock->msglen == BNET_HEARTBEAT) {
-	    bnet_sig(item->D_sock, BNET_HB_RESPONSE);
-	    g_slist_append(*list, g_string_new("<< Heartbeat signal received, answered. >>\n"));
-	 }
-	 else {
-	    str = g_string_sized_new(64);
-	    g_string_printf(str, "<< Unexpected signal received : %s >>\n", bnet_sig_to_ascii(item->D_sock));
-	    g_slist_append(*list, str);
-	 }
+         if (item->D_sock->msglen == BNET_EOD) {
+            //fprintf(stderr, "<< EOD >>\n");
+            return 1;
+         }
+         else if (item->D_sock->msglen == BNET_PROMPT) {
+            //fprintf(stderr, "<< PROMPT >>\n");
+            g_slist_append(*list, g_string_new(_("<< Error: BNET_PROMPT signal received. >>\n")));
+            return 0;
+         }
+         else if (item->D_sock->msglen == BNET_HEARTBEAT) {
+            bnet_sig(item->D_sock, BNET_HB_RESPONSE);
+            g_slist_append(*list, g_string_new(_("<< Heartbeat signal received, answered. >>\n")));
+         }
+         else {
+            str = g_string_sized_new(64);
+            g_string_printf(str, _("<< Unexpected signal received : %s >>\n"), bnet_sig_to_ascii(item->D_sock));
+            g_slist_append(*list, str);
+         }
       }
       else { /* BNET_HARDEOF || BNET_ERROR */
-	 g_slist_append(*list, g_string_new("<ERROR>\n"));
-	 item->D_sock = NULL;
-	 item->state = error;
-	 item->oldstate = error;
-	 changeStatusMessage(item, "Error : BNET_HARDEOF or BNET_ERROR");
-	 //fprintf(stderr, "<< ERROR >>\n");
-	 return 0;
+         g_slist_append(*list, g_string_new(_("<ERROR>\n")));
+         item->D_sock = NULL;
+         item->state = error;
+         item->oldstate = error;
+         changeStatusMessage(item, _("Error : BNET_HARDEOF or BNET_ERROR"));
+         //fprintf(stderr, _("<< ERROR >>\n"));
+         return 0;
       }
 
       if (is_bnet_stop(item->D_sock)) {
-	 g_string_append_printf(str, "<STOP>\n");
-	 item->D_sock = NULL;
-	 item->state = error;
-	 item->oldstate = error;
-	 changeStatusMessage(item, "Error : Connection closed.");
-	 //fprintf(stderr, "<< STOP >>\n");
-	 return 0;            /* error or term */
+         g_string_append_printf(str, _("<STOP>\n"));
+         item->D_sock = NULL;
+         item->state = error;
+         item->oldstate = error;
+         changeStatusMessage(item, _("Error : Connection closed."));
+         //fprintf(stderr, "<< STOP >>\n");
+         return 0;            /* error or term */
       }
    }
 }
@@ -966,41 +1018,41 @@ void updateStatusIcon(monitoritem* item) {
 
    if (item == NULL) {
       /* For the current status, select the two worse for blinking,
-	 but never blink a D_Sock == NULL error with idle. */
+         but never blink a D_Sock == NULL error with idle. */
       stateenum state1, state2, oldstate;
       gboolean onenull = FALSE;
       state1 = idle;
       state2 = idle;
       oldstate = idle;
       for (int i = 0; i < nitems; i++) {
-	 if (items[i].D_sock == NULL) onenull = TRUE;
-	 if (items[i].state >= state1) {
-	    state2 = state1;
-	    state1 = items[i].state;
-	 }
-	 else if (items[i].state > state2) {
-	    state2 = items[i].state;
-	 }
-	 if (items[i].oldstate > oldstate) oldstate = items[i].oldstate;
+         if (items[i].D_sock == NULL) onenull = TRUE;
+         if (items[i].state >= state1) {
+            state2 = state1;
+            state1 = items[i].state;
+         }
+         else if (items[i].state > state2) {
+            state2 = items[i].state;
+         }
+         if (items[i].oldstate > oldstate) oldstate = items[i].oldstate;
       }
 
       if ((onenull == TRUE) && (state2 == idle)) {
-	 state2 = error;
+         state2 = error;
       }
 
       xpm = generateXPM(blinkstate ? state1 : state2, oldstate);
    }
    else {
       if ((blinkstate) && (item->D_sock != NULL)) {
-	 if (item->state > 1) { //Warning or error while running
-	    xpm = generateXPM(running, item->oldstate);
-	 }
-	 else {
-	    xpm = generateXPM(idle, item->oldstate);
-	 }
+         if (item->state > 1) { //Warning or error while running
+            xpm = generateXPM(running, item->oldstate);
+         }
+         else {
+            xpm = generateXPM(idle, item->oldstate);
+         }
       }
       else {
-	 xpm = generateXPM(item->state, item->oldstate);
+         xpm = generateXPM(item->state, item->oldstate);
       }
    }
 
