@@ -4,7 +4,7 @@
 // Copyright transferred from Raider Solutions, Inc to
 //   Kern Sibbald and John Walker by express permission.
 //
-// Copyright (C) 2004 Kern Sibbald and John Walker
+// Copyright (C) 2004-2005 Kern Sibbald
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -24,21 +24,25 @@
  *
  * Author          : Christopher S. Hull
  * Created On      : Fri Jan 30 13:00:51 2004
- * Last Modified By: Nicolas Boichat
- * Last Modified On: Sat Apr 17 19:30:00 2004
+ * Last Modified By: Thorsten Engel
+ * Last Modified On: Fri Apr 22 19:30:00 2004
  * Update Count    : 218
- * $Id: compat.h,v 1.13 2004/06/27 15:22:55 kerns Exp $
+ * $Id: compat.h,v 1.24.2.1 2005/10/01 10:20:18 kerns Exp $
  */
 
 
 #ifndef __COMPAT_H_
 #define __COMPAT_H_
 
-#ifndef HAVE_MINGW
-#ifndef HAVE_WXCONSOLE
+#if (defined _MSC_VER) && (_MSC_VER >= 1400) // VC8+
+#pragma warning(disable : 4996) // Either disable all deprecation warnings,
+// #define _CRT_SECURE_NO_DEPRECATE // Or just turn off warnings about the newly deprecated CRT functions.
+#define HAVE_VC8
+#endif // VC8+
+
+#if (!defined HAVE_MINGW) && (!defined HAVE_VC8) && (!defined HAVE_WXCONSOLE)
 #define __STDC__ 1
 #endif
-#endif
 
 #include <stdio.h>
 #include <basetsd.h>
@@ -77,7 +81,7 @@
 
 #ifndef HAVE_MINGW
 #ifdef HAVE_CYGWIN
-#error should not be used under cygwin...
+#undef HAVE_CYGWIN
 #else
 #endif //HAVE_CYGWIN
 #endif //HAVE_MINGW
@@ -92,9 +96,12 @@ typedef unsigned char uint8_t;
 typedef float float32_t;
 typedef unsigned short uint16_t;
 typedef signed short int16_t;
-typedef long time_t;
 typedef signed char int8_t;
 
+#ifndef HAVE_VC8
+typedef long time_t;
+#endif
+
 #if __STDC__
 #ifndef HAVE_MINGW
 typedef _dev_t dev_t;
@@ -136,7 +143,9 @@ typedef UINT32 gid_t;
 typedef UINT32 uid_t;
 typedef UINT32 gid_t;
 typedef UINT32 mode_t;
+/* #ifndef _WX_DEFS_H_  ssize_t is defined in wx/defs.h */
 typedef INT64  ssize_t;
+/* #endif */
 #endif //HAVE_MINGW
 
 struct dirent {
@@ -168,6 +177,8 @@ int gettimeofday(struct timeval *, struct timezone *);
 #define ETIMEDOUT 55
 
 #ifndef HAVE_MINGW
+
+#ifndef _STAT_DEFINED
 struct stat
 {
     _dev_t      st_dev;
@@ -184,12 +195,14 @@ struct stat
     uint32_t    st_blksize;
     uint64_t    st_blocks;
 };
+#endif
 
 #undef  S_IFMT
 #define S_IFMT         0170000         /* file type mask */
 #undef  S_IFDIR
 #define S_IFDIR        0040000         /* directory */
 #define S_IFCHR        0020000         /* character special */
+#define S_IFBLK        0060000         /* block special */
 #define S_IFIFO        0010000         /* pipe */
 #undef  S_IFREG
 #define S_IFREG        0100000         /* regular */
@@ -200,10 +213,10 @@ struct stat
 #define S_IRUSR         S_IREAD
 #define S_IWUSR         S_IWRITE
 #define S_IXUSR         S_IEXEC
-#define S_ISREG(x)  (((x) & S_IFREG) == S_IFREG)
-#define S_ISDIR(x)  (((x) & S_IFDIR) == S_IFDIR)
+#define S_ISREG(x)  (((x) & S_IFMT) == S_IFREG)
+#define S_ISDIR(x)  (((x) & S_IFMT) == S_IFDIR)
 #define S_ISCHR(x) 0
-#define S_ISBLK(x) 0
+#define S_ISBLK(x)  (((x) & S_IFMT) == S_IFBLK)
 #define S_ISFIFO(x) 0
 #endif //HAVE_MINGW
 
@@ -236,17 +249,23 @@ struct stat
 #define iscsym  __iscsym
 #endif
 
-
+#ifndef HAVE_VC8
 int umask(int);
+off_t lseek(int, off_t, int);
+int dup2(int, int);
+int close(int fd);
+#ifndef HAVE_WXCONSOLE
+ssize_t read(int fd, void *, ssize_t nbytes);
+ssize_t write(int fd, const void *, ssize_t nbytes);
+#endif
+#endif
 int lchown(const char *, uid_t uid, gid_t gid);
 int chown(const char *, uid_t uid, gid_t gid);
 int chmod(const char *, mode_t mode);
-off_t lseek(int, off_t, int);
 int inet_aton(const char *cp, struct in_addr *inp);
 int kill(int pid, int signo);
 int pipe(int []);
 int fork();
-int dup2(int, int);
 int waitpid(int, int *, int);
 
 #ifndef HAVE_MINGW
@@ -264,12 +283,6 @@ int __snprintf(char *str, size_t count, const char *fmt, ...);
 #define sprintf __sprintf
 int __sprintf(char *str, const char *fmt, ...);
 
-#ifndef HAVE_WXCONSOLE
-ssize_t read(int fd, void *, ssize_t nbytes);
-ssize_t write(int fd, const void *, ssize_t nbytes);
-#endif
-
-int close(int fd);
 #endif //HAVE_MINGW
 
 
@@ -324,7 +337,8 @@ struct sigaction {
 #define sigfillset(x)
 #define sigaction(a, b, c)
 
-#define mkdir(p, m) _mkdir(p)
+#define mkdir(p, m) win32_mkdir(p)
+#define unlink win32_unlink
 #define chdir win32_chdir
 int syslog(int, const char *, const char *);
 #define LOG_DAEMON 0
@@ -349,8 +363,15 @@ extern "C" void *  __cdecl _alloca(size_t);
 
 #define getcwd win32_getcwd
 #define chdir win32_chdir
+#define fputs win32_fputs
 char *win32_getcwd(char *buf, int maxlen);
 int win32_chdir(const char *buf);
+int win32_mkdir(const char *buf);
+int win32_fputs(const char *string, FILE *stream);
+int win32_unlink(const char *filename);
+
+char* win32_cgets (char* buffer, int len);
+
 
 int WSA_Init(void);
 
@@ -358,4 +379,8 @@ int WSA_Init(void);
 void closelog();
 #endif //HAVE_MINGW
 
+#ifndef INVALID_FILE_ATTRIBUTES
+#define INVALID_FILE_ATTRIBUTES ((DWORD)-1)
+#endif
+
 #endif /* __COMPAT_H_ */
