@@ -11,24 +11,21 @@
  *
  *    Nicolas Boichat, April 2004
  *
- *    Version $Id: wxbtableparser.cpp,v 1.6 2004/08/06 15:14:36 nboichat Exp $
+ *    Version $Id: wxbtableparser.cpp,v 1.9 2005/07/08 10:03:27 kerns Exp $
  */
 /*
-   Copyright (C) 2004 Kern Sibbald and John Walker
+   Copyright (C) 2004-2005 Kern Sibbald
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
-   as published by the Free Software Foundation; either version 2
-   of the License, or (at your option) any later version.
+   version 2 as amended with additional clauses defined in the
+   file LICENSE in the main source directory.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+   the file LICENSE for additional details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 #include "wxbtableparser.h" // class's header file
@@ -91,7 +88,7 @@ bool wxbTableParser::Analyse(wxString str, int status) {
 
       if ((str.GetChar(0) == '|') && (str.GetChar(str.Length()-2) == '|') && (str.GetChar(str.Length()-1) == '\n')) {
          str.RemoveLast();
-         wxStringTokenizer tkz(str, "|", wxTOKEN_STRTOK);
+         wxStringTokenizer tkz(str, wxT("|"), wxTOKEN_STRTOK);
 
          if (separatorNum == 1) {
             while ( tkz.HasMoreTokens() ) {
